<?php
//module/Secure/src/Secure/Controller/ForgetpassController.php

/**************
	Purpose: Implements the forget password functionality for HTC.
	Author: Kevin Gyberg
	Date: 02/20/2013
*************/

namespace Login\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\Validator\Db\RecordExists;
use Login\Form\ForgetpassForm;
use Login\Model\Forgetpass;
use Zend\Mail;  
use Zend\Mime\Part as MimePart;  
use Zend\Mime\Message as MimeMessage; 

class ForgetpassController extends AbstractActionController
{
	/**
     * @var authservice
     */
	protected $authservice;
    
	/**
     * Gets an instance of authentication service.
     *
     * @return instance of authservice
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()
                                      ->get('AuthService');
        }
        
        return $this->authservice;
    }
    
    /**
     * Renders the forget password form and resets the password for entered email id.
     *
     * @return array(Forget password form object).
     */
	function indexAction()
    {   
		// get an instance of register form
		$form = new ForgetpassForm();
		$isLoggedIn = 'no';
		
		// check if the user is already logged in.
		if ($this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'yes';
			return $this->redirect()->toRoute('secure');
		}
		
		$this->getEvent()->getViewModel()->setVariables(
                array(
                    'is_logged_in'           => $isLoggedIn,
                    'noBetaFeedbackLink'     => TRUE
                )
        );
		
		// check if the request is a post request
        if ($this->getRequest()->isPost()) {
			$isAlreadyPresent = false;
			$user = new Forgetpass();
            $form->setInputFilter($user->getInputFilter());
            $form->setData($this->getRequest()->getPost());
			
			// apply validations on the submitted form
            if ($form->isValid()) {
				
				$emailAddress = strtolower($this->getRequest()->getPost()->email_address);
                $emailExists = $this->getUsersTable()->ifEmailIdExists($emailAddress);

				if ($emailExists) {
					$isAlreadyPresent = true;
					$greetingSrv = $this->getServiceLocator()->get('PasswordGenerator');
					// generate a random password
					$randomPassword = $greetingSrv->createPassword();
					$encryptedPassword = md5($randomPassword);
					
					// reset the database
					$userData = $this
									->getUsersTable()
									->resetPassword($emailAddress, $encryptedPassword);
					
					// if the password is updated properly
					if ($userData) {
						$config = $this->getServiceLocator()->get('Config');
					
						// send and email to user.
						$options = new Mail\Transport\SmtpOptions($config['mailer']['smtp_options']);
						$mailData = array(
							'firstname'		=>		$userData->first_name,
							'lastname'		=>		$userData->last_name,
							'email_address' => 		$emailAddress,
							'password' 		=> 		$randomPassword
						);
						$this->renderer = $this->getServiceLocator()->get('ViewRenderer');  
						$content = $this->renderer->render('email/view', $mailData);  
						  
						// make a header as html  
						$html = new MimePart($content);  
						$html->type = "text/html";  
						$body = new MimeMessage();  
						$body->setParts(array($html,)); 
						  
						// instance mail   
						$mail = new Mail\Message();  
						$mail->setBody($body); // will generate our code html from template.phtml  
						$mail->setFrom($config['mailer']['password_reset_mail_constants']['from_email'], $config['mailer']['password_reset_mail_constants']['from_name']);  
						$mail->setTo($emailAddress);
						$mail->setSubject($config['mailer']['password_reset_mail_constants']['subject']);
                        
						$transport = new Mail\Transport\Smtp($options);
						$transport->send($mail);
                        return array(
							'form' => $form,
							'is_email_id_present' => $isAlreadyPresent
						);
					}
					else {
						// unable to reset the password
						// send validation message unable to reset the password
						return array(
							'form' => $form,
							'message' => 'Unable to reset password.',
							'is_email_id_present' => $isAlreadyPresent
						);
					}
				}
				else {
					// email id does not exist.
					return array(
						'form' => $form, 
						'is_email_id_present' => $isAlreadyPresent
					);
				}
			}
		}
		return array('form' => $form);
	}
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
}