<?php
//module/Secure/src/Secure/Controller/RegisterController.php

/**************
	Purpose: Handels the registration of users into HTC.
	Author: Kevin Gyberg
	Date: 02/26/2013
*************/

namespace Login\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;	
use Login\Model\UsersTable;
use Login\Form\RegisterForm;
use Zend\Session\Container;
use Zend\Filter\StripTags;
use Zend\Filter\StringTrim;
use Zend\Validator\EmailAddress;
use Zend\Validator\Db\RecordExists;
use Zend\Mail;  
use Zend\Mime\Part as MimePart;  
use Zend\Mime\Message as MimeMessage; 
	
class RegisterController extends AbstractActionController
{
	/**
     * @var authservice
     */
	protected $authservice;
	
	/**
     * @var authservice
     */
	protected $usersTable;
    
	/**
     * Gets an instance of authentication service.
     *
     * @return instance of authservice
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()->get('AuthService');
        }
        
        return $this->authservice;
    }
	
	/**
     * Renders the register form and processes the posted data
     *
     * @return array(Registration for instance).
     */
	public function indexAction()
	{
		$isLoggedIn = 'no';
		
		// check if the user is already logged in.
		if ($this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'yes';
			return $this->redirect()->toRoute('secure');
		}
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'is_logged_in'           => $isLoggedIn,
                    'noBetaFeedbackLink'     => TRUE
                )
        );
        
        $activationIdString = $this->params()->fromRoute('activationid', 0);
        
        $decodedActivationString = '';
        if ($activationIdString) {

           $convert = $this->getServiceLocator()->get('ConvertParams');
           $decodedString = $convert->paramDecrypt($activationIdString);
           $decodedArray = $convert->stringToArray($decodedString);
           
           $decodedActivationString = $decodedArray['emailId'];
           $emailExists = $this->getUsersTable()->ifEmailIdExists($decodedActivationString);
           if ($emailExists) {
                $this->flashmessenger()->addMessage("Your login has been validated.");
                return $this->redirect()->toRoute('login');
           }
           
           $emailExistsInactive = $this
                                    ->getUsersTable()
                                    ->ifEmailIdExistsInactive($decodedActivationString);
           
           if ($emailExistsInactive) {
               $this->flashmessenger()->addMessage("Your email address is 
                            registered in Help To Care but not yet activated.");
               return $this->redirect()->toRoute('login');
           }
        }
		
		// get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
		
			// collect the posted data
			$postedData = $this->getRequest()->getPost();
			
			$errorMessages['messages'] = array();
			$isFormValidated = true;
			$userInsertdata = array();
			
			$userInsertdata['loginEmail'] = $this->filterFormElements($postedData['email_login']);
			
			if (!$userInsertdata['loginEmail']) {
				$isFormValidated = false;
				$errorMessages['messages']['loginEmail']['required'] = 'Please enter your E-mail address.';
			}
			
			$emailvalidator = new EmailAddress();
			if ($userInsertdata['loginEmail'] && !$emailvalidator->isValid($userInsertdata['loginEmail'])) {
				$isFormValidated = false;
				$errorMessages['messages']['loginEmail']['invalid'] = 'Please enter a valid E-mail address.';
			}
			else {
                // if the entered email address is valid
                $userInsertdata['loginEmail'] = strtolower($userInsertdata['loginEmail']);
                
                $emailExists = $this->getUsersTable()->ifEmailIdExists($userInsertdata['loginEmail']);

                // if email id already exists
                if ($emailExists) {
                    $isFormValidated = false;
                    $errorMessages['messages']['loginEmail']['exists'] = 'The entered E-mail address already exists.';
                }
            }
			
			$userInsertdata['password'] = $this->filterFormElements($postedData['password']);
            
            $config = $this->getServiceLocator()->get('Config');
            
			if (!$userInsertdata['password']) {
				$isFormValidated = false;
				$errorMessages['messages']['password']['required'] = 'Please enter Password.';
			}
            else if (strlen($userInsertdata['password']) < $config['minimum_password_char']) {
                $isFormValidated = false;
				$errorMessages['messages']['password']['invalidLength'] = 'Please enter Password minimum of 6 characters.';
            }
			
			$userInsertdata['confPassword'] = $this->filterFormElements($postedData["conf_password"]);
			if (!$userInsertdata['confPassword']) {
				$isFormValidated = false;
				$errorMessages['messages']['confPassword']['required'] = 'Please enter Confirm Password.';
			}
			
			if ($userInsertdata['password'] 
                    && $userInsertdata['confPassword'] 
                    && !$this->validateConfirmPassword($userInsertdata['password'], $userInsertdata['confPassword'])) {
				$isFormValidated = false;
				$errorMessages['messages']['confPassword']['identical'] = 'Password and Confirm Password do not match.';
			}
			
			$userInsertdata['firstName'] = $this->filterFormElements($postedData['first_name']);
			if (!$userInsertdata['firstName']) {
				$isFormValidated = false;
				$errorMessages['messages']['firstName']['required'] = 'Please enter First Name.';
			}
			
			$userInsertdata['lastName'] = $this->filterFormElements($postedData['last_name']);
			if (!$userInsertdata['lastName']) {
				$isFormValidated = false;
				$errorMessages['messages']['lastName']['required'] = 'Please enter Last Name.';
			}
			
			$userInsertdata['termsAgree'] = $this->filterFormElements($postedData['terms_conditions_agree']);
			if (!$userInsertdata['termsAgree']) {
				$isFormValidated = false;
				$errorMessages['messages']['termsAgree']['required'] = 'Please agree to the Terms of Service and Privacy Policy.';
			}
			
			// apply validations on the submitted form
            if ($isFormValidated) {
                
                // create a random code for 
                $activationCode = sha1($userInsertdata['loginEmail'] . time() . rand());
                
                // Also send the activation code to the users model in order to 
                // store it.
                $userInsertdata['activationCode'] = $activationCode;
				
				// save the user data to the users table.
                if (isset($postedData['email_address']) && $postedData['email_address'] != '') {
                    $lastUserIdInsert = $this->getUsersTable()->saveUser($userInsertdata, 'yes');
                }
                else {
                    $lastUserIdInsert = $this->getUsersTable()->saveUser($userInsertdata);
                }
                
                if ($lastUserIdInsert) {
                    
                    // we need to send an email if and only if 
                    if (!$postedData['email_address']) {
                        
                        // send an email with the generated activation code to the user's email id.
                        $convert = $this->getServiceLocator()->get('SendEmail');
                        $config = $this->getServiceLocator()->get('Config');

                        $activationLink = $config['activation_email_link'] . $activationCode;
                        // prepare the view for activation email template
                        $mailData = array(
                           'activation_link'    =>      $activationLink,
                           'email_address'      => 		$userInsertdata['loginEmail'],
                           'password'           => 		$userInsertdata['password']
                        );
                        $this->renderer = $this->getServiceLocator()->get('ViewRenderer');  
                        $content = $this->renderer->render('activation-email/view', $mailData);

                        // create the link for activation code.
                        $smtpOptions = $config['mailer']['smtp_options'];
                        $htmlBody = $content;
                        $subject = $config['mailer']['registration_mail_constants']['registration_email_subject'];
                        $fromName = $config['mailer']['registration_mail_constants']['registration_email_from_name'];
                        $fromEmail = $config['mailer']['registration_mail_constants']['registration_email_from_email'];
                        $to = $userInsertdata['loginEmail'];

                        // $htmlBody, $subject, $fromName, $fromEmail, $to
                        $convert->sendEmail($smtpOptions, $htmlBody, $subject, $fromName, $fromEmail, $to);
                        return $this->redirect()->toRoute(
                                                'register', 
                                                array(
                                                    'controller' => 'register', 
                                                    'action' => 'goto-login'
                                                )
                        );
                    }
                    $this
                        ->getUsersPermTempTable()
                        ->copyPermissions($userInsertdata['loginEmail'], $lastUserIdInsert);
                    
					return $this->redirect()->toRoute('login');
				}
				// Unable to register user.
				$errorMessages['messages']['registerStatus']['error'] = 'Unable to register user.';
            }
			
            if (isset($postedData['email_address']) && $postedData['email_address'] != '') {
                return array(
                    'messages'      => $errorMessages['messages'],
                    'setdata'       => $userInsertdata,
                    'email_login'   => $postedData['email_address']
                );
            }
            else {
                return array(
                    'messages'      => $errorMessages['messages'],
                    'setdata'       => $userInsertdata
                );
            }
        }
        
        if ($activationIdString) {
            return array(
				'email_login'  => $decodedActivationString
			);
        }
	}
    
    /**
     * Provide users the link for redirecting towards the login page.
     * 
     * @return NULL
     */
    public function gotoLoginAction() {
        
        $isLoggedIn = 'no';
		
		// check if the user is already logged in.
		if ($this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'yes';
			return $this->redirect()->toRoute('secure');
		}
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'is_logged_in'           => $isLoggedIn,
                    'noBetaFeedbackLink'     => TRUE
                )
        );
        return array();
    }

    /**
     * Renders the register form and processes the posted data
     *
     * @return array(Registration for instance).
     */
	public function activateAction()
	{   
        $activationCode = $this->params()->fromRoute('activationid', 0);
        
        if ($activationCode) {
            
            if ($this->getUsersTable()->activateUser($activationCode)) {
                
                // if the user is successfully activated then collect his/her user id and email id.
                
                $userRecord = $this->getUsersTable()->getUserDetailByActivationCode($activationCode);
                if ($userRecord) {
                    $this
                        ->getUsersPermTempTable()
                        ->copyPermissions($userRecord['email_login'], $userRecord['userid']);
                }
                
                $this
                    ->flashmessenger()
                    ->addMessage("Thank you for your interest in Help To Care!  Your account has been successfully activated. Please enter the e-mail address and password you selected when you registered.");
            }
            else {
                
                $this->flashmessenger()->addMessage("Your login has been validated.");
            }
            return $this->redirect()->toRoute('login');
        }
        // show error message or simply redirect the user to login page
    }
    
	
	/**
     * Apply strip tags to the provided input
     * 
     * @param String $valueToFilter
     * @return Filtered string on Success and false on failure.
     */
	public function filterFormElements($valueToFilter = '') {
        
		// instantiate the strip tag filter
		$stripTagFilter   = new StripTags();
		$StringTrimFilter = new StringTrim();
		if ($valueToFilter) {
			return $stripTagFilter->filter($StringTrimFilter->filter($valueToFilter));
		}
		return false;
	}
	
    /**
     * Check whether both the entered password and confirm password are matched.
     * 
     * @param String $password
     * @param String $confirmPassword
     * @return boolean true on success and false on failure.
     */
	public function validateConfirmPassword($password = '', $confirmPassword = '')
	{
		if ($password && $confirmPassword) {
			if ($password === $confirmPassword) {
				return true;
			}
		}
		return false;
	}
    
    /**
     * Function to render terms of service without any layout.
     * 
     */
    public function termsOfServiceAction()
    {
        $result = new ViewModel();
        $result->setTerminal(true);
        return $result;
    }
    
    /**
     * Function to render terms of service without any layout.
     * 
     */
    public function privacyPolicyAction()
    {
        $result = new ViewModel();
        $result->setTerminal(true);
        return $result;
    }


    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersPermTempTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersPermTempTable) {
			$sm = $this->getServiceLocator();
			$this->usersPermTempTable = $sm->get('Secure\Model\UsersPermTempTable');
		}
		return $this->usersPermTempTable;
	}
}