<?php
//module/Secure/Module.php

namespace Secure;

use Zend\ModuleManager\Feature\AutoloaderProviderInterface;
use Zend\Mvc\ModuleRouteListener;

use Zend\Authentication\AuthenticationService;
use Zend\Authentication\Adapter\DbTable as DbTableAuthAdapter;
use Secure\Model\Users;
use Secure\Model\UsersTable;
use Secure\Model\EventsTable;
use Secure\Model\CalendarTable;
use Secure\Model\PersonTable;
use Secure\Model\ProviderTable;
use Secure\Model\MedicationTable;
use Secure\Model\MedicationsTable;
use Secure\Model\TasksTable;
use Secure\Model\TaskTable;
use Secure\Model\ContactTable;
use Secure\Model\ContactsTable;
use Secure\Model\DocumentTable;
use Secure\Model\ResourceTypeTable;
use Secure\Model\DocumentsTable;
use Secure\Model\PostTable;
use Secure\Model\ResourceTable;
use Secure\Model\BetaFeedbackInfoTable;
use Secure\Model\Events;
use Secure\Model\UsersPermTempTable;
use Secure\Model\UserResourceTable;
use Zend\Db\ResultSet\ResultSet;
use Zend\Db\TableGateway\TableGateway;
use Zend\ModuleManager\Feature\ConfigProviderInterface;
use Zend\ModuleManager\Feature\BootstrapListenerInterface;
use Zend\Mvc\MvcEvent;

class Module implements AutoloaderProviderInterface,
                        ConfigProviderInterface
{
    public function getAutoloaderConfig()
    {
        return array(
            'Zend\Loader\ClassMapAutoloader' => array(
                __DIR__ . '/autoload_classmap.php',
            ),
            'Zend\Loader\StandardAutoloader' => array(
                'namespaces' => array(
		    // if we're in a namespace deeper than one level we need to fix the \ in the path
                    __NAMESPACE__ => __DIR__ . '/src/' . str_replace('\\', '/' , __NAMESPACE__),
                ),
            ),
        );
    }
    
    public function onBootstrap(MvcEvent $e)
    {
        $application   = $e->getApplication();
        $sm = $application->getServiceManager();
        $sharedManager = $application->getEventManager()->getSharedManager();

        $sharedManager->attach('Zend\Mvc\Application', 'dispatch.error',
                 function($e) use ($sm) {
                    if ($e->getParam('exception')) {
                        $sm->get('\Zend\Log\Logger')->crit($e->getParam('exception'));
                    }
                 }
        );
    }

    public function getConfig()
    {
        return include __DIR__ . '/config/module.config.php';
    }
    
    public function getServiceConfig()
    {
        return array(
			'invokables' => array(
				'PasswordGenerator' => 'Secure\Service\PasswordGeneratorService',
				'ConvertParams'     => 'Secure\Service\ConvertService',
				'SendEmail'         => 'Secure\Service\SendEmailService'
			),
            'factories'=>array(
                'Zend\Log\Logger' => function($sm){
                    $logger = new \Zend\Log\Logger;
                    $writer = new \Zend\Log\Writer\Stream('./data/log/' . date('Y-m-d') . '-error.log');

                    $logger->addWriter($writer); 

                    return $logger;
                },
				'Secure\Model\MyAuthStorage' => function($sm){
					return new \Secure\Model\MyAuthStorage('help_to_care');  
				},
				'Secure\Model\UsersTable' =>  function($sm) {
					$tableGateway = $sm->get('UsersTableGateway');
					$table = new UsersTable($tableGateway);
					return $table;
				},
				'UsersTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('users', $dbAdapter);
				},				
				'Secure\Model\PersonTable' =>  function($sm) {
					$tableGateway = $sm->get('PersonTableGateway');
					$table = new PersonTable($tableGateway);
					return $table;
				},
				'PersonTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('person', $dbAdapter);
				},
				'Secure\Model\CalendarTable' =>  function($sm) {
					$tableGateway = $sm->get('CalendarTableGateway');
					$table = new CalendarTable($tableGateway);
					return $table;
				},
				'CalendarTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('calendar', $dbAdapter);
				},
				'Secure\Model\UsersPermTempTable' =>  function($sm) {
					$tableGateway = $sm->get('UsersPermTempTableGateway');
					$table = new UsersPermTempTable($tableGateway);
					return $table;
				},
				'UsersPermTempTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('users_perm_temp', $dbAdapter);
				},
				'Secure\Model\ProviderTable' =>  function($sm) {
					$tableGateway = $sm->get('ProviderTableGateway');
					$table = new ProviderTable($tableGateway);
					return $table;
				},
				'ProviderTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('provider', $dbAdapter);
				},
				'Secure\Model\ResourceTable' =>  function($sm) {
					$tableGateway = $sm->get('ResourceTableGateway');
					$table = new ResourceTable($tableGateway);
					return $table;
				},
				'ResourceTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('resource', $dbAdapter);
				},
				'Secure\Model\MedicationTable' =>  function($sm) {
					$tableGateway = $sm->get('MedicationTableGateway');
					$table = new MedicationTable($tableGateway);
					return $table;
				},
				'MedicationTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('medication', $dbAdapter);
				},
				'Secure\Model\UserResourceTable' =>  function($sm) {
					$tableGateway = $sm->get('UserResourceTableGateway');
					$table = new UserResourceTable($tableGateway);
					return $table;
				},
				'UserResourceTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('user_resource', $dbAdapter);
				},
				'Secure\Model\MedicationsTable' =>  function($sm) {
					$tableGateway = $sm->get('MedicationsTableGateway');
					$table = new MedicationsTable($tableGateway);
					return $table;
				},
				'ResourceTypeTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('resource_type', $dbAdapter);
				},
				'Secure\Model\ResourceTypeTable' =>  function($sm) {
					$tableGateway = $sm->get('ResourceTypeTableGateway');
					$table = new ResourceTypeTable($tableGateway);
					return $table;
				},
				'MedicationsTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('medication_group', $dbAdapter);
				},
				'Secure\Model\ContactsTable' =>  function($sm) {
					$tableGateway = $sm->get('ContactsTableGateway');
					$table = new ContactsTable($tableGateway);
					return $table;
				},
				'ContactsTableGateway' => function ($sm) {		
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('contact_group', $dbAdapter);
				},
				'Secure\Model\TasksTable' =>  function($sm) {
					$tableGateway = $sm->get('TasksTableGateway');
					$table = new TasksTable($tableGateway);
					return $table;
				},
				'TasksTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('task_group', $dbAdapter);
				},
                'Secure\Model\TaskTable' =>  function($sm) {
					$tableGateway = $sm->get('TaskTableGateway');
					$table = new TaskTable($tableGateway);
					return $table;
				},
				'TaskTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('task', $dbAdapter);
				},
				'Secure\Model\BetaFeedbackInfoTable' =>  function($sm) {
					$tableGateway = $sm->get('BetaFeedbackInfoTableGateway');
					$table = new BetaFeedbackInfoTable($tableGateway);
					return $table;
				},
				'BetaFeedbackInfoTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('beta_feedback_info', $dbAdapter);
				},
				'Secure\Model\ContactTable' =>  function($sm) {
					$tableGateway = $sm->get('ContactTableGateway');
					$table = new ContactTable($tableGateway);
					return $table;
				},
				'ContactTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('contact', $dbAdapter);
				},
				'Secure\Model\EventsTable' =>  function($sm) {
					$tableGateway = $sm->get('EventsTableGateway');
					$table = new EventsTable($tableGateway);
					return $table;
				},
				'EventsTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					$resultSetPrototype = new ResultSet();
					$resultSetPrototype->setArrayObjectPrototype(new Events());
					return new TableGateway('events', $dbAdapter, null, $resultSetPrototype);
				},
				//Functionality for documents table
				'Secure\Model\DocumentsTable' =>  function($sm) {
					$tableGateway = $sm->get('DocumentsTableGateway');
					$table = new DocumentsTable($tableGateway);
					return $table;
				},
				'DocumentsTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('document_group', $dbAdapter);
				},
				//functionality for document table
				'Secure\Model\DocumentTable' =>  function($sm) {
					$tableGateway = $sm->get('DocumentTableGateway');
					$table = new DocumentTable($tableGateway);
					return $table;
				},
				'DocumentTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('document', $dbAdapter);
				},
				'Secure\Model\PostTable' =>  function($sm) {
					$tableGateway = $sm->get('PostTableGateway');
					$table = new PostTable($tableGateway);
					return $table;
				},
				'PostTableGateway' => function ($sm) {
					$dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
					return new TableGateway('post', $dbAdapter);
				},
				'AuthService' => function($sm) {
					$dbAdapter      = $sm->get('Zend\Db\Adapter\Adapter');
					$dbTableAuthAdapter  = new DbTableAuthAdapter($dbAdapter, 'users','email_login','password', 'MD5(?)');
                    $select = $dbTableAuthAdapter->getDbSelect();
                    $select->where("rec_status = '1'");
					
					$authService = new AuthenticationService();
					$authService->setAdapter($dbTableAuthAdapter);
					$authService->setStorage($sm->get('Secure\Model\MyAuthStorage'));
					 
					return $authService;
				},
            ),
        );
    }
}
