<?php
//module/Secure/src/Secure/Controller/AccountController.php

/**************
	Purpose: Provides ability to edit the profile of users.
	Author: Kevin Gyberg
	Date: 02/26/2013
*************/

namespace Secure\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Secure\Model\UsersTable;
use Secure\Model\Users;
use Secure\Model\Editusers;
use Secure\Form\AccountForm;
use Zend\Session\Container;
use Zend\Validator\EmailAddress;
use Zend\Filter\StripTags;
use Zend\Filter\StringTrim;
use Zend\Validator\Db\RecordExists;
	
class AccountController extends AbstractActionController
{
	/**
     * @var form
     */
	private $sessionInstance;
	
	/**
     * @var authservice
     */
	protected $authservice;
    
	/**
     * Gets an instance of authentication service.
     *
     * @return instance of authservice
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()->get('AuthService');
        }
        
        return $this->authservice;
    }
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Renders the register form and processes the posted data
     *
     * @return array(Registration for instance).
     */
	public function indexAction()
	{
		$isLoggedIn = 'yes';
		$session = $this->intializeSessionInstance();
		$id = $session->offsetGet('user_id');
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
        $rowSet = $this->getPersonTable()->getUserName($id);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
		}
		
		// Get the Album with the specified id.  An exception is thrown
        // if it cannot be found, in which case go to the index page.
        $userRecordSet = $this->getUsersTable()->getUser($id);
		
		$userData['userId']        = $userRecordSet->userid;
		$userData['userEmail']     = $userRecordSet->email_login; 
		$userData['userPassword']  = $userRecordSet->password;
		$userData['userFirstName'] = $userRecordSet->first_name;
		$userData['userLastName']  = $userRecordSet->last_name;
		
		// check if the request is a post request
        if ($this->getRequest()->isPost()) {
            
			$postData = $this->getRequest()->getPost();						
			if ($postData['cancel'] == 'Cancel') {				
				return $this->redirect()->toRoute('secure');		
			}
			$userInsertData = array();
			$isEditUserFormValid = true;
			$errorMessages['messages'] = array();
			
			// filter all inputs and add proper validations.
			// filter the email field
			$userInsertData['userEmail'] = $this->filterFormElements($postData['email_login']);
			
			if (!$userInsertData['userEmail']) {
				$isEditUserFormValid = false;
				$errorMessages['messages']['userEmail']['required'] = 'Please enter your email address.';
			}
			
			// Validate the email id field
			$emailvalidator = new EmailAddress();
			if ($userInsertData['userEmail'] && !$emailvalidator->isValid($userInsertData['userEmail'])) {
				$isEditUserFormValid = false;
				$errorMessages['messages']['userEmail']['invalid'] = 'The entered email address is invalid.';
			}
			else {
                $userInsertData['userEmail'] = strtolower($userInsertData['userEmail']);
                if ($userInsertData['userEmail'] != $userData['userEmail']) {
                    
                    
                    $emailExists = $this->getUsersTable()->ifEmailIdExists($userInsertData['userEmail']);
                    
                    if ($emailExists) {
                        $isEditUserFormValid = false;
                        $errorMessages['messages']['userEmail']['invalid'] = 'The entered email address already exists.';
                    }
                }
            }
			
			$userInsertData['password'] = $this->filterFormElements($postData['password']);
			$userInsertData['conf_password'] = $this->filterFormElements($postData['conf_password']);
			
			if (($userInsertData['password'] != '') || ($userInsertData['conf_password'] != '')) {
				if(!$this->validateConfirmPassword($userInsertData['password'], $userInsertData['conf_password'])) {
					$isEditUserFormValid = false;
					$errorMessages['messages']['conf_password']['invalid'] = 'The password and confirm password field do not match.';
				}
                
                $config = $this->getServiceLocator()->get('Config');
                if (strlen($userInsertData['password']) < $config['minimum_password_char']) {
                    $isEditUserFormValid = false;
                    $errorMessages['messages']['password']['invalidLength'] = 'Please enter Password minimum of 6 characters.';
                }
			}
			
			$userInsertData['userFirstName'] = $this->filterFormElements($postData['first_name']);
			if ($userInsertData['userFirstName'] == '') {
				$isEditUserFormValid = false;
				$errorMessages['messages']['userFirstName']['required'] = 'Please enter first name.';
			}
			
			$userInsertData['userLastName'] = $this->filterFormElements($postData['last_name']);
			if ($userInsertData['userLastName'] == '') {
				$isEditUserFormValid = false;
				$errorMessages['messages']['userLastName']['required'] = 'Please enter last name.';
			}
			
			// apply validations on the submitted form
            if ($isEditUserFormValid) {
			
				// save the user data to the users table.
                if ($this->getUsersTable()->updateUser($userInsertData, $id)) {
                    
                    // write the email id into the session
                    $session->offsetSet('email_login', $userInsertData['userEmail']);
                    
					return $this->redirect()->toRoute('secure');
				}
				// Unable to update user.
				$errorMessages['messages']['updateStatus']['error'] = 'Unable to update user data.';
            }
			
			return new ViewModel(
				array(
					'setdata'	=> $userInsertData,
					'messages'  => $errorMessages['messages']
				)
			);
        }
		
		return new ViewModel(
			array('setdata' => $userData)
		);
	}
	
	/**
     * Check whether both the entered password and confirm password are matched.
     * 
     * @param String $password
     * @param String $confirmPassword
     * @return boolean true on success and false on failure.
     * 
     */
	public function validateConfirmPassword($password = '', $confirmPassword = '')
	{
		if ($password && $confirmPassword) {
			if ($password === $confirmPassword) {
				return true;
			}
		}
		return false;
	}
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getPersonTable()
	{
		// check if usersTable previously defined.
		if (!$this->personTable) {
			$sm = $this->getServiceLocator();
			$this->personTable = $sm->get('Secure\Model\PersonTable');
		}
		return $this->personTable;
	}
	
	/**
     * Apply strip tags to the provided input
     * 
     * @param String $valueToFilter
     * @return Filtered string on Success and false on failure.
     * 
     */
	public function filterFormElements($valueToFilter = '')
    {
		// instantiate the strip tag filter
		$stripTagFilter   = new StripTags();
		$StringTrimFilter = new StringTrim();
		if ($valueToFilter) {
			return $stripTagFilter->filter($StringTrimFilter->filter($valueToFilter));
		}
		return false;
	}
}