<?php
//module/Secure/src/Secure/Controller/SecureController.php

/**************
	Purpose: Handels the stuff to do after successfully registered/logged in.
	Author: Kevin Gyberg
	Date: 02/28/2013
*************/

namespace Secure\Controller;
use Zend\Mvc\Controller\AbstractActionController;
use Secure\Model\EventsTable;
use Secure\Model\Events;
use Zend\View\Model\ViewModel;
use Zend\Session\Container;
use Zend\Validator\EmailAddress;
use Zend\Filter\StripTags;
use Zend\Filter\StringTrim;
use Zend\Validator\Date;
use Zend\Validator\Db\RecordExists;

class SecureController extends AbstractActionController
{
	/**
     * @var sessionInstance
     */
	private $sessionInstance;
	
	/**
     * @var authservice
     */
	private $authservice;
	
	/**
     * @var eventsTable
     */
	protected $eventsTable;
    
    /**
     * @var breadCrumbString
     */
    protected $breadCrumbString ='Home &rsaquo;&rsaquo; ';
    
    /**
     * Actions or operatons that can be performed on each tab
     */
    const ADD_ACTION           = 'add';
    const LIST_ACTION          = 'list';
    const UPDATE_ACTION        = 'update';
    
    /**
     * Contants for permission type
     */
    const VIEW_PERM          = 'view';
    const EDIT_PERM          = 'edit';
    const NO_ACCESS_PERM     = 'no_access';
    
    const DEFAULT_TIME          = '0000-00-00 00:00:00';
    
    /**
     * Contants for resources
     */
    const PERSON_RESOURCE           = 'person';
    const PROFILE_TYPE              = 'profile';
    const EVENT_RESOURCE            = 'event';
    const TASK_RESOURCE             = 'task';
    const PROVIDER_RESOURCE         = 'provider';
    const FOLDER_RESOURCE           = 'folder';
    const DOCUMENT_RESOURCE         = 'document';
    const POST_RESOURCE             = 'post';
    const CONTACT_RESOURCE          = 'contact';
    const MEDICATION_RESOURCE       = 'medication';
	
	/**
     * Gets an instance of authentication service.
     *
     * @return instance of the service locator
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()->get('AuthService');
        }
        
        return $this->authservice;
    }
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Renders the secure page with the data present in session.
     *
     * @return instance of view model(send data to view).
     */
    public function indexAction()
    {
		$isLoggedIn = 'yes';
        $parentResourceRecord = '';
        
		// check if the user is already logged in.
        if (! $this->getServiceLocator()->get('AuthService')->hasIdentity()) {
			$isLoggedIn = 'no';
            return $this->redirect()->toRoute('login');
        }
        
		$session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        
		$rowSet = $this->getPersonTable()->getUserName($userId);
        
        // If we get the user information then send it to the layouts file.
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
		}
        
        $convert = $this->getServiceLocator()->get('ConvertParams');
        $config = $this->getServiceLocator()->get('Config');
		$resourceidString = $this->params()->fromRoute('resourceid', 0);
        
        // initalization of the required variables.
        $resourcesUnderContainer = array();
        $breadCrumbString = '';
        
        $resourceArray = array();
        $postDataArray = array();
        
        // if we have the resource id string in the URL.
        if ($resourceidString) {
            
            $decodedPersonId = $convert->paramDecrypt($resourceidString);
            $resourceStringArray = $convert->stringToArray($decodedPersonId);
            
            $resourceRecord = $this
                                ->getResourceTable()
                                ->getResourceTypeById($resourceStringArray['resourceid']);
            
            $parentResourceRecord = $this
                                        ->getResourceTable()
                                        ->getParentResourceRecord($resourceStringArray['resourceid']);
            
            $inputArray         = $resourceStringArray;
            $inputArray['type'] = $resourceRecord['type'];
            
            /* get all the tabs from resource_type table and send them to the
                phtml/view file */
            $allTabsUnderProfile = $this
                                        ->getResourceTypeTable()
                                        ->getAllTabs($inputArray['resourceid']);
            
            // prepare the tabs array for sending it to the view file.
            $tabsArray = array();
            $tabCount = 0;
            foreach ($allTabsUnderProfile AS $tab) {
                
                if (($inputArray['tab_type'] == '') && ($tabCount == 0)) {
                    $inputArray['tab_type'] = $tab['type'];
                }
                
                switch ($tab['type']) {
                    case 'post':
                            $tabsArray[$tabCount]['type'] = $config['post'];
                            break;
                        
                    case 'event':
                            $tabsArray[$tabCount]['type'] = $config['event'];
                            break;
                        
                    case 'task':
                            $tabsArray[$tabCount]['type'] = $config['task'];
                            break;
                    
                    case 'document':
                            $tabsArray[$tabCount]['type'] = $config['document'];
                            break;
                        
                    case 'medication':
                            $tabsArray[$tabCount]['type'] = $config['medication'];
                            break;
                    
                    case 'contact':
                            $tabsArray[$tabCount]['type'] = $config['contact'];
                            break;
                    default:
                        break;
                }
                
                $tabHrefValue = array(
                    'resourceid'    => $inputArray['resourceid'],
                    'tab_type'      => $tab['type']
                );
                $tabsArray[$tabCount]['tab_href_value'] = $this->createEncodedString($tabHrefValue);
                
                if (isset($inputArray['tab_type']) && $inputArray['tab_type'] != '') {
                    
                    if ($inputArray['tab_type'] == $tab['type']) {
                        $tabsArray[$tabCount]['is_selected'] = TRUE;
                    }
                    else {
                        $tabsArray[$tabCount]['is_selected'] = FALSE;
                    }
                }
                else {
                    if ($inputArray['type'] == self :: FOLDER_RESOURCE) {
                        
                        if ($tab['type'] == self :: DOCUMENT_RESOURCE) {
                            $tabsArray[$tabCount]['is_selected'] = TRUE;
                        }
                    }
                    else if ($tab['type'] == self :: POST_RESOURCE) {
                        $tabsArray[$tabCount]['is_selected'] = TRUE;
                    }
                }
                $tabCount++;
            }
            
            $resourcesUnderContainer = $this
                                            ->getResourceTable()
                                            ->getResourcesUnderContainer($inputArray['resourceid'], $userId);
            
            // check if its a person or a provider profile
            if (($inputArray['type'] == self :: PERSON_RESOURCE)
                    || ($inputArray['type'] == self :: PROVIDER_RESOURCE)) {
                
                $resourceArray = $this->getResourceArray($userId, $inputArray['resourceid']);
            }
            else {
                $parentResourceRecord = $this
                                            ->getResourceTable()
                                            ->getParentResourceRecord($inputArray['resourceid']);

                if ($parentResourceRecord) {
                    $resourceArray = $this->getResourceArray($userId, $parentResourceRecord['resourceid']);
                }
            }
            
            // else send the parent resource id.
            $breadCrumbString = $this
                                   ->getResourceTable()
                                   ->createBreadcrumb($inputArray['resourceid'], $userId);
        }
        else {
            $resourcesUnderContainer = $this
                                            ->getResourceTable()
                                            ->getAllResourcesUnderRoot($userId);
            
            $resourceArray = $this->getResourceArray($userId);
            
            // collect the default breadcrumb string.
            $breadCrumbString = $this->breadCrumbString;
        }
		
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'resourceArray'     => $resourceArray,
                    'count'             => $resourceArray['count'],
                    'breadCrumbString'  => $breadCrumbString,
                    'is_logged_in'      => $isLoggedIn
                )
        );
        
        // check if the resourcetype is person or provider
        if ($inputArray['type'] == self :: PERSON_RESOURCE
                && $resourceStringArray['child_resourceid'] == '') {

            if ($inputArray['action'] == self :: UPDATE_ACTION) {

                $caredforResourceRecord = $this
                                        ->getResourceTable()
                                        ->getResourceTypeById($inputArray['resourceid']);

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {

                    $postedData = $this->getRequest()->getPost();

                    $sendArrayToEditCaredfor = $this->caredforEdit($userId, $caredforResourceRecord, $postedData);
                }
                else {
                    $permType = $this
                                    ->getUserResourceTable()
                                    ->isAnyPermissionExist($userId, $inputArray['resourceid']);
                    
                    if ($permType) {
                        $permType = $permType['perm_type'];
                    }
                    else {
                        $permType = self :: EDIT_PERM;
                    }
                    
                    if ($permType == self :: VIEW_PERM) {
                        $sendArrayToViewCaredfor = $this->caredforView($userId, $caredforResourceRecord);
                        
                        $encodeArray = array();
                        $encodeArray['resourceid']  = $inputArray['resourceid'];
                        $encodeArray['tab_type']    = self :: PERSON_RESOURCE;
                        $encodeArray['action']      = self :: LIST_ACTION;

                        $encodedString = $this->createEncodedString($encodeArray);
                        
                        $sendArrayToViewCaredfor = array_merge( (array) $sendArrayToViewCaredfor, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count'],
                                        'returnUrlString'           => $encodedString
                                    )
                        );

                        $view = new ViewModel($sendArrayToViewCaredfor);
                        $view->setTemplate('secure/caredfor-view');
                    }
                    else if ($permType == self :: EDIT_PERM) {
                        $sendArrayToEditCaredfor = $this->caredforEdit($userId, $caredforResourceRecord);
                        
                        $sendArrayToEditCaredfor = array_merge( (array) $sendArrayToEditCaredfor, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count']
                                )
                        );

                        $view = new ViewModel($sendArrayToEditCaredfor);
                        $view->setTemplate('secure/caredfor-edit');
                    }
                    return $view;
                }

                $sendArrayToEditCaredfor = array_merge( (array) $sendArrayToEditCaredfor, 
                        array(
                                'tabs'                      => $tabsArray,
                                'resourceArray'             => $resourceArray['resourceArray'],
                                'breadCrumbString'          => $breadCrumbString,
                                'count'                     => $resourceArray['count']
                        )
                );

                $view = new ViewModel($sendArrayToEditCaredfor);

                $view->setTemplate('secure/caredfor-edit');
                return $view;
            }
        }
        
        if ($inputArray['type'] == self :: PROVIDER_RESOURCE
                && $resourceStringArray['child_resourceid'] == '') {

            if ($inputArray['action'] == self :: UPDATE_ACTION) {

                $providerResourceRecord = $this
                                        ->getResourceTable()
                                        ->getResourceTypeById($inputArray['resourceid']);

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {

                    $postedData = $this->getRequest()->getPost();

                    $sendArrayToEditProvider = $this->providerEdit($userId, $providerResourceRecord, $postedData);
                }
                else {
                    
                    $permType = $this
                                    ->getUserResourceTable()
                                    ->getPermissionForResource($userId, $inputArray['resourceid'], '');
                    
                    if ($permType == self :: VIEW_PERM) {
                        $sendArrayToViewProvider = $this->providerView($userId, $providerResourceRecord);
                        
                        $encodeArray = array();
                        $encodeArray['resourceid']  = $inputArray['resourceid'];
                        $encodeArray['tab_type']    = self :: PROVIDER_RESOURCE;
                        $encodeArray['action']      = self :: LIST_ACTION;

                        $encodedString = $this->createEncodedString($encodeArray);
                        
                        $sendArrayToViewProvider = array_merge( (array) $sendArrayToViewProvider, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count'],
                                        'returnUrlString'           => $encodedString
                                    )
                        );

                        $view = new ViewModel($sendArrayToViewProvider);
                        $view->setTemplate('secure/provider-view');
                    }
                    else if ($permType == self :: EDIT_PERM) {
                        $sendArrayToEditProvider = $this->providerEdit($userId, $providerResourceRecord);
                        
                        $sendArrayToEditProvider = array_merge( (array) $sendArrayToEditProvider, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count']
                                )
                        );

                        $view = new ViewModel($sendArrayToEditProvider);
                        $view->setTemplate('secure/provider-edit');
                    }
                    return $view;
                }

                $sendArrayToEditProvider = array_merge( (array) $sendArrayToEditProvider, 
                        array(
                                'tabs'                      => $tabsArray,
                                'resourceArray'             => $resourceArray['resourceArray'],
                                'breadCrumbString'          => $breadCrumbString,
                                'count'                     => $resourceArray['count']
                        )
                );

                $view = new ViewModel($sendArrayToEditProvider);

                $view->setTemplate('secure/provider-edit');
                return $view;
            }
        }
        
        // check if the current tab is POST tab.
        if ($inputArray['tab_type'] == self :: POST_RESOURCE) {

            $request = $this->getRequest();
            $postDataArray = $this->getPostTable()->getAllPost($inputArray['resourceid']);

            // traverse $postDataArray array and prepare posts array to send it to the view.
            $postArrayToSend = array();
            $count = 0;
            foreach ($postDataArray AS $postArray) {

                $postArrayToSend[$count]['postid'] = $postArray['postid'];

                $rowSet = $this->getPersonTable()->getUserName($postArray['fk_userid_owner']);
                if ($rowSet) {
                    $postArrayToSend[$count]['name']  = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
                }

                if ($postArray['created'] == self :: DEFAULT_TIME) {
                    $postArrayToSend[$count]['updated'] = '';
                }
                else {
                    $postArrayToSend[$count]['updated'] = date("m/d/Y H:i:s", strtotime($postArray['created']));
                }

                $postArrayToSend[$count]['post']      = $postArray['post'];
                $count++;
            }
            
            $permType = $this
                            ->getUserResourceTable()
                            ->getPermissionForResource($userId, $inputArray['resourceid'], '');

            // check if the request is a post request
            if ($request->isPost()) {
                $postedData = $this->getRequest()->getPost();

                $postedData['parent_resource_id'] = $inputArray['resourceid'];

                // call addPost function to insert/create a post record.
                $viewArray = $this->addPost(
                                            $postedData, 
                                            $userId, 
                                            $resourceArray['resourceArray'],
                                            $breadCrumbString,
                                            $resourceArray['count'],
                                            $tabsArray,
                                            $inputArray['tab_type']
                );
                
                $viewArray = array_merge(
                                    array('resourcePermission' => $permType), 
                                    $viewArray
                );
                
                $view = new ViewModel($viewArray);
                $view->setTemplate('secure/post-view');
                return $view;
            }

            $sendArrayToPostView = array(
                'tabs'                      => $tabsArray,
                'resourceArray'             => $resourceArray['resourceArray'],
                'breadCrumbString'          => $breadCrumbString,
                'count'                     => $resourceArray['count'],
                'addTaskEncryptedString'    => $addTaskEncryptedString,
                'postDataArray'             => $postArrayToSend,
                'resourcePermission'        => $permType
            );

            $view = new ViewModel($sendArrayToPostView);
            $view->setTemplate('secure/post-view');
            return $view;
        }

        // check if the current tab is an event tab.
        if ($inputArray['tab_type'] == self :: EVENT_RESOURCE) {

            $breadCrumbString = $this
                                   ->getResourceTable()
                                   ->createBreadcrumb($inputArray['resourceid'], $userId);

            // Fetch all the events present under this folder or container
            $eventRecords = $this->getEventsTable()->fetchAll($inputArray['resourceid']);
            
            $permType = $this
                            ->getUserResourceTable()
                            ->getPermissionForResource($userId, $inputArray['resourceid'], '');

            $sendArrayToPostView = array(
                'tabs'                      => $tabsArray,
                'resourceArray'             => $resourceArray['resourceArray'],
                'breadCrumbString'          => $breadCrumbString,
                'count'                     => $resourceArray['count'],
                'postDataArray'             => $postArrayToSend,
                'calendarId'                => $inputArray['resourceid'],
                'eventRecords'              => $eventRecords,
                'resourcePermission'        => $permType
            );
            // just load the calendar view for now.
            $view = new ViewModel($sendArrayToPostView);
            $view->setTemplate('resource/calendar');
            return $view;
        }

        // check if the current tab is a task tab.
        if ($inputArray['tab_type'] == self :: TASK_RESOURCE) {

            // create the encrypted string for add task 
            $inputString = $convert->arrayToString(array_merge(
                                                                (array) $inputArray, 
                                                                array(
                                                                    'tab_type' => self :: TASK_RESOURCE,
                                                                    'action' =>   self :: ADD_ACTION
                                                                )
                    )
            );

            $addTaskEncryptedString = $convert->paramEncrypt($inputString);

            // check if its an add action.
            if ($inputArray['action'] == self :: ADD_ACTION) {

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {
                    $postedData = $this->getRequest()->getPost();

                    $postedData['parent_resource_id'] = $inputArray['resourceid'];
                    $viewArray = $this->addTask(
                                                $postedData, 
                                                $userId, 
                                                $resourceArray['resourceArray'],
                                                $breadCrumbString,
                                                $resourceArray['count'],
                                                $tabsArray,
                                                $inputArray['tab_type']
                    );
                    $view = new ViewModel($viewArray);
                    $view->setTemplate('resource/add_tasks');
                    return $view;
                }

                $sendArrayToAddTaskView = array(
                    'action'                    => self :: LIST_ACTION,
                    'tabs'                      => $tabsArray,
                    'resourceArray'             => $resourceArray['resourceArray'],
                    'breadCrumbString'          => $breadCrumbString,
                    'count'                     => $resourceArray['count']
                );
                $view = new ViewModel($sendArrayToAddTaskView);

                $view->setTemplate('resource/add_tasks');
                return $view;
            } // check if its an update action.
            else if ($inputArray['action'] == self :: UPDATE_ACTION) {

                $taskResourceRecord = $this
                                        ->getResourceTable()
                                        ->getResourceTypeById($inputArray['child_resourceid']);

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {

                    $postedData = $this->getRequest()->getPost();
                    $sendArrayToEditTask = $this->editTask($userId, $taskResourceRecord, $postedData);
                }
                else {
                    
                    $permType = $this
                                    ->getUserResourceTable()
                                    ->getPermissionForResource($userId, $inputArray['child_resourceid'], $inputArray['resourceid']);
                    
                    if ($permType == self :: VIEW_PERM) {
                        $sendArrayToViewTask = $this->viewTask($userId, $taskResourceRecord);
                        
                        $encodeArray = array();
                        $encodeArray['resourceid']  = $inputArray['resourceid'];
                        $encodeArray['tab_type']    = self :: TASK_RESOURCE;
                        $encodeArray['action']      = self :: LIST_ACTION;

                        $encodedString = $this->createEncodedString($encodeArray);
                        
                        $sendArrayToViewTask = array_merge( (array) $sendArrayToViewTask, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count'],
                                        'returnUrlString'           => $encodedString
                                    )
                        );

                        $view = new ViewModel($sendArrayToViewTask);
                        $view->setTemplate('secure/task-view');
                    }
                    else if ($permType == self :: EDIT_PERM) {
                        $sendArrayToEditTask = $this->editTask($userId, $taskResourceRecord);
                        
                        $sendArrayToEditTask = array_merge( (array) $sendArrayToEditTask, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count']
                                )
                        );

                        $view = new ViewModel($sendArrayToEditTask);
                        $view->setTemplate('secure/task-edit');
                    }
                    return $view;
                }
            }
            else { // else its listing action.
                $viewArray = $this->listTasks(
                                            $inputArray,
                                            $parentResourceRecord
                );
                
                $permType = $this
                                ->getUserResourceTable()
                                ->getPermissionForResource($userId, $inputArray['resourceid'], '');
                
                // resource/tasks_listing
                $sendArrayToTaskListView = array_merge( (array) $viewArray,
                        array(
                            'tabs'                      => $tabsArray,
                            'resourceArray'             => $resourceArray['resourceArray'],
                            'breadCrumbString'          => $breadCrumbString,
                            'count'                     => $resourceArray['count'],
                            'addTaskEncryptedString'    => $addTaskEncryptedString,
                            'resourcePermission'        => $permType
                        )
                );
                $view = new ViewModel($sendArrayToTaskListView);

                $view->setTemplate('resource/tasks_listing');
                return $view;
            }
        }

        // check if the current tab is a document tab
        if ($inputArray['tab_type'] == self :: DOCUMENT_RESOURCE) {
            $sendArrayToEditDocument = array();
            // create the encrypted string for add task
            $inputString = $convert->arrayToString( array_merge(
                                                                (array) $inputArray, 
                                                                array(
                                                                    'tab_type'  => self :: DOCUMENT_RESOURCE,
                                                                    'action'    => self :: ADD_ACTION
                                                                )
                    )
            );

            $addDocumentEncryptedString = $convert->paramEncrypt($inputString);

            // check if the current action is add.
            if ($inputArray['action'] == self :: ADD_ACTION) {

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {
                    $postedData = $this->getRequest()->getPost();
                    $files = $this->getRequest()->getFiles();

                    $postedData['parent_resource_id'] = $inputArray['resourceid'];

                    // call add document funtion to add the document into the system.
                    $viewArray = $this->addDocument(
                                                $postedData,
                                                $userId,
                                                $resourceArray['resourceArray'],
                                                $breadCrumbString,
                                                $resourceArray['count'],
                                                $tabsArray,
                                                $inputArray['tab_type'],
                                                $files
                    );
                    $view = new ViewModel($viewArray);
                    $view->setTemplate('resource/add_documents');
                    return $view;
                }

                $sendArrayToAddDocumentView = array(
                    'action'                    => self :: LIST_ACTION,
                    'tabs'                      => $tabsArray,
                    'resourceArray'             => $resourceArray['resourceArray'],
                    'breadCrumbString'          => $breadCrumbString,
                    'count'                     => $resourceArray['count']
                );
                $view = new ViewModel($sendArrayToAddDocumentView);

                $view->setTemplate('resource/add_documents');
                return $view;
            } // check if the current acction is update.
            else if ($inputArray['action'] == self :: UPDATE_ACTION) {
                $documentResourceRecord = $this
                                        ->getResourceTable()
                                        ->getResourceTypeById($inputArray['child_resourceid']);

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {

                    $postedData = $this->getRequest()->getPost();
                    $sendArrayToEditDocument = $this->editDocument($userId, $documentResourceRecord, $postedData);
                }
                else {
                    $permType = $this
                                    ->getUserResourceTable()
                                    ->getPermissionForResource($userId, $inputArray['child_resourceid'], $inputArray['resourceid']);
                    
                    if ($permType == self :: VIEW_PERM) {
                        $sendArrayToViewDocument = $this->viewDocument($userId, $documentResourceRecord);
                        
                        $encodeArray = array();
                        $encodeArray['resourceid']  = $inputArray['resourceid'];
                        $encodeArray['tab_type']    = self :: DOCUMENT_RESOURCE;
                        $encodeArray['action']      = self :: LIST_ACTION;

                        $encodedString = $this->createEncodedString($encodeArray);
                        
                        $sendArrayToViewDocument = array_merge( (array) $sendArrayToViewDocument,
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count'],
                                        'returnUrlString'           => $encodedString
                                    )
                        );

                        $view = new ViewModel($sendArrayToViewDocument);
                        $view->setTemplate('secure/document-view');
                    }
                    else if ($permType == self :: EDIT_PERM) {
                        $sendArrayToEditDocument = $this->editDocument($userId, $documentResourceRecord);
                        
                        $sendArrayToEditDocument = array_merge( (array) $sendArrayToEditDocument, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count']
                                )
                        );

                        $view = new ViewModel($sendArrayToEditDocument);
                        $view->setTemplate('secure/document-edit');
                    }
                    return $view;
                }
               
                $sendArrayToEditDocument = array_merge( (array) $sendArrayToEditDocument, 
                        array(
                                'tabs'                      => $tabsArray,
                                'resourceArray'             => $resourceArray['resourceArray'],
                                'breadCrumbString'          => $breadCrumbString,
                                'count'                     => $resourceArray['count']
                        )
                );
                
                $view = new ViewModel($sendArrayToEditDocument);

                $view->setTemplate('secure/document-edit');
                return $view;
            }
            else { // check if the current acction is listing.
                $viewArray = $this->listDocuments(
                                            $inputArray
                );
                
                $permType = $this
                                ->getUserResourceTable()
                                ->getPermissionForResource($userId, $inputArray['resourceid'], '');
                
                // check if its parent resource is a profile i.e. person or provider.
                $parentResourceRecord = $this
                                        ->getResourceTable()
                                        ->getParentResourceRecord($inputArray['resourceid']);
                
                $parentRecordString = '';
                if (($parentResourceRecord['type'] == self :: PERSON_RESOURCE)
                        || ($parentResourceRecord['type'] == self :: PROVIDER_RESOURCE)) {
                    // Create the once level up resource string.
                    $encodeArray = array(
                            'resourceid'    => $parentResourceRecord['resourceid']
                    );
                    
                    $parentRecordString = $this->createEncodedString($encodeArray);
                }

                $sendArrayToDocumentListView = array_merge( (array) $viewArray,
                        array(
                            'tabs'                          => $tabsArray,
                            'resourceArray'                 => $resourceArray['resourceArray'],
                            'breadCrumbString'              => $breadCrumbString,
                            'count'                         => $resourceArray['count'],
                            'addDocumentEncryptedString'    => $addDocumentEncryptedString,
                            'resourcePermission'            => $permType,
                            'parentRecordString'            => $parentRecordString
                        )
                );
                $view = new ViewModel($sendArrayToDocumentListView);

                $view->setTemplate('resource/documents_listing');
                return $view;
            }
        }

        // check if the current tab is contact tab.
        if ($inputArray['tab_type'] == self :: CONTACT_RESOURCE) {

            // create the encrypted string for add contact
            $inputString = $convert->arrayToString(array_merge(
                                                                (array) $inputArray, 
                                                                array(
                                                                    'tab_type'  => self :: CONTACT_RESOURCE,
                                                                    'action'    => self :: ADD_ACTION
                                                                )
                    )
            );

            $addContactEncryptedString = $convert->paramEncrypt($inputString);

            // check if the current action is an add action.
            if ($inputArray['action'] == self :: ADD_ACTION) {

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {
                    $postedData = $this->getRequest()->getPost();

                    $postedData['parent_resource_id'] = $inputArray['resourceid'];

                    // call the add contact action for adding contacts.
                    $viewArray = $this->addContact(
                                                $postedData,
                                                $userId,
                                                $resourceArray['resourceArray'],
                                                $breadCrumbString,
                                                $resourceArray['count'],
                                                $tabsArray,
                                                $inputArray['tab_type']
                    );
                    $view = new ViewModel($viewArray);
                    $view->setTemplate('resource/add_contacts');
                    return $view;
                }

                $sendArrayToAddContactView = array(
                    'action'                    => self :: LIST_ACTION,
                    'tabs'                      => $tabsArray,
                    'resourceArray'             => $resourceArray['resourceArray'],
                    'breadCrumbString'          => $breadCrumbString,
                    'count'                     => $resourceArray['count']
                );
                $view = new ViewModel($sendArrayToAddContactView);

                $view->setTemplate('resource/add_contacts');
                return $view;
            } // check if its an update action.
            else if ($inputArray['action'] == self :: UPDATE_ACTION) {

                // render view for updating contact
                $contactResourceRecord = $this
                                        ->getResourceTable()
                                        ->getResourceTypeById($inputArray['child_resourceid']);

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {

                    $postedData = $this->getRequest()->getPost();
                    $sendArrayToEditContact = $this
                                                    ->editContact($userId, $contactResourceRecord, $postedData);
                }
                else {
                    $permType = $this
                                    ->getUserResourceTable()
                                    ->getPermissionForResource($userId, $inputArray['child_resourceid'], $inputArray['resourceid']);
                    
                    if ($permType == self :: VIEW_PERM) {
                        $sendArrayToViewContact = $this->viewContact($userId, $contactResourceRecord);
                        
                        $encodeArray = array();
                        $encodeArray['resourceid']  = $inputArray['resourceid'];
                        $encodeArray['tab_type']    = self :: CONTACT_RESOURCE;
                        $encodeArray['action']      = self :: LIST_ACTION;

                        $encodedString = $this->createEncodedString($encodeArray);
                        
                        $sendArrayToViewContact = array_merge( (array) $sendArrayToViewContact, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count'],
                                        'returnUrlString'           => $encodedString
                                    )
                        );

                        $view = new ViewModel($sendArrayToViewContact);
                        $view->setTemplate('secure/contact-view');
                    }
                    else if ($permType == self :: EDIT_PERM) {
                        
                        $sendArrayToEditContact = $this->editContact($userId, $contactResourceRecord);
                        $sendArrayToEditContact = array_merge( (array) $sendArrayToEditContact, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count']
                                )
                        );

                        $view = new ViewModel($sendArrayToEditContact);
                        $view->setTemplate('secure/contact-edit');
                    }
                    return $view;
                }

                $sendArrayToEditContact = array_merge( (array) $sendArrayToEditContact, 
                        array(
                                'tabs'                      => $tabsArray,
                                'resourceArray'             => $resourceArray['resourceArray'],
                                'breadCrumbString'          => $breadCrumbString,
                                'count'                     => $resourceArray['count']
                        )
                );

                $view = new ViewModel($sendArrayToEditContact);

                $view->setTemplate('secure/contact-edit');
                return $view;
            }
            else {  // default is the listing of contacts.
                $viewArray = $this->listContacts(
                                            $inputArray
                );
                
                $permType = $this
                                ->getUserResourceTable()
                                ->getPermissionForResource($userId, $inputArray['resourceid'], '');

                $sendArrayToContactListView = array_merge( (array) $viewArray,
                        array(
                            'tabs'                          => $tabsArray,
                            'resourceArray'                 => $resourceArray['resourceArray'],
                            'breadCrumbString'              => $breadCrumbString,
                            'count'                         => $resourceArray['count'],
                            'addContactEncryptedString'     => $addContactEncryptedString,
                            'resourcePermission'            => $permType
                        )
                );
                $view = new ViewModel($sendArrayToContactListView);

                $view->setTemplate('resource/contacts_listing');
                return $view;
            }
        }

        // check if the current tab is medication tab.
        if ($inputArray['tab_type'] == self :: MEDICATION_RESOURCE) {

            // create the encrypted string for add task 
            $inputString = $convert->arrayToString(array_merge(
                                                                (array) $inputArray, 
                                                                array(
                                                                    'tab_type'  => self :: MEDICATION_RESOURCE,
                                                                    'action'    => self :: ADD_ACTION
                                                                )
                    )
            );

            $addMedicationEncryptedString = $convert->paramEncrypt($inputString);

            // check if its an add action
            if ($inputArray['action'] == self :: ADD_ACTION) {

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {
                    $postedData = $this->getRequest()->getPost();

                    $postedData['parent_resource_id'] = $inputArray['resourceid'];
                    $viewArray = $this->addMedication(
                                                $postedData, 
                                                $userId,
                                                $resourceArray['resourceArray'],
                                                $breadCrumbString,
                                                $resourceArray['count'],
                                                $tabsArray,
                                                $inputArray['tab_type']
                    );
                    $view = new ViewModel($viewArray);
                    $view->setTemplate('resource/add_medications');
                    return $view;
                }

                $sendArrayToAddMedicationView = array(
                    'action'                    => self :: LIST_ACTION,
                    'tabs'                      => $tabsArray,
                    'resourceArray'             => $resourceArray['resourceArray'],
                    'breadCrumbString'          => $breadCrumbString,
                    'count'                     => $resourceArray['count']
                );
                $view = new ViewModel($sendArrayToAddMedicationView);

                $view->setTemplate('resource/add_medications');
                return $view;
            } // check if its an update action.
            else if ($inputArray['action'] == self :: UPDATE_ACTION) {

                // render view for updating medication
                $medicationResourceRecord = $this
                                            ->getResourceTable()
                                            ->getResourceTypeById($inputArray['child_resourceid']);

                $request = $this->getRequest();

                // check if the request is a post request
                if ($request->isPost()) {

                    $postedData = $this->getRequest()->getPost();
                    $sendArrayToEditMedication = $this
                                                    ->editMedication($userId, $medicationResourceRecord, $postedData);
                }
                else {
                    $permType = $this
                                    ->getUserResourceTable()
                                    ->getPermissionForResource($userId, $inputArray['child_resourceid'], $inputArray['resourceid']);
                    
                    if ($permType == self :: VIEW_PERM) {
                        $sendArrayToViewMedication = $this->viewMedication($userId, $medicationResourceRecord);
                        
                        $encodeArray = array();
                        $encodeArray['resourceid']  = $inputArray['resourceid'];
                        $encodeArray['tab_type']    = self :: MEDICATION_RESOURCE;
                        $encodeArray['action']      = self :: LIST_ACTION;

                        $encodedString = $this->createEncodedString($encodeArray);
                        
                        $sendArrayToViewMedication = array_merge( (array) $sendArrayToViewMedication, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count'],
                                        'returnUrlString'           => $encodedString
                                    )
                        );

                        $view = new ViewModel($sendArrayToViewMedication);
                        $view->setTemplate('secure/medication-view');
                    }
                    else if ($permType == self :: EDIT_PERM) {
                        
                        $sendArrayToEditMedication = $this->editMedication($userId, $medicationResourceRecord);
                        $sendArrayToEditMedication = array_merge( (array) $sendArrayToEditMedication, 
                                array(
                                        'tabs'                      => $tabsArray,
                                        'resourceArray'             => $resourceArray['resourceArray'],
                                        'breadCrumbString'          => $breadCrumbString,
                                        'count'                     => $resourceArray['count']
                                )
                        );

                        $view = new ViewModel($sendArrayToEditMedication);
                        $view->setTemplate('secure/medication-edit');
                    }
                    return $view;
                }

                $sendArrayToEditMedication = array_merge( (array) $sendArrayToEditMedication, 
                                                array(
                                                        'tabs'                      => $tabsArray,
                                                        'resourceArray'             => $resourceArray['resourceArray'],
                                                        'breadCrumbString'          => $breadCrumbString,
                                                        'count'                     => $resourceArray['count']
                                                )
                );

                $view = new ViewModel($sendArrayToEditMedication);

                $view->setTemplate('secure/medication-edit');
                return $view;

            } // else it's a listing action.
            else {
                $viewArray = $this->listMedications(
                                            $inputArray
                );
                
                $permType = $this
                                ->getUserResourceTable()
                                ->getPermissionForResource($userId, $inputArray['resourceid'], '');
                
                // check if its parent resource is a profile i.e. person or provider.
                $parentResourceRecord = $this
                                        ->getResourceTable()
                                        ->getParentResourceRecord($inputArray['resourceid']);
                
                $parentRecordString = '';
                if (($parentResourceRecord['type'] == self :: PERSON_RESOURCE)
                        || ($parentResourceRecord['type'] == self :: PROVIDER_RESOURCE)) {
                    // Create the once level up resource string.
                    $encodeArray = array(
                            'resourceid'    => $parentResourceRecord['resourceid']
                    );
                    
                    $parentRecordString = $this->createEncodedString($encodeArray);
                }

                $sendArrayToMedicationListView = array_merge( (array) $viewArray,
                        array(
                            'tabs'                          => $tabsArray,
                            'breadCrumbString'              => $breadCrumbString,
                            'count'                         => $resourceArray['count'],
                            'addMedicationEncryptedString'  => $addMedicationEncryptedString,
                            'resourcePermission'            => $permType,
                            'parentRecordString'            => $parentRecordString
                        )
                );
                $view = new ViewModel($sendArrayToMedicationListView);

                $view->setTemplate('resource/medications_listing');
                return $view;
            }
        }
        
        // check if there is any profile under the root folder.
        if (count($resourcesUnderContainer) > 0) {

            $arrayForListing = array();
            $count = 0;
            // traverse the resource/profile collected.
            foreach ($resourcesUnderContainer as $resource) {
                    
                if ($resource['type'] == self :: PERSON_RESOURCE
                        || $resource['type'] == self :: PROVIDER_RESOURCE) {

                    $encodeArray = array();

                    $encodeArray['resourceid'] = $resource['resourceid'];
                    $arrayForListing[$count]['resourceid'] = $this->createEncodedString($encodeArray);
                    
                    
                    // Check if the logged in user is the owner of given profile record.
                        // allow share, edit and delete.
                    
                    // Check if the logged in user is being shared with "edit" permission
                        // allow share, edit and delete.
                    
                    // Check if the logged in user is being shared with "view" permission
                        // do not allow the user to share or( edit/delete )
                    
                    // Then list all the resources present under it exp calendar, contact_group, task_group.
                    if ($resource['ownerid'] != $userId) {
                        
                        // the navigated resource is a shared resource.
                        $arrayForListing[$count]['container_name'] = $resource['description'] . ' (Shared)';
                        $arrayForListing[$count]['perm_type'] = $resource['perm_type'];
                        
                        if ($resource['perm_type'] == self :: EDIT_PERM) {
                            
                            $arrayForListing[$count]['can_share'] = TRUE;
                        }
                        else if ($resource['perm_type'] == self :: VIEW_PERM) {
                            
                            $arrayForListing[$count]['can_share'] = FALSE;
                        }
                    }
                    else {
                        // the navigated 
                        if ($resource['person_owner_id'] == $userId) {
                            $arrayForListing[$count]['container_name'] = 'My Profile';
                        }
                        else {
                            $arrayForListing[$count]['container_name'] = $resource['description'];
                        }
                        
                        $arrayForListing[$count]['can_share'] = TRUE;
                    }

                    $arrayForListing[$count]['owner_name'] = $resource['owner_firstname'] 
                            . ' ' . $resource['owner_lastname'];

                    $arrayForListing[$count]['pid'] = $convert->encode($resource['pid']);
                    
                    $encodeArray = array();

                    $encodeArray['resourceid']  = $resource['resourceid'];
                    $encodeArray['action']      = self :: UPDATE_ACTION;
                    
                    $arrayForListing[$count]['link'] = $this->createEncodedString($encodeArray);

                    $arrayForListing[$count]['type'] = $resource['type'];
                    
                    if ($resource['updated'] == self :: DEFAULT_TIME) {
                        $arrayForListing[$count]['updated'] = '';
                    }
                    else {
                        $arrayForListing[$count]['updated'] = date("m/d/Y H:i:s", strtotime($resource['updated']));
                    }
                }
                $count++;
            }
        }
        
        
        // Here we are sending resource array to the index view.
        $arrayForIndexView = array(
                'resourceType'      => $inputArray['type'],
                'tabs'              => $tabsArray,
                'tabType'           => $inputArray['tab_type'],
                'resourceId'        => $inputArray['resourceid'],
                'recordSets'        => $arrayForListing,
                'resourceString'    => $resourceidString,
                'count'             => $resourceArray['count']
        );
        
        return new ViewModel($arrayForIndexView);
    }
    
    /**
     * Function to Add post.
     * 
     * @param array     $postedData
     * @param int       $userId
     * @param array     $resourceArray
     * @param string    $breadcrumbString
     * @param int       $count
     * @param array     $tabsArray
     * @param string    $tabType
     * @return array
     */
    public function addPost($postedData, $userId, $resourceArray, $breadcrumbString, $count, $tabsArray, $tabType)
    {
        // check if the request is a post request
        if ($postedData) {
            
			$isPostFormValid = true;
			$errorMessages = array();
			$postUpdateData = array();
            $postUpdateData['post_new_comment'] = $this->filterFormElements($postedData['post_new_comment']);
            if ($postUpdateData['post_new_comment'] == '') {
				$isPostFormValid = false;
				$errorMessages['messages']['task_desc']['required'] = 'Please enter Task Description.';
			}
            
            $postUpdateData['parent_resource_id'] = $postedData['parent_resource_id'];
            
            if ($isPostFormValid) {
                
                // insert the medication data in to the DB
                $insertStatus = $this
                                    ->getPostTable()
                                    ->savePost($postUpdateData, $userId);
                if ($insertStatus) {
                    
                    $encodeArray['resourceid']  = $postUpdateData['parent_resource_id'];
                    $encodeArray['tab_type']    = self :: POST_RESOURCE;
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $this->redirect()->toRoute(
                                                'secure',
                                                array(
                                                    'action'        => 'index', 
                                                    'resourceid'    => $this->createEncodedString($encodeArray)
                            )
                    );
                }
            }
            
            // populate the form with the retrieved data
            return array(
                    'messages'              =>  $errorMessages,
                    'resourceArray'         =>  $resourceArray,
                    'count'                 =>  $count,
                    'breadCrumbString'      =>  $breadcrumbString,
                    'tabs'                  =>  $tabsArray,
                    'tabType'               =>  $tabType
            );
        }
    }
    
    /**
     * Function to add tasks under a function.
     * 
     * @param array     $postedData
     * @param int       $userId
     * @param array     $resourceArray
     * @param string    $breadcrumbString
     * @param int       $count
     * @param array     $tabsArray
     * @param string    $tabType
     * @return array
     */
	public function addTask($postedData, $userId, $resourceArray, $breadcrumbString, $count, $tabsArray, $tabType)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
		// check if the request is a post request
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
                
                /* we will think about this part in the near future */
				$returnArray = array(
                    'resourceid'    => $postedData['parent_resource_id'],
                    'tab_type'      =>  self :: TASK_RESOURCE,
                    'action'        =>  self :: LIST_ACTION
                );
                $returnString = $convert->arrayToString($returnArray);
                $encodedReturnString = $convert->paramEncrypt($returnString);
                return $this->redirect()->toRoute('secure', array('resourceid'    => $encodedReturnString));
			}
            
			$isTaskFormValid = true;
			$errorMessages = array();
			$taskUpdateData = array();
            $taskUpdateData['task_description'] = $this->filterFormElements($postedData['task_description']);
            if ($taskUpdateData['task_description'] == '') {
				$isTaskFormValid = false;
				$errorMessages['messages']['task_desc']['required'] = 'Please enter Task Description.';
			}
            
            $taskUpdateData['due_date'] = $this->filterFormElements($postedData['due_date']);
            
            // if the prescribed date is not equal to null
            if ($taskUpdateData['due_date'] != '') {
                $validator = new Date(array('format' => 'm/d/Y'));
                $isValid = $validator->isValid($taskUpdateData['due_date']);
                
                if (!$isValid) {
                    $isTaskFormValid = false;
                    unset($taskUpdateData['due_date']);
                    $errorMessages['messages']['due_date']['invalid'] = 'Please provide Due date in mm/dd/yyyy format or use the popup calendar to select a date';
                }
                else {
                    $taskUpdateData['due_date'] = date('Y-m-d', strtotime($taskUpdateData['due_date']));
                }
            }
            
            if (isset($postedData['priority']) && $postedData['priority'] != '') {
                $taskUpdateData['priority'] = $postedData['priority'];
            }
            else {
                $taskUpdateData['priority'] = '';
            }
            
            $taskUpdateData['person_responsibile'] = $this->filterFormElements($postedData['person_responsibile']);
            
            $taskUpdateData['task_notes'] = $this->filterFormElements($postedData['task_notes']);
            
            if (isset($postedData['is_completed']) && $postedData['is_completed'] != '') {
                $taskUpdateData['is_completed'] = $this->filterFormElements($postedData['is_completed']);
            }
            else {
                $taskUpdateData['is_completed'] = '';
            }
            
            $taskUpdateData['parent_resource_id'] = $postedData['parent_resource_id'];
            
            if ($isTaskFormValid) {
                
                // insert the medication data in to the DB
                $insertStatus = $this
                                    ->getTaskTable()
                                    ->saveTask($taskUpdateData, $userId);
                if ($insertStatus) {
                    
                    $encodeArray['resourceid']  = $taskUpdateData['parent_resource_id'];
                    $encodeArray['tab_type']    = self :: TASK_RESOURCE;
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $this->redirect()->toRoute(
                                                'secure',
                                                array(
                                                    'action'        => 'index', 
                                                    'resourceid'    => $this->createEncodedString($encodeArray)
                            )
                    );
                }
            }
            
            // populate the form with the retrieved data
            return array(
                    'taskAddData'         =>  $taskUpdateData,
                    'messages'            =>  $errorMessages,
                    'resourceArray'       =>  $resourceArray,
                    'count'               =>  $count,
                    'breadCrumbString'    =>  $breadcrumbString,
                    'tabs'                =>  $tabsArray,
                    'tabType'             =>  $tabType
            );
        }
    }
    
    /**
     * Function to add Medication under the medication tab.
     * 
     * @param array     $postedData
     * @param int       $userId
     * @param array     $resourceArray
     * @param string    $breadcrumbString
     * @param int       $count
     * @param array     $tabsArray
     * @param string    $tabType
     * @return array
     */
	public function addMedication($postedData, $userId, $resourceArray, $breadcrumbString, $count, $tabsArray, $tabType)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
		
		// check if the request is a post request
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
                
				/* we will think about this part in the near future */
				$returnArray = array(
                    'resourceid'    => $postedData['parent_resource_id'],
                    'tab_type'      =>  self :: MEDICATION_RESOURCE,
                    'action'        =>  self :: LIST_ACTION
                );
                $returnString = $convert->arrayToString($returnArray);
                $encodedReturnString = $convert->paramEncrypt($returnString);
                return $this->redirect()->toRoute('secure', array('resourceid'    => $encodedReturnString));
			}
            
			$isMedicationFormValid = true;
			$errorMessages = array();
			$medicationUpdateData = array();
			
			// adding validations to the input fields
			$medicationUpdateData['med_name'] = $this->filterFormElements($postedData['med_name']);
			if ($medicationUpdateData['med_name'] == '') {
				$isMedicationFormValid = false;
				$errorMessages['messages']['med_name']['required'] = 'Please enter medication name.';
			}
			
			$medicationUpdateData['generic_name'] = $this->filterFormElements($postedData['generic_name']);
			$medicationUpdateData['dosage'] = $this->filterFormElements($postedData['dosage']);
			$medicationUpdateData['dosage_frequency'] = $this->filterFormElements($postedData['dosage_frequency']);
			$medicationUpdateData['med_notes'] = $this->filterFormElements($postedData['med_notes']);
			$medicationUpdateData['physician'] = $this->filterFormElements($postedData['physician']);
			
            $medicationUpdateData['date_prescribed'] = $this->filterFormElements($postedData['date_prescribed']);
            
            // if the prescribed date is not equal to null
            if ($medicationUpdateData['date_prescribed'] != '') {
                $validator = new Date(array('format' => 'm/d/Y'));
                $isValid = $validator->isValid($medicationUpdateData['date_prescribed']);
                
                if (!$isValid) {
                    $isMedicationFormValid = false;
                    unset($medicationUpdateData['date_prescribed']);
                    $errorMessages['messages']['pre_date']['invalid'] = 'Please provide Date Prescribed in mm/dd/yyyy format or use the popup
                        calendar to select a date.';
                }
                else {
                    $medicationUpdateData['date_prescribed'] = date('Y-m-d', strtotime($medicationUpdateData['date_prescribed']));
                }
            }
            
			$medicationUpdateData['pharmacy'] = $this->filterFormElements($postedData['pharmacy']);
			$medicationUpdateData['symptoms_address'] = $this->filterFormElements($postedData['symptoms_address']);
			$medicationUpdateData['side_effects'] = $this->filterFormElements($postedData['side_effects']);
			$medicationUpdateData['drug_interactions'] = $this->filterFormElements($postedData['drug_interactions']);
			$medicationUpdateData['physical_desc'] = $this->filterFormElements($postedData['physical_desc']);
			$medicationUpdateData['parent_resource_id'] = $postedData['parent_resource_id'];
            
            if ($isMedicationFormValid) {
                
                // insert the medication data in to the DB
                $insertStatus = $this
                                    ->getMedicationTable()
                                    ->saveMedication($medicationUpdateData, $userId);
                if ($insertStatus) {
                    $encodeArray['resourceid']  = $medicationUpdateData['parent_resource_id'];
                    $encodeArray['tab_type']    = self :: MEDICATION_RESOURCE;
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $this->redirect()->toRoute(
                                                'secure',
                                                array(
                                                    'action'        => 'index', 
                                                    'resourceid'    => $this->createEncodedString($encodeArray)
                            )
                    );
                }
            }
            
            return array(
                    'medicationAddData'   =>  $medicationUpdateData,
                    'messages'            =>  $errorMessages,
                    'resourceArray'       =>  $resourceArray,
                    'count'               =>  $count,
                    'breadCrumbString'    =>  $breadcrumbString,
                    'tabs'                =>  $tabsArray,
                    'tabType'             =>  $tabType
            );
        }
    }
    
    /**
     * Function to add contacts
     * 
     * @param array     $postedData
     * @param int       $userId
     * @param array     $resourceArray
     * @param string    $breadcrumbString
     * @param int       $count
     * @param array     $tabsArray
     * @param stringt   $tabType
     * @return array
     */
	public function addContact($postedData, $userId, $resourceArray, $breadcrumbString, $count, $tabsArray, $tabType)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
        
		// check if the request is a post request
        if ($postedData) {
           
			if ($postedData['cancel'] == 'Cancel') {
                
				/* we will think about this part in the near future */
				$returnArray = array(
                    'resourceid'    => $postedData['parent_resource_id'],
                    'tab_type'      =>  self :: CONTACT_RESOURCE,
                    'action'        =>  self :: LIST_ACTION
                );
                $returnString = $convert->arrayToString($returnArray);
                $encodedReturnString = $convert->paramEncrypt($returnString);
                return $this->redirect()->toRoute('secure', array('resourceid'    => $encodedReturnString));
			}
            
			$isContactFormValid = true;
			$errorMessages = array();
			$contactUpdateData = array();
            
            $contactUpdateData['contact_name'] = $this->filterFormElements($postedData['contact_name']);
            if ($contactUpdateData['contact_name'] == '') {
				$isContactFormValid = false;
				$errorMessages['messages']['contact_name']['required'] = 'Please enter contact name.';
			}
            
            $contactUpdateData['business_name']     = $this->filterFormElements($postedData['business_name']);
            $contactUpdateData['contact_category']  = $this->filterFormElements($postedData['contact_category']);
            $contactUpdateData['email_address1']    = $this->filterFormElements($postedData['email_address1']);
            if ($contactUpdateData['email_address1'] != '') {
                
                $contactUpdateData['email_address1'] = strtolower($contactUpdateData['email_address1']);
                $emailvalidator = new EmailAddress();
                if (!$emailvalidator->isValid($contactUpdateData['email_address1'])) {
                    $isContactFormValid = false;
                    $errorMessages['messages']['email_invalid']['invalid'] = 'The format of the e-mail address entered is incorrect.';
                }
            }
            $contactUpdateData['mobile_number']     = $this->filterFormElements($postedData['mobile_number']);
            $contactUpdateData['home_number']     = $this->filterFormElements($postedData['home_number']);
            $contactUpdateData['work_number']       = $this->filterFormElements($postedData['work_number']);
            $contactUpdateData['work_address']      = $this->filterFormElements($postedData['work_address']);
            $contactUpdateData['work_city']      = $this->filterFormElements($postedData['work_city']);
            $contactUpdateData['work_state']     = $this->filterFormElements($postedData['work_state']);
            $contactUpdateData['work_zip']       = $this->filterFormElements($postedData['work_zip']);
            
            $contactUpdateData['home_address']      = $this->filterFormElements($postedData['home_address']);
            $contactUpdateData['home_city']      = $this->filterFormElements($postedData['home_city']);
            $contactUpdateData['home_state']     = $this->filterFormElements($postedData['home_state']);
            $contactUpdateData['home_zip']       = $this->filterFormElements($postedData['home_zip']);
            
            $contactUpdateData['home_page']         = $this->filterFormElements($postedData['home_page']);
            $contactUpdateData['contact_notes']     = $this->filterFormElements($postedData['contact_notes']);
            
            $contactUpdateData['parent_resource_id'] = $postedData['parent_resource_id'];
            
            if ($isContactFormValid) {
                
                // insert the medication data in to the DB
                $insertStatus = $this
                                    ->getContactTable()
                                    ->saveContact($contactUpdateData, $userId);
                if ($insertStatus) {
                    
                    $encodeArray['resourceid']  = $contactUpdateData['parent_resource_id'];
                    $encodeArray['tab_type']    = self :: CONTACT_RESOURCE;
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $this->redirect()->toRoute(
                                                'secure',
                                                array(
                                                    'action'        => 'index', 
                                                    'resourceid'    => $this->createEncodedString($encodeArray)
                            )
                    );
                }
            }
            
            return array(
                    'contactAddData'        =>  $contactUpdateData,
                    'messages'              =>  $errorMessages,
                    'resourceArray'         =>  $resourceArray,
                    'count'                 =>  $count,
                    'breadCrumbString'      =>  $breadcrumbString,
                    'tabs'                  =>  $tabsArray,
                    'tabType'               =>  $tabType
            );
        }
    }
    
    /**
     * Function to add documents under the documents tab.
     * 
     * @param array     $postedData
     * @param int       $userId
     * @param array     $resourceArray
     * @param string    $breadcrumbString
     * @param int       $count
     * @param array     $tabsArray
     * @param string    $tabType
     * @param array     $files
     * @return array     
     */
	public function addDocument($postedData, $userId, $resourceArray, $breadcrumbString, $count, $tabsArray, $tabType, $files)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
		$config = $this->getServiceLocator()->get('Config');
		// check if the request is a post request
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
                
				/* we will think about this part in the near future */
				$returnArray = array(
                    'resourceid'    => $postedData['parent_resource_id'],
                    'tab_type'      =>  self :: DOCUMENT_RESOURCE,
                    'action'        =>  self :: LIST_ACTION
                );
                $returnString = $convert->arrayToString($returnArray);
                $encodedReturnString = $convert->paramEncrypt($returnString);
                return $this->redirect()->toRoute('secure', array('resourceid'    => $encodedReturnString));
			}
            
			$isDocumentFormValid = true;
			$errorMessages = array();
			$documentUpdateData = array();
			
            $documentUpdateData['document_description'] = $this->filterFormElements($postedData['document_description']);
			
            if ($documentUpdateData['document_description'] == '') {
				$isDocumentFormValid = false;
				$errorMessages['messages']['document_desc']['required'] = 'Please enter document description.';
			}
            
            $documentUpdateData['parent_resource_id'] = $postedData['parent_resource_id'];
            
            $ext = end(explode('.', $files['document_file']['name']));
            
            if ($files['document_file']['name']) {
               
                if (!($this->isValidMimeType($files['document_file']['type'])) 
                        || !($this->isValidFileType($ext))) {
                    $isDocumentFormValid = false;
                    $errorMessages['messages']['document_file']['file_type'] = 'You 
                        have selected a filetype that is not allowed. Please upload 
                        approved types only such as:<br />
                        All "Office" documents: doc, docx, xls, xlsx, mdb<br />
                        All common image types: jpg, jepg, bmp, tif, gif<br />
                        Other common types: txt, zip pdf, ppt, 7z, mp3, mp4, mpeg and mpg';
                }
            }
            
			//Checking for file uploding error.
			if ($files['document_file']['error'] != 0) {
				$isDocumentFormValid = false;
				$errorMessages['messages']['document_file']['required'] = 'Please select a file to upload.';
			}
            
			//Checking for file upload size error.
			if ($files['document_file']['size'] > $config['max_upload_filesize']) {
				$isDocumentFormValid = false;
				$errorMessages['messages']['document_file']['max'] = 'Uploaded file exceeded max upload file size i.e. ' 
                    . ($config['max_upload_filesize'] / (1024 * 1024)) . ' MB';
			}
			
			//If no file error and form validated correctly the move the uploded file
			if (($files['document_file']['error'] === 0) && ($isDocumentFormValid)) {
			
				//Getting file path from config data
				$uploadPath = $config['document_upload_location'];
				//The new file name will be user id with current timestamp separated by underscore
				$newFileName = $userId . '_' . time();
				
                
                $documentUpdateData['actual_file_name'] = $files['document_file']['name'];
			
				//Moving file to a destination
				if(!(move_uploaded_file($files['document_file']['tmp_name'], $uploadPath . $newFileName . '.' . $ext))) {
					$isDocumentFormValid = false;
					$errorMessages['messages']['document_file']['required'] = 'Uploaded file could not saved. Please try again.';
				};
			}
			
            if ($isDocumentFormValid) {
                
                //insert the medication data in to the DB
				$documentUpdateData['file_name'] = $newFileName . '.' . $ext;
				
                $insertStatus = $this
                                    ->getDocumentTable()
                                    ->saveDocument($documentUpdateData, $userId);
                if ($insertStatus) {
                    
                    $encodeArray['resourceid']  = $documentUpdateData['parent_resource_id'];
                    $encodeArray['tab_type']    = self :: DOCUMENT_RESOURCE;
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $this->redirect()->toRoute(
                                                'secure',
                                                array(
                                                    'action'        => 'index', 
                                                    'resourceid'    => $this->createEncodedString($encodeArray)
                            )
                    );
                }
            }
            
            // populate the form with the retrieved data
            return array(
                    'documentAddData'     =>  $documentUpdateData,
                    'messages'            =>  $errorMessages,
                    'resourceArray'       =>  $resourceArray,
                    'count'               =>  $count,
                    'breadCrumbString'    =>  $breadcrumbString,
                    'tabs'                =>  $tabsArray,
                    'tabType'             =>  $tabType
            );
        }
    }
    
    /**
     * Function to check it the given mime type is valid i.e. it should belong to 
     * one of the value present in the constant file.
     * 
     * @param type $mimeType
     * @return boolean
     */
    public function isValidMimeType($mimeType)
    {
        
        if ($mimeType) {
            
             $config = $this->getServiceLocator()->get('Config');
             
            if (in_array($mimeType, $config['mimetypes'])) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Function to check if the given file type is valid i.e. it should belong to
     * one of the value present in the constant file.
     * 
     * @param type $fileType
     * @return boolean
     */
    public function isValidFileType($fileType)
    {
        
        if ($fileType) {
            
             $config = $this->getServiceLocator()->get('Config');
             
            if (in_array(strtolower($fileType), $config['filetypes'])) {
                
                return true;
            }
        }
        return false;
    }
    
    /**
     * Function to list the tasks present under a folder
     * 
     * @param  array $inputArray
     * @return array
     */
    public function listTasks($inputArray)
    {
        // get the service locater instantce for the convert params service.
        $convert = $this->getServiceLocator()->get('ConvertParams');
        
        // collect tasks available for the given resource id.
        $taskRecords = $this->getTaskTable()->getAllTasks($inputArray['resourceid']);
        
        $encodeArray = array();
        $encodeArray['resourceid']          = $inputArray['resourceid'];
        $encodeArray['tab_type']            = self :: TASK_RESOURCE;
        $encodeArray['action']              = self :: UPDATE_ACTION;
        
        // Create the required encoded string.
        $taskRecordsToSend = array();
        if (count($taskRecords) > 0) {

            $count = 0;
            foreach ($taskRecords as $taskRecord) {

                $taskRecordsToSend[$count]['task_description'] = $taskRecord['task_description'];
                $taskRecordsToSend[$count]['due_date'] = $taskRecord['due_date'];
                $taskRecordsToSend[$count]['priority'] = $taskRecord['priority'];
                $taskRecordsToSend[$count]['person_responsibile'] = $taskRecord['person_responsibile'];
                $taskRecordsToSend[$count]['is_completed'] = $taskRecord['is_completed'];
                $taskRecordsToSend[$count]['taskid'] = $convert->encode($taskRecord['taskid']);
                $taskRecordsToSend[$count]['taskid_not_encoded'] = $taskRecord['taskid'];
                $taskRecordsToSend[$count]['resourceid'] = $convert->encode($taskRecord['resourceid']);
                
                $encodeArray['child_resourceid']    = $taskRecord['resourceid'];
                $arrayEncodedString = $this->createEncodedString($encodeArray);
                
                $taskRecordsToSend[$count]['link'] = $arrayEncodedString;
                
                $count++;
            }
        }

        return array(
            'taskRecords'           =>  $taskRecordsToSend
        );
    }
    
    /**
     * Function to list all the medication resources present under the current folder.
     * 
     * @param array $inputArray
     * @return type
     */
    public function listMedications($inputArray)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
        // collect medications available for the given resource id.
        $medicationRecords = $this->getMedicationTable()->getAllMedications($inputArray['resourceid']);
        
        $encodeArray = array();
        $encodeArray['resourceid']          = $inputArray['resourceid'];
        $encodeArray['tab_type']            = self :: MEDICATION_RESOURCE;
        $encodeArray['action']              = self :: UPDATE_ACTION;

        $medicationRecordsToSend = array();
        if (count($medicationRecords) > 0) {

            $count = 0;
            foreach ($medicationRecords as $medicationRecord) {

                $medicationRecordsToSend[$count]['med_name'] = $medicationRecord['med_name'];
                $medicationRecordsToSend[$count]['dosage'] = $medicationRecord['dosage'];
                $medicationRecordsToSend[$count]['dosage_frequency'] = $medicationRecord['dosage_frequency'];
                $medicationRecordsToSend[$count]['med_notes'] = $medicationRecord['med_notes'];
                $medicationRecordsToSend[$count]['medicationid'] = $convert
                                                                        ->encode($medicationRecord['medicationid']);
                $medicationRecordsToSend[$count]['resourceid'] = $convert->encode($medicationRecord['resourceid']);
                $medicationRecordsToSend[$count]['medicationid_not_encoded'] = $medicationRecord['medicationid'];
                
                $encodeArray['child_resourceid']    = $medicationRecord['resourceid'];
                $arrayEncodedString = $this->createEncodedString($encodeArray);
                
                $medicationRecordsToSend[$count]['link'] = $arrayEncodedString;
                $count++;
            }
        }

        return array(
            'medicationRecords'     =>  $medicationRecordsToSend
        );
    }
    
    /**
     * Function to list all the contact resources present under the present folder.
     * 
     * @param type $inputArray
     */
    public function listContacts($inputArray)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
        $contactRecords = $this->getContactTable()->getAllContacts($inputArray['resourceid']);
        
        
        $encodeArray = array();
        $encodeArray['resourceid']          = $inputArray['resourceid'];
        $encodeArray['tab_type']            = self :: CONTACT_RESOURCE;
        $encodeArray['action']              = self :: UPDATE_ACTION;
        $contactRecordsToSend = array();
        if (count($contactRecords) > 0) {

            $count = 0;
            foreach ($contactRecords as $contactRecord) {

                $contactRecordsToSend[$count]['contact_name'] = $contactRecord['contact_name'];
                $contactRecordsToSend[$count]['business_name'] = $contactRecord['business_name'];
                $contactRecordsToSend[$count]['email_address1'] = $contactRecord['email_address1'];
                $contactRecordsToSend[$count]['mobile_number'] = $contactRecord['mobile_number'];
                $contactRecordsToSend[$count]['work_number'] = $contactRecord['work_number'];
                $contactRecordsToSend[$count]['contactid'] = $convert->encode($contactRecord['contactid']);
                $contactRecordsToSend[$count]['not_encoded_contactid'] = $contactRecord['contactid'];
                $contactRecordsToSend[$count]['resourceid'] = $convert->encode($contactRecord['resourceid']);
                
                $encodeArray['child_resourceid']    = $contactRecord['resourceid'];
                $arrayEncodedString = $this->createEncodedString($encodeArray);
                
                $contactRecordsToSend[$count]['link'] = $arrayEncodedString;
                $count++;
            }
        }

        return array(
            'contactRecords'     =>  $contactRecordsToSend
        );
    }
    
    /**
     * Function to list all the documents uploaded.
     * 
     * @param type $inputArray
     */
    public function listDocuments($inputArray)
    {
        $convert = $this->getServiceLocator()->get('ConvertParams');
        $documentRecords = $this->getDocumentTable()->getAllDocuments($inputArray['resourceid']);
        
        $encodeArray = array();
        $encodeArray['resourceid']          = $inputArray['resourceid'];
        $encodeArray['tab_type']            = self :: DOCUMENT_RESOURCE;
        $encodeArray['action']              = self :: UPDATE_ACTION;

        $documentRecordsToSend = array();
        if (count($documentRecords) > 0) {

            $count = 0;
            foreach ($documentRecords as $documentRecord) {

                $documentRecordsToSend[$count]['documentid'] = $convert->encode($documentRecord['documentid']);
                $documentRecordsToSend[$count]['document_name'] = $documentRecord['actual_file_name'];
                $documentRecordsToSend[$count]['file_name'] = $documentRecord['file_name'];
                $documentRecordsToSend[$count]['document_description'] = $documentRecord['document_description'];
                $documentRecordsToSend[$count]['documentid_not_encoded'] = $documentRecord['documentid'];
                $documentRecordsToSend[$count]['resourceid'] = $convert->encode($documentRecord['resourceid']);

                $encodeArray['child_resourceid']    = $documentRecord['resourceid'];
                $arrayEncodedString = $this->createEncodedString($encodeArray);
                
                $documentRecordsToSend[$count]['link'] = $arrayEncodedString;
                $count++;
            }
        }

        return array(
            'documentRecords'     =>  $documentRecordsToSend
        );
    }
    
    /**
     * Get the array for resource to list them in the left menu.
     * 
     * @param int $userId
     * @param int $resourceId
     * @return boolean
     */
    public function getResourceArray($userId, $resourceId = 0)
    {   
        if ($userId) {
            // Code for left menu listing
            $containersUnderFolder = $this
                                        ->getResourceTable()
                                        ->getAllResourcesUnderRoot($userId);

            $resourceArray = array();
            $currentCount = '';
            if (count($containersUnderFolder) > 0) {

                $upperCount = 0;
                foreach ($containersUnderFolder as $containerFolder) {

                    if ($containerFolder['type'] == self :: PERSON_RESOURCE 
                            || $containerFolder['type'] == self :: PROVIDER_RESOURCE ) {
                        
                        if ($resourceId == $containerFolder['resourceid']) {
                            $currentCount = $upperCount;
                        }
                        $upperCount++;

                        $encodeArray = array();
                        $encodeArray['resourceid']  = $containerFolder['resourceid'];

                        $containerName = '<a href="/secure/' . $this->createEncodedString($encodeArray) . '">';
                        
                        if($containerFolder['person_owner_id'] == $userId) {
                            $containerName .= 'My Profile';
                        }
                        else {
                            if ($containerFolder['ownerid'] != $userId) {
                                $containerName .= $containerFolder['description'] . ' (Shared)';
                            }
                            else {
                                $containerName .= $containerFolder['description'];
                            }
                        }
                        
                        $containerName .= '</a>';

                        $resourcesUnderContainer = $this
                                                        ->getResourceTable()
                                                        ->getResourcesUnderContainer($containerFolder['resourceid'], $userId);

                        if (($containerFolder['type'] == self :: PERSON_RESOURCE 
                            || $containerFolder['type'] == self :: PROVIDER_RESOURCE)) {
                            if (count($resourcesUnderContainer) > 0) {

                                $count = 0;
                                foreach ($resourcesUnderContainer as $resource) {
                                    
                                    $encodeArray = array();
                                    $encodeArray['resourceid'] = $resource['resourceid'];

                                    $containerName1 = '<a href="/secure/' . $this->createEncodedString($encodeArray) . '">';
                                    
                                    if (($resource['ownerid'] == $userId) 
                                            || ($this->getUserResourceTable()->isResourceShared($userId, $resource['resourceid']))) {
                                        $resourceArray[$containerName][$count] = $containerName1 
                                            . $resource['description'] . '</a>';
                                        $count++;
                                    }
                                }
                                
                                // If there is no element in the array.
                                if ($count == 0) {
                                    $resourceArray[$containerName] = '';
                                }
                            }
                            else {
                                $resourceArray[$containerName] = '';   
                            }
                        }
                        else {
                            $resourceArray[$containerName] = '';   
                        }
                    }
                }
                
                return array(
                    'resourceArray' => $resourceArray, 
                    'count'         => $currentCount
                );
            }
        }
        return false;
    }
    
    /**
     * Get array for the sharing of resources.
     * 
     * @param type $userId
     * @param type $resourceId (default) 0
     * @param type $emailAddress (default) NULL string
     * @return boolean
     */
    public function getResourceSharingArray($resourceId = 0, $emailAddress = '', $setDataArray = array())
    {
        $isEmailIdActive = $this->getUsersTable()->ifEmailIdExists($emailAddress);
        
        if ($isEmailIdActive) {
            
            // collect the userId for the given email id.
            $userRecord = $this->getUsersTable()->getUserIdbyEmail($emailAddress);
        }
        
        $resourceArray = array();
        // preparing the checkbox tree for a single resource.
        if ($resourceId) {
            
            // Code for left menu listing
            $profileResourceRecord = $this
                                        ->getResourceTable()
                                        ->getResourceTypeById($resourceId);
            
            if ($profileResourceRecord) {
                
                if ($profileResourceRecord['type'] == self :: PERSON_RESOURCE 
                        || $profileResourceRecord['type'] == self :: PROVIDER_RESOURCE) {

                    $convert = $this->getServiceLocator()->get('ConvertParams');
                    $encodeArray = array(
                        'resourceid'    => $resourceId
                    );
                    $arrayConvertedString = $convert->arrayToString($encodeArray);
                    $encryptedResourceId = $convert->paramEncrypt($arrayConvertedString);
                    
                    if ($isEmailIdActive) {
                        
                        // check if the permission exist in the user_resource table.
                        $permStatus = $this
                                        ->getUserResourceTable()
                                        ->isAnyPermissionExist($userRecord['userid'], $resourceId);
                    }
                    else {
                        // check if the permission exist in the user_resource_temp table.
                        $permStatus = $this
                                        ->getUsersPermTempTable()
                                        ->checkIfPermissionExist($emailAddress, $resourceId);
                    }
                    
                    if (($permStatus) && $permStatus['perm_type'] == self :: VIEW_PERM) {
                        $containerName = $profileResourceRecord['description'] 
                                   . "<br /><input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                   . $encryptedResourceId . "]\" value=\"view\" /> View<br />";
                    }
                    else if ((count($setDataArray) > 0) && in_array(array('permType' => self :: VIEW_PERM, 'resourceId' => $encryptedResourceId), $setDataArray)) {
                        
                        $containerName = $profileResourceRecord['description'] 
                                   . "<br /><input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                   . $encryptedResourceId . "]\" value=\"view\" /> View<br />";
                    }
                    else {
                        $containerName = $profileResourceRecord['description'] 
                                   . "<br /><input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                   . $encryptedResourceId . "]\" value=\"view\" /> View<br />";
                    }

                    if (($permStatus) && $permStatus['perm_type'] == self :: EDIT_PERM) {
                        $containerName .= "<input type=\"radio\" checked=\"checked\" name=\"perm[" 
                            . $encryptedResourceId . "]\" value=\"edit\" /> Edit";
                    }
                    else if ((count($setDataArray) > 0) && in_array(array('permType' => self :: EDIT_PERM, 'resourceId' => $encryptedResourceId), $setDataArray)) {
                        $containerName .= "<input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                   . $encryptedResourceId . "]\" value=\"edit\" /> Edit<br />";
                    }
                    else {
                        $containerName .= "<input type=\"radio\" name=\"perm["
                            . $encryptedResourceId . "]\" value=\"edit\" /> Edit";
                    }
                    
                    $resourcesUnderContainer = $this
                                                    ->getResourceTable()
                                                    ->getResourcesUnderContainer($profileResourceRecord['resourceid'], $userId);
                    
                    if ($profileResourceRecord['type'] == self :: PERSON_RESOURCE 
                            || $profileResourceRecord['type'] == self :: PROVIDER_RESOURCE) {
                        
                        if (count($resourcesUnderContainer) > 0) {

                            $count = 0;
                            foreach ($resourcesUnderContainer as $resource) {

                                $encodeArray = array(
                                    'resourceid'    => $resource['resourceid']
                                );
                                $arrayConvertedString = $convert->arrayToString($encodeArray);
                                $encryptedResourceId = $convert->paramEncrypt($arrayConvertedString);
                                
                                if ($isEmailIdActive) {
                        
                                    // check if the permission exist in the user_resource table.
                                    $permStatus = $this
                                                    ->getUserResourceTable()
                                                    ->isAnyPermissionExist($userRecord['userid'], $resource['resourceid']);
                                }
                                else {
                                    // check if the permission exist in the user_resource_temp table.
                                    $permStatus = $this
                                                    ->getUsersPermTempTable()
                                                    ->checkIfPermissionExist($emailAddress, $resource['resourceid']);
                                }
                                
                                $isNoAccessChecked = TRUE;
                                
                                if ($permStatus && ($permStatus['perm_type'] == self :: VIEW_PERM)) {
                                    $isNoAccessChecked = FALSE;
                                    $resourceArray[$containerName][$count] = $resource['description'] 
                                               . "<br /><input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                               . $encryptedResourceId . "]\" value=\"view\" /> View<br />";
                                }
                                else if ((count($setDataArray) > 0) && in_array(array('permType' => self :: VIEW_PERM, 'resourceId' => $encryptedResourceId), $setDataArray)) {
                                    $isNoAccessChecked = FALSE;
                                    $resourceArray[$containerName][$count] = $resource['description']  
                                               . "<br /><input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                               . $encryptedResourceId . "]\" value=\"view\" /> View<br />";
                                }
                                else {
                                    $resourceArray[$containerName][$count] = $resource['description'] 
                                               . "<br /><input type=\"radio\" name=\"perm[" 
                                               . $encryptedResourceId . "]\" value=\"view\" /> View<br />";
                                }

                                if ($permStatus && ($permStatus['perm_type'] == self :: EDIT_PERM)) {
                                    $isNoAccessChecked = FALSE;
                                    $resourceArray[$containerName][$count] .= "<input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                        . $encryptedResourceId . "]\" value=\"edit\" /> Edit";
                                }
                                else if ((count($setDataArray) > 0) && in_array(array('permType' => self :: EDIT_PERM, 'resourceId' => $encryptedResourceId), $setDataArray)) {
                                    
                                    $isNoAccessChecked = FALSE;
                                    $resourceArray[$containerName][$count] .= "<input type=\"radio\" checked=\"checked\" name=\"perm[" 
                                               . $encryptedResourceId . "]\" value=\"edit\" /> Edit<br />";
                                    
                                }
                                else {
                                    $resourceArray[$containerName][$count] .= "<input type=\"radio\" name=\"perm["
                                        . $encryptedResourceId . "]\" value=\"edit\" /> Edit";
                                }
                                
                                if ($isNoAccessChecked) {
                                    $resourceArray[$containerName][$count] .= "<br /><input type=\"radio\" name=\"perm["
                                    . $encryptedResourceId . "]\" checked=\"checked\" value=\"no_access\" /> No Access";
                                }
                                else {
                                    $resourceArray[$containerName][$count] .= "<br /><input type=\"radio\" name=\"perm["
                                    . $encryptedResourceId . "]\" value=\"no_access\" /> No Access";
                                }
                                
                                $count++;
                            }
                        }
                        else {
                            $resourceArray[$containerName] = '';
                        }
                    }
                    else {
                        $resourceArray[$containerName] = '';
                    }
                }
            }

            // return the obtained array here.
            return array(
                'resourceArray' => $resourceArray
            );
        } // end of resourceid if.
        return false;
    }
    
    /**
     * Renders the form for sharing of resources.
     *
     * @return instance of view model(send data to view).
     */
    public function sharingAction()
    {
        $isLoggedIn = 'yes';
        $emailExists = '';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
		$emailLogin = $session->offsetGet('email_login');
        
		$rowSet = $this->getPersonTable()->getUserName($userId);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] 
                    . ' ' . $rowSet['last_name'];
		}
        
        // get the instantiation of the ConvertParams class.
        $convert = $this->getServiceLocator()->get('ConvertParams');
        
        // collect the encrypted resouce id.
        $encryptedResourceId = $this->params()->fromRoute('resourceid', 0);
        
        $decryptedResourceString = $convert->paramDecrypt($encryptedResourceId);
        $resourceArray = $convert->stringToArray($decryptedResourceString);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceArray['resourceid'], $userId);
        
        // Get all the array elements for the left menu
        $leftMenuArray = $this->getResourceArray($userId, $resourceArray['resourceid']);
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'resourceArray'     => $leftMenuArray,
                    'count'             => $leftMenuArray['count'],
                    'breadCrumbString'  => $breadCrumbString,
                    'is_logged_in'      => $isLoggedIn
                )
        );
        
        // Call getResourceSharingArray function to show the Radio button tree.
        $resourceSharingArray = $this->getResourceSharingArray($resourceArray['resourceid']);
        
        $request = $this->getRequest();

        // check if the request is a post request
        if ($request->isPost()) {
            $postedData = $this->getRequest()->getPost();
            
            if ($postedData['cancel'] == 'Cancel') {
                
                /* we will think about this part in the near future */
                return $this->redirect()->toRoute('secure');
			}
            
            $addPeopleInsertData = array();
            $isAddPeopleFormValid = TRUE;
            
            // validation for the email address.
            $addPeopleInsertData['email_address'] = $this->filterFormElements($postedData['email_address']);
            if ($addPeopleInsertData['email_address'] == '') {
                
				$isAddPeopleFormValid = FALSE;
				$errorMessages['messages']['emailAddress']['required'] = 'Please enter Email address.';
			}
            else {
                $emailvalidator = new EmailAddress();
                if (!$emailvalidator->isValid($addPeopleInsertData['email_address'])) {
                    $isAddPeopleFormValid = FALSE;
                    $errorMessages['messages']['emailAddress']['invalid'] = 'The entered email address is in invalid format.';
                }
                else {
                    
                    $addPeopleInsertData['email_address'] = strtolower($addPeopleInsertData['email_address']);
                    
                    // check if the email id exist in the system
                    $emailExists = $this->getUsersTable()->ifEmailIdExists($addPeopleInsertData['email_address']);
                    
                    // if the entered email address exist in the system.
                    if ($emailExists) {
                        
                        if ($addPeopleInsertData['email_address'] == $emailLogin) {
                            $isAddPeopleFormValid = FALSE;
                            $errorMessages['messages']['emailAddress']['invalid'] = 'Resource cannot be shared with owner.';
                        }
                        else {
                            // get user id for the given email address.
                            $userRecord = $this
                                            ->getUsersTable()
                                            ->getUserbyEmail($addPeopleInsertData['email_address']);

                            $addPeopleInsertData['userId']     = $userRecord['userid'];
                        }
                    }
                }
            }
            
            if (count($postedData['perm']) > 0) {
            
                // prepare the permission array for setting the radio buttons.
                $setDataPermArray = array();
                $count = 0;
                foreach ($postedData['perm'] AS $resourceid => $permissionType) {
                    
                    $setDataPermArray[$count]['permType']   = $permissionType;
                    $setDataPermArray[$count]['resourceId'] = $resourceid;
                    $count++;
                }
                
                $resourceSharingArray = $this
                                            ->getResourceSharingArray($resourceArray['resourceid'], $addPeopleInsertData['email_address'], $setDataPermArray);
            }
            else {
                $isAddPeopleFormValid = FALSE;
				$errorMessages['messages']['checkBoxTree']['required'] = 'Please click on the radio buttons to assign permission.';
            }
            
            // if the add people form is valid.
            if ($isAddPeopleFormValid) {
                
                // get resource id of the email id entered.
                foreach ($postedData['perm'] AS $resourceid => $permissionType) {
                        
                    $addPeopleInsertData['permType']    = $permissionType;

                    // decrypt the resource ids
                    $decryptedResourceString = $convert->paramDecrypt($resourceid);
                    $resourcesArray = $convert->stringToArray($decryptedResourceString);
                    $addPeopleInsertData['resourceId']  = $resourcesArray['resourceid'];

                    // before inserting permission check whether permission exist for
                    // same resource id  and same user id.
                    if ($emailExists) {
                        
                        if ($this->getUserResourceTable()->isAnyPermissionExist($addPeopleInsertData['userId'], $addPeopleInsertData['resourceId'])) {
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                $this->getUserResourceTable()->updatePermission($addPeopleInsertData, $userId);
                            }
                            else if ($permissionType == self :: NO_ACCESS_PERM) {
                                // delete the assigned permission
                                $this->getUserResourceTable()->deleteUserPermissionByUserId($addPeopleInsertData);
                            }
                        }
                        else {  // if the permission doesn't pre-exist in the system.
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                $this->getUserResourceTable()->savePermission($addPeopleInsertData, $userId);
                            }
                        }
                    }
                    else {
                        
                        $userData = array(
                                    'fk_resourceid'     => $addPeopleInsertData['resourceId'],
                                    'fk_email_login'    => $addPeopleInsertData['email_address'],
                                    'perm_type'         => $permissionType
                        );
                        
                        // first check if any permission exist
                        if ($this->getUsersPermTempTable()->checkIfPermissionExist($addPeopleInsertData['email_address'], $addPeopleInsertData['resourceId'])) {
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                               
                                $userData['userid_updated'] = $userId;
                                $this->getUsersPermTempTable()->updateTempUserPerm($userData);
                            }
                            else if ($permissionType == self :: NO_ACCESS_PERM) {
                                // delete the assigned permission
                                $this->getUsersPermTempTable()->deleteTempPerm($userData);
                            }
                        }
                        else {  // if permission does not exist in the user perm temp table
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                
                                $userData['created']        = date('Y-m-d H:i:s');
                                $userData['userid_created'] = $userId;
                                $this->getUsersPermTempTable()->createTempUser($userData);
                            }
                        }
                    }
                }   // end of foreach loop
                    
                if ($emailExists) {
                    
                    $tempArray = array('resourceid' => $resourceArray['resourceid']);
                
                    $decryptedResourceString = $convert->arrayToString($tempArray);
                    $tempString = $convert->paramencrypt($decryptedResourceString);

                    return $this->redirect()->toRoute(
                                                    'secure', 
                                                    array(
                                                        'action'        => 'permission-listing', 
                                                        'resourceid'    => $tempString
                                                    )
                    );
                }
                else {
                    $emailExistsInactive = $this
                                                ->getUsersTable()
                                                ->ifEmailIdExistsInactive($addPeopleInsertData['email_address']);
                        
                    if (!$emailExistsInactive) {
                        $convert = $this->getServiceLocator()->get('SendEmail');
                        $config = $this->getServiceLocator()->get('Config');
                        
                        $arrayToEncode = array(
                            'emailId'   => $addPeopleInsertData['email_address']
                        );
                        
                        $encodedEmailAddress = $this->createEncodedString($arrayToEncode);

                        $inviteLink = $config['mailer']['registration_mail_constants']['registration_index_url'] . $encodedEmailAddress;

                        // prepare the view for activation email template
                        $mailData = array(
                           'invitation_link'    =>      $inviteLink,
                           'sharer_name'        =>      $rowSet['first_name'] . ' ' . $rowSet['last_name']
                        );

                        $this->renderer = $this->getServiceLocator()->get('ViewRenderer');
                        $content = $this->renderer->render('invitation-email/view', $mailData);

                        // create the link for activation code.
                        $smtpOptions = $config['mailer']['smtp_options'];
                        $htmlBody = $content;
                        $subject = $config['mailer']['invitation_mail_constants']['subject'];
                        $fromName = $config['mailer']['invitation_mail_constants']['from_name'];
                        $fromEmail = $config['mailer']['invitation_mail_constants']['from_email'];
                        $to = $addPeopleInsertData['email_address'];

                        // $htmlBody, $subject, $fromName, $fromEmail, $to
                        $convert->sendEmail($smtpOptions, $htmlBody, $subject, $fromName, $fromEmail, $to);
                        $view = new ViewModel(array('inactive' => 'no'));
                    }
                    else {
                        $view = new ViewModel(array('inactive' => 'yes'));
                    }

                    $view->setTemplate('invitation/view'); // path to phtml file under view folder
                    return $view;
                }   
            }
            
            // if add people form not valid then show the required validation messages.
            $addPeopleFormArray = array(
                'count'                 => $leftMenuArray['count'],
                'resourceSharingArray'  => $resourceSharingArray['resourceArray'],
                'setdata'               => $addPeopleInsertData,
                'encryptedResourceId'   => $encryptedResourceId,
                'breadCrumbString'      => $breadCrumbString,
                'messages'              => $errorMessages
            );

            // load the Add people view/template.
            $view = new ViewModel($addPeopleFormArray);
            $view->setTemplate('secure/add-people');
            return $view;
        }
        
        /* Get all the container resources for which the currently logged in
            user is the owner */
        $addPeopleFormArray = array(
            'resourceArray'         => $leftMenuArray['resourceArray'],
            'resourceSharingArray'  => $resourceSharingArray['resourceArray'],
            'breadCrumbString'      => $breadCrumbString,
            'count'                 => $leftMenuArray['count'],
        );
        
		// load the Add people view/template.
        $view = new ViewModel($addPeopleFormArray);
        $view->setTemplate('secure/add-people');
        return $view;
    }
    
    /**
     * 
     * @return \Zend\View\Model\ViewModel
     */
    public function permissionListingAction()
    {
        $isLoggedIn = 'yes';
        $emailExists = '';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        $emailLogin = $session->offsetGet('email_login');
        
		$rowSet = $this->getPersonTable()->getUserName($userId);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
		}
        
        $encryptedResourceId = $this->params()->fromRoute('resourceid', 0);
        
        $convert = $this->getServiceLocator()->get('ConvertParams');
        $decryptedResourceString = $convert->paramDecrypt($encryptedResourceId);
        $resourceArray = $convert->stringToArray($decryptedResourceString);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceArray['resourceid'], $userId);
        
        $getAllShares = array();
        $count = 0;
        $getAllActiveShares = $this->getUserResourceTable()->getAllShares($resourceArray['resourceid']);
        
        foreach ($getAllActiveShares as $getAllActiveShare) {
            
            // collect the email address from the user id of the record.
            $userRecord = $this->getUsersTable()->getUser($getAllActiveShare['fk_userid']);
            
            $encodeArray = array(
                                'resourceid'    => $getAllActiveShare['fk_resourceid'],
                                'emailAddress'  => $userRecord['email_login']
            );
            $arrayConvertedString = $convert->arrayToString($encodeArray);
            $encryptedResourceIdFirst = $convert->paramEncrypt($arrayConvertedString);
            
            $getAllShares[$count]['user_resourceid'] = $getAllActiveShare['user_resourceid'];
            $getAllShares[$count]['encrypted_resourceid'] = $encryptedResourceIdFirst;
            
            if ($getAllActiveShare['created'] == '0000-00-00 00:00:00') {
                $getAllShares[$count]['created'] = '';
            }
            else {
                $getAllShares[$count]['created'] = date("m/d/Y H:i:s", strtotime($getAllActiveShare['created']));
            }
            
            $usersRecord = $this->getUsersTable()->getUserbyEmail($getAllActiveShare['email_login']);
            
            if ($usersRecord) {
                $personsRecord = $this->getUsersTable()->getUser($usersRecord['userid']);
            }
            
            $getAllShares[$count]['email_login'] = $personsRecord['first_name'] . ' ' . $personsRecord['last_name'];
            
            if ($getAllActiveShare['perm_type'] == self :: EDIT_PERM) {
                $getAllShares[$count]['perm_type'] = 'Edit';
            }
            
            if ($getAllActiveShare['perm_type'] == self :: VIEW_PERM) {
                $getAllShares[$count]['perm_type'] = 'Display';
            }
            
            $getAllShares[$count]['is_active'] = true;
            $count++;
        }
        
        $getAllPendingShares = $this
                                    ->getUsersPermTempTable()
                                    ->getAllPendingShares($resourceArray['resourceid']);
        
        foreach ($getAllPendingShares as $getAllPendingShare) {
            $getAllShares[$count]['users_permisson_tempid'] = $getAllPendingShare['users_permisson_tempid'];
            $getAllShares[$count]['fk_resourceid'] = $getAllPendingShare['fk_resourceid'];
            
            $encodeArray = array(
                                'resourceid'    => $getAllPendingShare['fk_resourceid'],
                                'emailAddress'  => $getAllPendingShare['fk_email_login']
            );
            $arrayConvertedString = $convert->arrayToString($encodeArray);
            $encryptedResourceIdSecond = $convert->paramEncrypt($arrayConvertedString);
            $getAllShares[$count]['encrypted_resourceid'] = $encryptedResourceIdSecond;
            
            $getAllShares[$count]['email_login'] = $getAllPendingShare['fk_email_login'];
            
            if ($getAllPendingShare['perm_type'] == self :: EDIT_PERM) {
                $getAllShares[$count]['perm_type'] = 'Edit - Pending';
            }
            
            if ($getAllPendingShare['perm_type'] == self :: VIEW_PERM) {
                $getAllShares[$count]['perm_type'] = 'Display - Pending';
            }
            
            if ($getAllPendingShare['created'] == '0000-00-00 00:00:00') {
                $getAllShares[$count]['created'] = '';
            }
            else {
                $getAllShares[$count]['created'] = date("m/d/Y H:i:s", strtotime($getAllPendingShare['created']));
            }
            
            $getAllShares[$count]['is_active'] = false;
            $count++;
        }
        
        // get the left menu listing.
        $leftMenuArray = $this->getResourceArray($userId, $resourceArray['resourceid']);
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'resourceArray'     => $leftMenuArray,
                    'count'             => $leftMenuArray['count'],
                    'breadCrumbString'  => $breadCrumbString,
                    'is_logged_in'      => $isLoggedIn
                )
        );
        
        return new ViewModel(
                array(
                    'listAllShares'         => $getAllShares,
                    'count'                 => $leftMenuArray['count'],
                    'encryptedResourceId'   => $encryptedResourceId,
                    'ownerEmailId'          => $emailLogin,
                    'ownerFullName'         => $rowSet['first_name'] . ' ' . $rowSet['last_name']
                )
        );
    }
    
    /**
     * Funtion to edit permissions which are assigned for existing and non-existing users.
     * 
     * @return instance of view model(send data to view).
     */
    public function editPermissionAction()
    {
        $isLoggedIn = 'yes';
        $emailExists = '';

        // check if the user is already logged in.
        if (!$this->getAuthService()->hasIdentity()) {
            $isLoggedIn = 'no';
            return $this->redirect()->toRoute('login');
        }
        
        $session = $this->intializeSessionInstance();
        $userId = $session->offsetGet('user_id');
        $emailLogin = $session->offsetGet('email_login');

        $rowSet = $this->getPersonTable()->getUserName($userId);
        if ($rowSet) {
            $this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
        }

        $convert = $this->getServiceLocator()->get('ConvertParams');
        $encryptedResourceId = $this->params()->fromRoute('resourceid', 0);
        
        $decryptedResourceString = $convert->paramDecrypt($encryptedResourceId);
        $resourceArray = $convert->stringToArray($decryptedResourceString);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceArray['resourceid'], $userId);
        
        $leftMenuArray = $this->getResourceArray($userId, $resourceArray['resourceid']);
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'resourceArray'     => $leftMenuArray,
                    'count'             => $leftMenuArray['count'],
                    'breadCrumbString'  => $breadCrumbString,
                    'is_logged_in'      => $isLoggedIn
                )
        );
            
        $resourceSharingArray = $this
                                    ->getResourceSharingArray(
                                                        $resourceArray['resourceid'],
                                                        $resourceArray['emailAddress']
        );
        
        $request = $this->getRequest();

        // check if the request is a post request
        if ($request->isPost()) {
            $postedData = $this->getRequest()->getPost();
            
            if ($postedData['cancel'] == 'Cancel') {
                
                /* we will think about this part in the near future */
                return $this->redirect()->toRoute('secure');
			}
            
            $addPeopleUpdateData = array();
            $isAddPeopleFormValid = TRUE;
            
            // validation for the email address.
            $addPeopleUpdateData['email_address'] = $this->filterFormElements($postedData['email_address']);
            if ($addPeopleUpdateData['email_address'] == '') {
                
				$isAddPeopleFormValid = FALSE;
				$errorMessages['messages']['emailAddress']['required'] = 'Please enter Email address.';
			}
            else {
                $emailvalidator = new EmailAddress();
                if (!$emailvalidator->isValid($addPeopleUpdateData['email_address'])) {
                    $isAddPeopleFormValid = FALSE;
                    $errorMessages['messages']['emailAddress']['invalid'] = 'The entered email address is in invalid format.';
                }
                else {
                    $addPeopleUpdateData['email_address'] = strtolower($addPeopleUpdateData['email_address']);
                    
                    // check if the email id exist in the system
                    $emailExists = $this->getUsersTable()->ifEmailIdExists($addPeopleUpdateData['email_address']);
                    
                    // if the entered email address exist in the system.
                    if ($emailExists) {
                        
                        if ($addPeopleUpdateData['email_address'] == $emailLogin) {
                            $isAddPeopleFormValid = FALSE;
                            $errorMessages['messages']['emailAddress']['invalid'] = 'Resource cannot be shared with owner.';
                        }
                        else {
                            // get user id for the given email address.
                            $userRecord = $this
                                            ->getUsersTable()
                                            ->getUserbyEmail($addPeopleUpdateData['email_address']);

                            $addPeopleUpdateData['userId']     = $userRecord['userid'];
                        }
                    }
                }
            }
            
            if (count($postedData['perm']) > 0) {
            
                // prepare the permission array for setting the radio buttons.
                $setDataPermArray = array();
                $count = 0;
                foreach ($postedData['perm'] AS $resourceid => $permissionType) {
                    
                    $setDataPermArray[$count]['permType']   = $permissionType;
                    $setDataPermArray[$count]['resourceId'] = $resourceid;
                    $count++;
                }
                
                $resourceSharingArray = $this
                                            ->getResourceSharingArray(
                                                                $resourceArray['resourceid'],
                                                                $addPeopleUpdateData['email_address'],
                                                                $setDataPermArray
                );
            }
            else {
                $isAddPeopleFormValid = FALSE;
				$errorMessages['messages']['checkBoxTree']['required'] = 'Please click on the radio buttons to assign permission.';
            }
            
            if ($isAddPeopleFormValid) {
                
                // get resource id of the email id entered.
                foreach ($postedData['perm'] AS $resourceid => $permissionType) {
                        
                    $addPeopleUpdateData['permType']    = $permissionType;

                    // decrypt the resource ids
                    $decryptedResourceString = $convert->paramDecrypt($resourceid);
                    $resourcesArray = $convert->stringToArray($decryptedResourceString);
                    $addPeopleUpdateData['resourceId']  = $resourcesArray['resourceid'];

                    // before inserting permission check whether permission exist for
                    // same resource id  and same user id.
                    if ($emailExists) {
                        
                        if ($this->getUserResourceTable()->isAnyPermissionExist($addPeopleUpdateData['userId'], $addPeopleUpdateData['resourceId'])) {
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                $this->getUserResourceTable()->updatePermission($addPeopleUpdateData, $userId);
                            }
                            else if ($permissionType == self :: NO_ACCESS_PERM) {
                                // delete the assigned permission
                                $this->getUserResourceTable()->deleteUserPermissionByUserId($addPeopleUpdateData);
                            }
                        }
                        else {  // if the permission doesn't pre-exist in the system.
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                $this->getUserResourceTable()->savePermission($addPeopleUpdateData, $userId);
                            }
                        }
                    }
                    else {
                        
                        $userData = array(
                                    'fk_resourceid'     => $addPeopleUpdateData['resourceId'],
                                    'fk_email_login'    => $addPeopleUpdateData['email_address'],
                                    'perm_type'         => $permissionType
                        );
                        
                        // first check if any permission exist
                        if ($this->getUsersPermTempTable()->checkIfPermissionExist($addPeopleUpdateData['email_address'], $addPeopleUpdateData['resourceId'])) {
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                
                                $userData['userid_updated'] = $userId;
                                $this->getUsersPermTempTable()->updateTempUserPerm($userData);
                            }
                            else if ($permissionType == self :: NO_ACCESS_PERM) {
                                // delete the assigned permission
                                $this->getUsersPermTempTable()->deleteTempPerm($userData);
                            }
                        }
                        else {  // if permission does not exist in the user perm temp table
                            
                            if (($permissionType == self :: EDIT_PERM) || ($permissionType == self :: VIEW_PERM)) {
                                
                                $userData['created']        = date('Y-m-d H:i:s');
                                $userData['userid_created'] = $userId;
                                $this->getUsersPermTempTable()->createTempUser($userData);
                            }
                        }
                    }
                }   // end of foreach loop
                
                $tempArray = array('resourceid' => $resourceArray['resourceid']);
                
                $decryptedResourceString = $convert->arrayToString($tempArray);
                $tempString = $convert->paramencrypt($decryptedResourceString);
                
                return $this->redirect()->toRoute(
                                                'secure', 
                                                array(
                                                    'action'        => 'permission-listing', 
                                                    'resourceid'    => $tempString
                                                )
                );
            }
            
            $editPermissionViewArray = array(
                'emailAddress'          => $resourceArray['emailAddress'],
                'resourceSharingArray'  => $resourceSharingArray['resourceArray'],
                'breadCrumbString'      => $breadCrumbString,
                'messages'              => $errorMessages,
                'count'                 => $leftMenuArray['count']
            );
            return new viewModel($editPermissionViewArray);
        }
        
        $editPermissionViewArray = array(
            'emailAddress'          => $resourceArray['emailAddress'],
            'resourceSharingArray'  => $resourceSharingArray['resourceArray'],
            'breadCrumbString'      => $breadCrumbString,
            'count'                 => $leftMenuArray['count']
        );
        return new viewModel($editPermissionViewArray);
    }
    
    /**
     * 
     */
    public function deleteSharingAction()
    {
        // get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            $postedData = $this->getRequest()->getPost();
            $userResourceId = $postedData['user_resourceid'];
            
            if ($userResourceId) {
                
                $isActive = $postedData['is_active'];
                if ($isActive == 'yes') {
                    $deleteStatus = $this->getUserResourceTable()->deleteUserPermission($userResourceId);
                }
                else if ($isActive == 'no') {
                    $deleteStatus = $this->getUsersPermTempTable()->deleteUserPermission($userResourceId);
                }
                
                if ($deleteStatus) {
                    echo 'success';
                    exit;
                }
            }
        }
        echo 'failure';
        exit;
    }

    /**
     * Function to create an encoded string from the array provided as input.
     * 
     * @param Array $inputArray
     * @return String encoded string on success and false on failure.
     */
	public function createEncodedString($inputArray)
    {
        if (is_array($inputArray)) {
            $convert = $this->getServiceLocator()->get('ConvertParams');
            $string = $convert->arrayToString($inputArray);
            return $convert->paramEncrypt($string);
        }
        return false;
    }
	
	/**
     * Renders the calendar associated with the given container
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function caredforAction()
    {
		$isLoggedIn = 'yes';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$session = $this->intializeSessionInstance();
		$user_id = $session->offsetGet('user_id');
		$root_folder_id = $session->offsetGet('root_folder_id');
        
		$rowSet = $this->getPersonTable()->getUserName($user_id);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] 
                    . ' ' . $rowSet['last_name'];
		}
        
        $breadCrumbString = 'Home &rsaquo;&rsaquo; ';
        
        $resourceArray = $this->getResourceArray($user_id);
        
        $this->getEvent()->getViewModel()->setVariables(
               array(
                   'resourceArray'     => $resourceArray,
                   'count'             => $resourceArray['count'],
                   'breadCrumbString'  => $breadCrumbString,
                   'is_logged_in'      => $isLoggedIn
               )
        );
		
		// get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
		
			// collect the posted data
			$postedData = $this->getRequest()->getPost();
			if ($postedData['cancel'] == 'Cancel') {
				return $this->redirect()->toRoute('secure');
			}
			else if ($postedData['submit'] == 'Submit') {
				$caredforInsertData = array();
				$isCaredforFormValid = true;
				$errorMessages['messages'] = array();
				
				$caredforInsertData['firstName'] = $this->filterFormElements($postedData['first_name']);
				if ($caredforInsertData['firstName'] == '') {
					$isCaredforFormValid = false;
					$errorMessages['messages']['firstName']['required'] = 'Please enter First Name.';
				}
				
				$caredforInsertData['lastName'] = $this->filterFormElements($postedData['last_name']);
				if ($caredforInsertData['lastName'] == '') {
					$isCaredforFormValid = false;
					$errorMessages['messages']['lastName']['required'] = 'Please enter Last Name.';
				}
				
				$caredforInsertData['phoneMain'] = $this->filterFormElements($postedData['phone_main']);
				$caredforInsertData['phoneMobile'] = $this->filterFormElements($postedData['phone_mobile']);
				$caredforInsertData['phoneEmergency'] = $this->filterFormElements($postedData['phone_emergency']);
				$caredforInsertData['address1'] = $this->filterFormElements($postedData['address1']);
				$caredforInsertData['address2'] = $this->filterFormElements($postedData['address2']);
				$caredforInsertData['city'] = $this->filterFormElements($postedData['city']);
				$caredforInsertData['state'] = $this->filterFormElements($postedData['state']);
				$caredforInsertData['zipcode'] = $this->filterFormElements($postedData['zipcode']);
				
				$emailvalidator = new EmailAddress();
				$caredforInsertData['emailPrimary'] = $this->filterFormElements($postedData['email_primary']);
				
				// validation for primary email
				if ($caredforInsertData['emailPrimary'] != '') {
                    
                    $caredforInsertData['emailPrimary'] = strtolower($caredforInsertData['emailPrimary']);
					if (!$emailvalidator->isValid($caredforInsertData['emailPrimary'])) {
						$isCaredforFormValid = false;
						$errorMessages['messages']['emailPrimary']['invalid'] = 'The primary email is invalid.';
					}
				}
				
				$caredforInsertData['emailSecondary'] = $this->filterFormElements($postedData['email_secondary']);
				// validation for secondary email
				if ($caredforInsertData['emailSecondary'] != '') {
                    
                    $caredforInsertData['emailSecondary'] = strtolower($caredforInsertData['emailSecondary']);
					if (!$emailvalidator->isValid($caredforInsertData['emailSecondary'])) {
						$isCaredforFormValid = false;
						$errorMessages['messages']['emailSecondary']['invalid'] = 'The secondary email is invalid.';
					}
				}
				
				$caredforInsertData['notes'] = $this->filterFormElements($postedData['notes']);
				$caredforInsertData['userId'] = $session->offsetGet('user_id');
				
				if ($isCaredforFormValid) {
					// insert data into the database for person and resource table.
					if ($this->getPersonTable()->savePerson($caredforInsertData, $root_folder_id)) {
						return $this->redirect()->toRoute('secure');
					}
				}
				
				// send validation error messages to the view.
				return new ViewModel(
					array(
						'messages'      => $errorMessages['messages'],
						'setdata'       => $caredforInsertData,
                        'resourceArray'     => $resourceArray['resourceArray'],
                        'count'             => $resourceArray['count'],
                        'breadCrumbString'  => $breadCrumbString
					)
				);
			}
		}
		return new ViewModel(
			array(
				'resourceArray'     => $resourceArray['resourceArray'],
				'count'             => $resourceArray['count'],
				'breadCrumbString'  => $breadCrumbString
			)
        );
	}
    
    /**
     * Renders the view template for a cared for container if it being shared using
     * view permission.
     *
     * @return instance of view model(send data to view).
     */
	public function caredforView($userId, $resourceRecord, $postedData = array())
	{   
        $personData = $this
                        ->getPersonTable()
                        ->getPersonInfo($resourceRecord['fk_type_recid']);
        
        return array(
            'presentData'       => $personData, 
            'resourceid'        => $resourceRecord['resourceid'],
            'resourceArray'     => $resourceArray['resourceArray'],
            'count'             => $resourceArray['count'],
            'breadCrumbString'  => $breadCrumbString
        );
    }
	
	/**
     * Renders the calendar associated with the given container
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function providerAction()
    {
		$isLoggedIn = 'yes';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$session = $this->intializeSessionInstance();
		$user_id = $session->offsetGet('user_id');
        $root_folder_id = $session->offsetGet('root_folder_id');
		$rowSet = $this->getPersonTable()->getUserName($user_id);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
		}
        
        $resourceArray = $this->getResourceArray($user_id);
        
        $breadCrumbString = $this->breadCrumbString;
        
        $this->getEvent()->getViewModel()->setVariables(
               array(
                   'resourceArray'     => $resourceArray,
                   'count'             => $resourceArray['count'],
                   'breadCrumbString'  => $breadCrumbString,
                   'is_logged_in'      => $isLoggedIn
                   
               )
        );
		
		// get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
		
			// collect the posted data
			$postedData = $this->getRequest()->getPost();
			if ($postedData['cancel'] == 'Cancel') {
				return $this->redirect()->toRoute('secure');
			}
			else if ($postedData['submit'] == 'Submit') {
				$caredforInsertData = array();
				$isCaredforFormValid = true;
				$errorMessages['messages'] = array();
				
				$caredforInsertData['providerName'] = $this->filterFormElements($postedData['provider_name']);
				if ($caredforInsertData['providerName'] == '') {
					$isCaredforFormValid = false;
					$errorMessages['messages']['providerName']['required'] = 'Please enter Provider name.';
				}
				
				$caredforInsertData['providerDesc'] = $this->filterFormElements($postedData['provider_descr']);
				$caredforInsertData['phoneMain'] = $this->filterFormElements($postedData['phone_main']);
				$caredforInsertData['phoneMobile'] = $this->filterFormElements($postedData['phone_mobile']);
				$caredforInsertData['phoneEmergency'] = $this->filterFormElements($postedData['phone_emergency']);
				$caredforInsertData['address1'] = $this->filterFormElements($postedData['address1']);
				$caredforInsertData['address2'] = $this->filterFormElements($postedData['address2']);
				$caredforInsertData['city'] = $this->filterFormElements($postedData['city']);
				$caredforInsertData['state'] = $this->filterFormElements($postedData['state']);
				$caredforInsertData['zipcode'] = $this->filterFormElements($postedData['zipcode']);
				
				$emailvalidator = new EmailAddress();
				$caredforInsertData['emailPrimary'] = $this->filterFormElements($postedData['email_primary']);
				
				// validation for primary email
				if ($caredforInsertData['emailPrimary'] != '') {
                    
                    $caredforInsertData['emailPrimary'] = strtolower($caredforInsertData['emailPrimary']);
					if (!$emailvalidator->isValid($caredforInsertData['emailPrimary'])) {
						$isCaredforFormValid = false;
						$errorMessages['messages']['emailPrimary']['invalid'] = 'The primary email is invalid.';
					}
				}
				
				$caredforInsertData['emailSecondary'] = $this->filterFormElements($postedData['email_secondary']);
				// validation for secondary email
				if ($caredforInsertData['emailSecondary'] != '') {
                    
                    $caredforInsertData['emailSecondary'] = strtolower($caredforInsertData['emailSecondary']);
					if (!$emailvalidator->isValid($caredforInsertData['emailSecondary'])) {
						$isCaredforFormValid = false;
						$errorMessages['messages']['emailSecondary']['invalid'] = 'The secondary email is invalid.';
					}
				}
				
				$caredforInsertData['notes'] = $this->filterFormElements($postedData['notes']);
				$session = $this->intializeSessionInstance();
				$caredforInsertData['userId'] = $user_id;
				$caredforInsertData['userid_updated'] = $user_id;
				if ($isCaredforFormValid) { // need work
					// insert data into the database for person and resource table.
					if ($this->getProviderTable()->saveProvider($caredforInsertData, $root_folder_id)) {
						return $this->redirect()->toRoute('secure');
					}
				}
				
				// send validation error messages to the view.
				return new ViewModel(
					array(
						'messages'          => $errorMessages['messages'], 
						'setdata'           => $caredforInsertData,
                        'resourceArray'     =>  $resourceArray['resourceArray'],
                        'count'             => $resourceArray['count'],
                        'breadCrumbString'  => $breadCrumbString
					)
				);
			}
		}
		return new ViewModel(
			array(
				'resourceArray'     =>  $resourceArray['resourceArray'],
				'count'             => $resourceArray['count'],
				'breadCrumbString'  => $breadCrumbString
			)
        );
	}
    
    /**
     * Renders the provider's view template to show a shared resource that is being
     * shared with a view permission.
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function providerView($userId, $resourceRecord, $postedData = array())
    {	
		$providerData = $this
                            ->getProviderTable()
                            ->getProviderInfo($resourceRecord['fk_type_recid']);
        
		$breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceRecord['resourceid'], $userId);
        
        
        $resourceArray = $this->getResourceArray($userId, $resourceRecord['resourceid']);
        
        // populate the form with the retrieved data
		return 
            array(
                'presentData'       => $providerData, 
                'resourceid'        => $resourceRecord['resourceid'],
                'resourceArray'     => $resourceArray['resourceArray'],
                'count'             => $resourceArray['count'],
                'breadCrumbString'  => $breadCrumbString,
            );
    }
    
    /**
     * Viewing a container name.
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function containerView($type, $typeRecId, $resourceRecord)
    {   
        if ($type == 'calendar') {
            $calendarRecord = $this
                                ->getCalendarTable()
                                ->getCalendarNameFromId($typeRecId);
        }
        
        if ($type == 'task_group') {
            $calendarRecord = $this
                                ->getTasksTable()
                                ->getTasksNameFromId($typeRecId);
        }
        
        if ($type == 'contact_group') {
            $calendarRecord = $this
                                ->getContactsTable()
                                ->getContactsNameFromId($typeRecId);
        }
        
        if ($type == 'medication_group') {
            $calendarRecord = $this
                                ->getMedicationsTable()
                                ->getMedicationsNameFromId($typeRecId);
        }
        
        if ($type == 'document_group') {
            $calendarRecord = $this
                                ->getDocumentsTable()
                                ->getDocumentsNameFromId($typeRecId);
        }
        
        $session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        $resourceArray = array();
        
        // first collect the parent resource id from the given resource id
        $parentResourceRecord = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['resourceid']);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceRecord['resourceid'], $userId);
        
        if ($parentResourceRecord) {
            $resourceArray = $this->getResourceArray($userId, $parentResourceRecord['resourceid']);
        }
        
        // populate the form with the retrieved data
		return array(
                'presentData'       => $calendarRecord,
                'type'              => $type,
                'resourceArray'     => $resourceArray['resourceArray'],
                'count'             => $resourceArray['count'],
                'breadCrumbString'  => $breadCrumbString,
        );
    }
	
	/**
     * Renders the calendar associated with the given container
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function createProfileAction()
    {
		$isLoggedIn = 'yes';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
		
		$session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
		$rowSet = $this->getPersonTable()->getUserName($userId);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
		}
        
        $resourceArray = array();
        
        $resourceArray = $this->getResourceArray($userId);
		
		// get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
		
			// collect the posted data
			$postedData = $this->getRequest()->getPost();
			if ($postedData['set_up_resource'] == 'provider') {
				return $this->redirect()->toRoute('secure', array('action' => 'provider'));
			}
			else if ($postedData['set_up_resource'] == 'cared_for') {
				return $this->redirect()->toRoute('secure',  array('action' => 'caredfor'));
			}
		}
        
        // populate the form with the retrieved data
		return array(
                'resourceArray' => $resourceArray['resourceArray'],
                'count'         => $resourceArray['count']
        );
	}
    
    /**
     * Shows the detailed text to the user in case the entered email id does not
     * exist in the system.
     * 
     * @return \Zend\View\Model\ViewModel
     */
    public function invitationAction()
    {
        $isLoggedIn = 'yes';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
		
		$session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
		$rowSet = $this->getPersonTable()->getUserName($userId);
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
		}
        
        // populate the form with the retrieved data
		return new ViewModel();
    }
	
	/**
	 * Funcntion to show the uploade document
	 * It is login protected 
	 * User having permission can download it
	 * @param is the file name
	 */
	public function showDocumentAction()
    {
		
		$isLoggedIn = 'yes';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
		
		// get an instance of register form
		$request = $this->getRequest();
		//Getting data from url
		$encryptedRecordId = $this->params()->fromRoute('resourceid', 0);
        
        $convert = $this->getServiceLocator()->get('ConvertParams');
        $documentId = $convert->decode($encryptedRecordId);
        
        $documentRecord = '';
        if ($documentId) {
            $documentRecord = $this->getDocumentTable()->getDocumentRecordById($documentId);
        }
		
		//Getting data from config file
		$config = $this->getServiceLocator()->get('Config');
		
		//Creating file path
		$filePath = $config['document_upload_location'] . $documentRecord->file_name;
		
		//Accessing the library
		$response = new \Zend\Http\PhpEnvironment\Response();
		
		//If file exists
		if(file_exists($filePath)) {
		
			//Declaring the headers
			header('Content-Type: application/force-download');
			header('Content-Disposition: attachment; filename="' . $documentRecord->actual_file_name . '"');
			header("Content-Transfer-Encoding: binary");
			$response->setContent(file_get_contents($filePath));
		} else {
			
			//Declaring the headers
			header('Content-Type: application/force-download');
			header('Content-Disposition: attachment; filename="Uknown.txt"');
			header("Content-Transfer-Encoding: binary");
			$response->setContent('File does not exists!!');
		}
		
        return $response;
	 }
	 
    /**
     * Renders the calendar associated with the given container
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function caredforEdit($userId, $resourceRecord, $postedData = array())
	{   
        $personData = $this
                        ->getPersonTable()
                        ->getPersonInfo($resourceRecord['fk_type_recid']);
		
		// check if the request is a post request
        if ($postedData) {
			
			if ($postedData['cancel'] == 'Cancel') {
				return $this->redirect()->toRoute('secure');
			}
			
			$isCaredforEditFormValid = true;
			$errorMessages = array();
			
			// adding validations to the input fields
			$personUpdateData['first_name'] = $this->filterFormElements($postedData['first_name']);
			if ($personUpdateData['first_name'] == '') {
				$isCaredforEditFormValid = false;
				$errorMessages['messages']['firstName']['required'] = 'Please enter First Name.';
			}
			
			// adding validations to the input fields
			$personUpdateData['last_name'] = $this->filterFormElements($postedData['last_name']);
			if ($personUpdateData['last_name'] == '') {
				$isCaredforEditFormValid = false;
				$errorMessages['messages']['lastName']['required'] = 'Please enter Last Name.';
			}
			
			$personUpdateData['phone_main'] = $this->filterFormElements($postedData['phone_main']);
			$personUpdateData['phone_mobile'] = $this->filterFormElements($postedData['phone_mobile']);
			$personUpdateData['phone_emergency'] = $this->filterFormElements($postedData['phone_emergency']);
			$personUpdateData['address1'] = $this->filterFormElements($postedData['address1']);
			$personUpdateData['address2'] = $this->filterFormElements($postedData['address2']);
			$personUpdateData['city'] = $this->filterFormElements($postedData['city']);
			$personUpdateData['state'] = $this->filterFormElements($postedData['state']);
			$personUpdateData['zipcode'] = $this->filterFormElements($postedData['zipcode']);
			
			$emailvalidator = new EmailAddress();
			$personUpdateData['email_primary'] = $this->filterFormElements($postedData['email_primary']);
			
			// validation for primary email
			if ($personUpdateData['email_primary'] != '') {
                $personUpdateData['email_primary'] = strtolower($personUpdateData['email_primary']);
				if (!$emailvalidator->isValid($personUpdateData['email_primary'])) {
					$isCaredforEditFormValid = false;
					$errorMessages['messages']['emailPrimary']['invalid'] = 'The primary email is invalid.';
				}
			}
			
			// validation for secondary email
			$personUpdateData['email_secondary'] = $this->filterFormElements($postedData['email_secondary']);
			if ($personUpdateData['email_secondary'] != '') {
                $personUpdateData['email_secondary'] = strtolower($personUpdateData['email_secondary']);
				if (!$emailvalidator->isValid($personUpdateData['email_secondary'])) {
					$isCaredforEditFormValid = false;
					$errorMessages['messages']['emailSecondary']['invalid'] = 'The secondary email is invalid.';
				}
			}
			
			$personUpdateData['notes'] = $this->filterFormElements($postedData['notes']);
            $personUpdateData['userid_updated'] = $userId;
			
			if ($isCaredforEditFormValid) {
			
				// then insert data into the database
				if ($this->getPersonTable()->updatePersonInfo($personUpdateData, $personData['personid'])) {
					return $this->redirect()->toRoute('secure');
				}
			}
			
			return 
				array(
					'presentData'       => $personData,
					'messages'          => $errorMessages,
                    'resourceid'        => $resourceRecord['resourceid'],
                    'resourceArray'     =>  $resourceArray['resourceArray'],
                    'count'             => $resourceArray['count'],
                    'breadCrumbString'  => $breadCrumbString
				);
		}
		
        return array(
            'presentData'       => $personData, 
            'resourceid'        => $resourceRecord['resourceid'],
            'resourceArray'     =>  $resourceArray['resourceArray'],
            'count'             => $resourceArray['count'],
            'breadCrumbString'  => $breadCrumbString
        );
	}
    
    /**
     * Renders the calendar associated with the given container
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function providerEdit($userId, $resourceRecord, $postedData = array())
    {
		$providerData = $this
                            ->getProviderTable()
                            ->getProviderInfo($resourceRecord['fk_type_recid']);
        
		$breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceRecord['resourceid'], $userId);
        
        
        $resourceArray = $this->getResourceArray($userId, $resourceRecord['resourceid']);
        
		// check if the request is a post request
        if ($postedData) {
			
			if ($postedData['cancel'] == 'Cancel') {
				return $this->redirect()->toRoute('secure');
			}
            
			$isProviderEditFormValid = true;
			$errorMessages = array();
			$providerUpdateData = array();
			
			// adding validations to the input fields
			$providerUpdateData['provider_name'] = $this->filterFormElements($postedData['provider_name']);
			if ($providerUpdateData['provider_name'] == '') {
				$isProviderEditFormValid = false;
				$errorMessages['messages']['providerName']['required'] = 'Please enter Provider name.';
			}
			
			$providerUpdateData['provider_descr'] = $this->filterFormElements($postedData['provider_descr']);
			$providerUpdateData['phone_main'] = $this->filterFormElements($postedData['phone_main']);
			$providerUpdateData['phone_mobile'] = $this->filterFormElements($postedData['phone_mobile']);
			$providerUpdateData['phone_emergency'] = $this->filterFormElements($postedData['phone_emergency']);
			$providerUpdateData['address1'] = $this->filterFormElements($postedData['address1']);
			$providerUpdateData['address2'] = $this->filterFormElements($postedData['address2']);
			$providerUpdateData['city'] = $this->filterFormElements($postedData['city']);
			$providerUpdateData['state'] = $this->filterFormElements($postedData['state']);
			$providerUpdateData['zipcode'] = $this->filterFormElements($postedData['zipcode']);
			
			$emailvalidator = new EmailAddress();
			$providerUpdateData['email_primary'] = $this->filterFormElements($postedData['email_primary']);
			
			// validation for primary email
			if ($providerUpdateData['email_primary'] != '') {
                $providerUpdateData['email_primary'] = strtolower($providerUpdateData['email_primary']);
				if (!$emailvalidator->isValid($providerUpdateData['email_primary'])) {
					$isProviderEditFormValid = false;
					$errorMessages['messages']['emailPrimary']['invalid'] = 'The primary email is invalid.';
				}
			}
			
			$providerUpdateData['email_secondary'] = $this->filterFormElements($postedData['email_secondary']);
			// validation for secondary email
			if ($providerUpdateData['email_secondary'] != '') {
                
                $providerUpdateData['email_secondary'] = strtolower($providerUpdateData['email_secondary']);
				if (!$emailvalidator->isValid($providerUpdateData['email_secondary'])) {
					$isProviderEditFormValid = false;
					$errorMessages['messages']['emailSecondary']['invalid'] = 'The secondary email is invalid.';
				}
			}
			
			$providerUpdateData['notes'] = $this->filterFormElements($postedData['notes']);
            $providerUpdateData['userid_updated'] = $userId;
            
			if ($isProviderEditFormValid) {
			
				// then insert data into the database
				if ($this->getProviderTable()->updateProviderInfo($providerUpdateData, $providerData['providerid'])) {
					return $this->redirect()->toRoute('secure');
				}
			}
			
			
			// populate the form with the retrieved data
			return 
				array(
                    'presentData'       => $providerData,
                    'messages'          => $errorMessages,
                    'resourceid'        => $resourceRecord['resourceid'],
                    'resourceArray'     => $resourceArray['resourceArray'],
                    'count'             => $resourceArray['count'],
                    'breadCrumbString'  => $breadCrumbString,
            );
		}
		
		// populate the form with the retrieved data
		return 
            array(
                'presentData'       => $providerData, 
                'resourceid'        => $resourceRecord['resourceid'],
                'resourceArray'     => $resourceArray['resourceArray'],
                'count'             => $resourceArray['count'],
                'breadCrumbString'  => $breadCrumbString,
            );
	}
    
    /**
     * Editing the calendar name.
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function containerEdit($userId, $resourceRecord, $postedData)
    {
        if ($resourceRecord['type'] == 'calendar') {
            $calendarRecord = $this
                                ->getCalendarTable()
                                ->getCalendarNameFromId($resourceRecord['fk_type_recid']);
        }
        
        if ($resourceRecord['type'] == 'task_group') {
            $calendarRecord = $this
                                ->getTasksTable()
                                ->getTasksNameFromId($resourceRecord['fk_type_recid']);
        }
        
        if ($resourceRecord['type'] == 'contact_group') {
            $calendarRecord = $this
                                ->getContactsTable()
                                ->getContactsNameFromId($resourceRecord['fk_type_recid']);
        }
        
        if ($resourceRecord['type'] == 'medication_group') {
            $calendarRecord = $this
                                ->getMedicationsTable()
                                ->getMedicationsNameFromId($resourceRecord['fk_type_recid']);
        }
		
		//For documents
		if ($resourceRecord['type'] == 'document_group') {
            $calendarRecord = $this
                                ->getDocumentsTable()
                                ->getDocumentsNameFromId($resourceRecord['fk_type_recid']);
        }
        
        $session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceRecord['resourceid'], $userId);
        
        // first collect the parent resource id from the given resource id
        $parentResourceRecord = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['resourceid']);
        if ($parentResourceRecord) {
            $resourceArray = $this->getResourceArray($userId, $parentResourceRecord['resourceid']);
        }
		
		// check if the request is a post request
        if ($postedData) {
			
			if ($postedData['cancel'] == 'Cancel') {
                
                // get parent resourceid and type
                $encodedString = $this
                                    ->getReturnEncodedStringForContainer($resourceRecord['resourceid']);
                
				// check if its a shared resource
                if ($this->getUserResourceTable()->isResourceShared($userId, $resourceRecord['resourceid'])) {
                    return $this->redirect()->toRoute('secure', array('action'=>'index'));
                }
                
                return $this->redirect()->toRoute('secure', array('action'=>'index', 'resourceid' => $encodedString));
			}
            
            $isCalendarFormValid = true;
            $calendarUpdateData['container_name'] = $this->filterFormElements($postedData['container_name']);
            
            if ($calendarUpdateData['container_name'] == '') {
                
                $isCalendarFormValid = false;
                $errorMessages['messages']['calendarName']['required'] = 'Please enter name for the resource.';
            }
            
            if ($isCalendarFormValid) {
                
                $updateStatus = false;
                
                // Update the required tables depending upon the conditions.
                if ($resourceRecord['type'] == 'calendar') {
                    
                    // update the calendar table with the help of resource id given.
                    $updateStatus = $this
                                        ->getCalendarTable()
                                        ->updateCalendarRecordById($resourceRecord['fk_type_recid'], $calendarUpdateData['container_name']);
                }
                
                else if ($resourceRecord['type'] == 'task_group') {
                    $updateStatus = $this
                                        ->getTasksTable()
                                        ->updateTasksRecordById($resourceRecord['fk_type_recid'], $calendarUpdateData['container_name']);
                }
                
                else if ($resourceRecord['type'] == 'contact_group') {
                    $updateStatus = $this
                                        ->getContactsTable()
                                        ->updateContactsRecordById($resourceRecord['fk_type_recid'], $calendarUpdateData['container_name']);
                }
				
                else if ($resourceRecord['type'] == 'medication_group') {
                    $updateStatus = $this
                                        ->getMedicationsTable()
                                        ->updateMedicationsRecordById($resourceRecord['fk_type_recid'], $calendarUpdateData['container_name']);
                }
				
				else if ($resourceRecord['type'] == 'document_group') {
                    $updateStatus = $this
                                        ->getDocumentsTable()
                                        ->updateDocumentsRecordById($resourceRecord['fk_type_recid'], $calendarUpdateData['container_name']);
                }
                
                if ($updateStatus) {
                    
                    // redirect to secure section for now
                    $encodedString = $this->getReturnEncodedStringForContainer($resourceRecord['resourceid']);
                    
                    // check if the resource is being shared
                    if ($this->getUserResourceTable()->isResourceShared($userId, $resourceRecord['resourceid'])) {
                        return $this->redirect()->toRoute('secure');
                    }
                    else {
                        return $this->redirect()->toRoute('secure', array('resourceid' => $encodedString));
                    }
                }
            }
            
            // reload the form showing the validation messages and the fields non-empty.
            return 
                array(
                    'presentData'       => $calendarRecord,
                    'type'              => $resourceRecord['type'],
                    'resourceid'        => $resourceRecord['resourceid'],
                    'messages'          => $errorMessages,
                    'resourceArray'     => $resourceArray['resourceArray'],
                    'count'             => $resourceArray['count'],
                    'breadCrumbString'  => $breadCrumbString,
            );
        }
        // populate the form with the retrieved data
		return 
			array(
                'presentData'       => $calendarRecord,
                'type'              => $resourceRecord['type'],
                'resourceid'        => $resourceRecord['resourceid'],
                'resourceArray'     => $resourceArray['resourceArray'],
                'count'             => $resourceArray['count'],
                'breadCrumbString'  => $breadCrumbString,
        );
    }
    
    /**
     * Function to edit task.
     * 
     * @param int   $userId
     * @param array $resourceRecord
     * @param array $postedData
     * @return type
     */
	public function editTask($userId, $resourceRecord, $postedData = array())
    {       
        $taskRecord = $this
                        ->getTaskTable()
                        ->getTaskRecordById($resourceRecord['fk_type_recid']);
		
		// check if the request is a post request
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
				
                $encodeArray = array();
                $encodeArray['resourceid']  = $resourceRecord['fk_resourceid_parent'];
                $encodeArray['tab_type']    = self :: TASK_RESOURCE;
                $encodeArray['action']      = self :: LIST_ACTION;
                
                $encodedString = $this->createEncodedString($encodeArray);
                
                return $this->redirect()->toRoute('secure', array('action'=>'index', 'resourceid' => $encodedString));
			}
            
			$isTaskFormValid = true;
			$errorMessages = array();
			$taskUpdateData = array();
            
            $taskUpdateData['task_description'] = $this->filterFormElements($postedData['task_description']);
            if ($taskUpdateData['task_description'] == '') {
                $isTaskFormValid = False;
                $errorMessages['messages']['task_description']['required'] = 'Please enter Task Description.';
            }
            
            $taskUpdateData['due_date'] = $this->filterFormElements($postedData['due_date']);
            if ($taskUpdateData['due_date'] != '') {
                
                // check if the entered date is present in the valid format.
                $validator = new Date(array('format' => 'm/d/Y'));
                $isValid = $validator->isValid($taskUpdateData['due_date']);
                
                if (!$isValid) {
                    $isTaskFormValid = False;
                    unset($taskUpdateData['due_date']);
                    $errorMessages['messages']['due_date']['invalid'] = 'Please provide Due date in mm/dd/yyyy format or use the popup calendar to select a date.';
                }
                else {
                    $taskUpdateData['due_date'] = date('Y-m-d', strtotime($taskUpdateData['due_date']));
                }
            }
            
            $taskUpdateData['priority'] = $this->filterFormElements($postedData['priority']);
            $taskUpdateData['person_responsibile'] = $this->filterFormElements($postedData['person_responsibile']);
            $taskUpdateData['task_notes'] = $this->filterFormElements($postedData['task_notes']);
            $taskUpdateData['is_completed'] = $this->filterFormElements($postedData['is_completed']);
            $taskUpdateData['taskid'] = $resourceRecord['fk_type_recid'];
            
            if ($isTaskFormValid) {
                
                // Update the task record in the database
                // then update medication data update in to the table
                $updateStatus = $this
                                    ->getTaskTable()
                                    ->updateTask($taskUpdateData, $userId);
                
                if ($updateStatus) {
                    
                    /* prepare the array as required so that it navigates to the
                        tasks tab */
                    $encodeArray = array();
                    
                    // resourceid of the parent
                    $encodeArray['resourceid'] = $resourceRecord['fk_resourceid_parent'];
                    
                    // mention the tab type.
                    $encodeArray['tab_type']    = self :: TASK_RESOURCE;
                    
                    // mentiont the list action
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $encodedString = $this->createEncodedString($encodeArray);
                    return $this->redirect()->toRoute('secure', array('resourceid' => $encodedString));
                }
            }
            return 
                array(
                    'task_record'       =>  $taskUpdateData,
                    'messages'          =>  $errorMessages,
                    'resourceid'        =>  $resourceRecord['resourceid'],
                    'resourceArray'     =>  $resourceArray['resourceArray'],
                    'count'             =>  $resourceArray['count'],
                    'breadCrumbString'  =>  $breadCrumbString
            );
        }
        
        // populate the form with the retrieved data
		return array(
            'task_record'           =>  $taskRecord,
            'resourceid'            =>  $resourceRecord['resourceid'],
            'resourceArray'         =>  $resourceArray['resourceArray'],
            'count'                 =>  $resourceArray['count'],
            'breadCrumbString'      =>  $breadCrumbString
        );
    }
    
    /**
     * Function for view only Contacts
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function viewTask($userId, $resourceRecord)
    {       
        $taskRecord = $this
                        ->getTaskTable()
                        ->getTaskRecordById($resourceRecord['fk_type_recid']);
        
        $resourceArray = array();
        // first collect the parent resource id from the given resource id
        $parentResourceRecordOne = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['resourceid']);
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($parentResourceRecordOne['resourceid'], $userId);
        
        if ($parentResourceRecordOne) {
            $parentResourceRecordTwo = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($parentResourceRecordOne['resourceid']);
            $resourceArray = $this->getResourceArray($userId, $parentResourceRecordTwo['resourceid']);
        }
        
        $encodedString = $this->getReturnEncodedStringForContainer($resourceRecord['resourceid']);
        
        return array(
            'taskRecord'        =>  $taskRecord,
            'return_resourceid' =>  $encodedString,
            'resourceArray'     =>  $resourceArray['resourceArray'],
            'count'             =>  $resourceArray['count'],
            'breadCrumbString'  =>  $breadCrumbString,
        );
    }
    
    public function viewDocument($userId, $resourceRecord)
    {
        $documentRecord = $this
                        ->getDocumentTable()
                        ->getDocumentRecordById($resourceRecord['fk_type_recid']);
        
        $session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        
        $resourceArray = array();
        
        //accessing config data
		$config = $this->getServiceLocator()->get('Config');
        
        // first collect the parent resource id from the given resource id
        $parentResourceRecord = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['fk_resourceid_parent']);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($resourceRecord['fk_resourceid_parent'], $userId);
        
        
        $resourceArray = $this->getResourceArray($userId, $parentResourceRecord['resourceid']);
        
        //populate the form with the retrieved data
		return array(
            'documentRecord'       =>  $documentRecord,
            'resourceid'            =>  $resourceRecord['resourceid'],
            'resourceArray'         =>  $resourceArray['resourceArray'],
            'count'                 =>  $resourceArray['count'],
            'breadCrumbString'      =>  $breadCrumbString
        );
    }
    
	/**
     * Function to edit Document
     *
	 * @param   int $userId
	 * @param   array $resourceRecord
	 * @param   array $postedData
     * @return  instance of view model(send data to view).
     */
	public function editDocument($userId, $resourceRecord, $postedData = array())
    {
        $documentRecord = $this
                        ->getDocumentTable()
                        ->getDocumentRecordById($resourceRecord['fk_type_recid']);
        
        //accessing config data
		$config = $this->getServiceLocator()->get('Config');
		
		// check if the request is a post request
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
				
                $encodeArray = array();
                $encodeArray['resourceid']  = $resourceRecord['fk_resourceid_parent'];
                $encodeArray['tab_type']    = self :: DOCUMENT_RESOURCE;
                $encodeArray['action']      = self :: LIST_ACTION;
                
                $encodedString = $this->createEncodedString($encodeArray);
                
                return $this->redirect()->toRoute('secure', array('action'=>'index', 'resourceid' => $encodedString));
			}
            
			$isDocumentFormValid = true;
			$errorMessages = array();
			$documentUpdateData = array();
			
            $documentUpdateData['document_description'] = $this->filterFormElements($postedData['document_description']);
			
            if ($documentUpdateData['document_description'] == '') {
				$isDocumentFormValid = false;
				$errorMessages['messages']['document_desc']['required'] = 'Please enter document description.';
			}
            
            $documentUpdateData['parent_resource_id'] = $postedData['document_recid'];
			
			//Functionality for validating files
			$files = $this->getRequest()->getFiles();
            
            $ext = end(explode('.', $files['document_file']['name']));
            
            if ($files['document_file']['name']) {
                if (!($this->isValidMimeType($files['document_file']['type'])) 
                        || !($this->isValidFileType($ext))) {
                    $isDocumentFormValid = false;
                    $errorMessages['messages']['document_file']['file_type'] = 'You 
                        have selected a filetype that is not allowed. Please upload 
                        approved types only such as:<br />
                        All "Office" documents: doc, docx, xls, xlsx, mdb<br />
                        All common image types: jpg, jepg, bmp, tif, gif<br />
                        Other common types: txt, zip pdf, ppt, 7z, mp3, mp4, mpeg and mpg';
                }
            }
			
			//If no file error and form validated correctly the move the uploded file
			if(($files['document_file']['error'] === 0) && ($isDocumentFormValid)) {
			
				//Getting file path from config data
				$uploadPath = $config['document_upload_location'];
				//The new file name will be user id with current timestamp separated by underscore
				$newFileName = $userId . '_' . time();
				
			
				//Moving file to a destination
				if(!(move_uploaded_file($files['document_file']['tmp_name'], $uploadPath .$newFileName . '.' . $ext))) {
					$isDocumentFormValid = false;
					$errorMessages['messages']['document_file']['required'] = 'Uploaded file could not saved. Please try again.';
				};
			}
            
            $isEmpty = 'yes';
            if (isset($files['document_file']['name']) && ($files['document_file']['name'] !== '')) {
                $isEmpty = 'no';
                $documentUpdateData['actual_file_name'] = $files['document_file']['name'];
                
                if ($files['document_file']['size'] > $config['max_upload_filesize']) {
                    $isDocumentFormValid = false;
                    $errorMessages['messages']['document_file']['max'] = 'Uploaded file exceeded max upload file size.';
                }
            }
            
			$documentUpdateData['documentid'] = $resourceRecord['fk_type_recid'];
			
            if ($isDocumentFormValid) {

				if($files['document_file']['error'] === 0) {
					//insert the medication data in to the DB
					$documentUpdateData['file_error'] = 0;
					$documentUpdateData['file_name'] = $newFileName.'.'.$ext;
				}
                
                $updateStatus = $this
                                    ->getDocumentTable()
                                    ->updateDocument($documentUpdateData, $userId, $isEmpty);
                if ($updateStatus) {
                    
                    $encodeArray = array();
                    $encodeArray['resourceid']  = $resourceRecord['fk_resourceid_parent'];
                    $encodeArray['tab_type']    = self :: DOCUMENT_RESOURCE;
                    $encodeArray['action']      = self :: LIST_ACTION;

                    $encodedString = $this->createEncodedString($encodeArray);
                    return $this->redirect()->toRoute('secure', array('resourceid' => $encodedString));
                }
            }
            
            return 
                array(
                    'document_record'   =>  $documentUpdateData,
                    'messages'          =>  $errorMessages,
                    'resourceid'        =>  $resourceRecord['resourceid'],
                    'resourceArray'     =>  $resourceArray['resourceArray'],
                    'count'             =>  $resourceArray['count'],
                    'breadCrumbString'  =>  $breadCrumbString
            );
        }
        
        //populate the form with the retrieved data
		return array(
            'document_record'       =>  $documentRecord,
            'resourceid'            =>  $resourceRecord['resourceid'],
            'resourceArray'         =>  $resourceArray['resourceArray'],
            'count'                 =>  $resourceArray['count'],
            'breadCrumbString'      =>  $breadCrumbString
        );
    }
	
    /**
     * Function to edit contacts
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function editContact($userId, $resourceRecord, $postedData = array())
    {
        $contactRecord = $this
                            ->getContactTable()
                            ->getContactRecordById($resourceRecord['fk_type_recid']);
		
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
                
                /* prepare the array as required so that it navigates to the
                    tasks tab */
                $encodeArray = array();

                // resourceid of the parent
                $encodeArray['resourceid'] = $resourceRecord['fk_resourceid_parent'];

                // mention the tab type.
                $encodeArray['tab_type']    = self :: CONTACT_RESOURCE;

                // mentiont the list action
                $encodeArray['action']      = self :: LIST_ACTION;

                $encodedString = $this->createEncodedString($encodeArray);
                return $this->redirect()->toRoute('secure', array('action'=>'index', 'resourceid' => $encodedString));
			}
            
            $contactDataUpdate = array();
            $isContactFormValid = true;
			$errorMessages = array();
            
            $contactDataUpdate['contact_name'] = $this->filterFormElements($postedData['contact_name']);
            if ($contactDataUpdate['contact_name'] == '') {
                $isContactFormValid = False;
                $errorMessages['messages']['contact_name']['required'] = 'Please enter contact name.';
            }
            
            $contactDataUpdate['business_name'] = $this->filterFormElements($postedData['business_name']);
            $contactDataUpdate['contact_category'] = $this->filterFormElements($postedData['contact_category']);
            $contactDataUpdate['email_address1'] = $this->filterFormElements($postedData['email_address1']);
            
            if ($contactDataUpdate['email_address1'] != '') {
                $contactDataUpdate['email_address1'] = strtolower($contactDataUpdate['email_address1']);
                $emailvalidator = new EmailAddress();
                if (!$emailvalidator->isValid($contactDataUpdate['email_address1'])) {
                    $isContactFormValid = false;
                    $errorMessages['messages']['email_address1']['invalid'] = 'The entered Email address is invalid.';
                }
            }
                
            
            $contactDataUpdate['mobile_number'] = $this->filterFormElements($postedData['mobile_number']);
            $contactDataUpdate['home_number']   = $this->filterFormElements($postedData['home_number']);
            $contactDataUpdate['work_number']   = $this->filterFormElements($postedData['work_number']);
            $contactDataUpdate['work_address']  = $this->filterFormElements($postedData['work_address']);
            $contactDataUpdate['work_city']     = $this->filterFormElements($postedData['work_city']);
            $contactDataUpdate['work_state']    = $this->filterFormElements($postedData['work_state']);
            $contactDataUpdate['work_zip']      = $this->filterFormElements($postedData['work_zip']);
            $contactDataUpdate['home_city']     = $this->filterFormElements($postedData['home_city']);
            $contactDataUpdate['home_address']  = $this->filterFormElements($postedData['home_address']);
            $contactDataUpdate['home_state']    = $this->filterFormElements($postedData['home_state']);
            $contactDataUpdate['home_zip']      = $this->filterFormElements($postedData['home_zip']);
            $contactDataUpdate['home_page']     = $this->filterFormElements($postedData['home_page']);
            $contactDataUpdate['contact_notes'] = $this->filterFormElements($postedData['contact_notes']);
            $contactDataUpdate['contactid']     = $this->filterFormElements($resourceRecord['fk_type_recid']);
            
            if ($isContactFormValid) {
                // then update medication data update in to the table
                $updateStatus = $this
                                    ->getContactTable()
                                    ->updateContact($contactDataUpdate, $userId);
                
                if ($updateStatus) {
                    
                    /* prepare the array as required so that it navigates to the
                        tasks tab */
                    $encodeArray = array();
                    
                    // resourceid of the parent
                    $encodeArray['resourceid'] = $resourceRecord['fk_resourceid_parent'];
                    
                    // mention the tab type.
                    $encodeArray['tab_type']    = self :: CONTACT_RESOURCE;
                    
                    // mentiont the list action
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $encodedString = $this->createEncodedString($encodeArray);
                    return $this->redirect()->toRoute('secure', array('resourceid' => $encodedString));
                }
            }
             
            // repopulate the form with validation messages.
            return array(
                    'contact_record'    =>  $contactDataUpdate,
                    'messages'          =>  $errorMessages,
                    'resourceid'        =>  $resourceRecord['resourceid'],
                    'resourceArray'     =>  $resourceArray['resourceArray'],
                    'count'             =>  $resourceArray['count'],
                    'breadCrumbString'  =>  $breadCrumbString
                );
        }
        
        // populate the form with the retrieved data
		return array(
            'contact_record'        =>  $contactRecord,
            'resourceid'            =>  $resourceRecord['resourceid'],
            'resourceArray'         =>  $resourceArray['resourceArray'],
            'count'                 =>  $resourceArray['count'],
            'breadCrumbString'      =>  $breadCrumbString
        );
    }
    
    /**
     * Function to edit contacts
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function viewContact($userId, $resourceRecord)
    {
        $contactRecord = $this
                            ->getContactTable()
                            ->getContactRecordById($resourceRecord['fk_type_recid']);
        
        $resourceArray = array();
        // first collect the parent resource id from the given resource id
        $parentResourceRecordOne = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['resourceid']);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($parentResourceRecordOne['resourceid'], $userId);
        
        if ($parentResourceRecordOne) {
            $parentResourceRecordTwo = $this
                                        ->getResourceTable()
                                        ->getParentResourceRecord($parentResourceRecordOne['resourceid']);
            $resourceArray = $this->getResourceArray($userId, $parentResourceRecordTwo['resourceid']);
        }
        
        // populate the form with the retrieved data
		return array(
            'contactRecord'     => $contactRecord,
            'resourceid'        => $resourceRecord['resourceid'],
            'resourceArray'     => $resourceArray['resourceArray'],
            'count'             => $resourceArray['count'],
            'breadCrumbString'  => $breadCrumbString
        );
    }
    
    /**
     * Function to edit medication.
     * 
     * @param int $userId
     * @param array $resourceRecord
     * @param array $postedData
     * @return array
     */
	public function editMedication($userId, $resourceRecord, $postedData = array())
    {
        $medicationRecord = $this
                                ->getMedicationTable()
                                ->getMedicationRecordById($resourceRecord['fk_type_recid']);
        
        $resourceArray = array();
        // first collect the parent resource id from the given resource id
        $parentResourceRecordOne = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['resourceid']);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($parentResourceRecordOne['resourceid'], $userId);
        if ($parentResourceRecordOne) {
            $parentResourceRecordTwo = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($parentResourceRecordOne['resourceid']);
            $resourceArray = $this->getResourceArray($userId, $parentResourceRecordTwo['resourceid']);
        }
		
        if ($postedData) {
            
			if ($postedData['cancel'] == 'Cancel') {
                
                 /* prepare the array as required so that it navigates to the
                        tasks tab */
                $encodeArray = array();

                // resourceid of the parent
                $encodeArray['resourceid'] = $resourceRecord['fk_resourceid_parent'];

                // mention the tab type.
                $encodeArray['tab_type']    = self :: MEDICATION_RESOURCE;

                // mentiont the list action
                $encodeArray['action']      = self :: LIST_ACTION;

                $encodedString = $this->createEncodedString($encodeArray);
                
                return $this->redirect()->toRoute('secure', array('action'=>'index', 'resourceid' => $encodedString));
			}
            
            $medicationDataUpdate = array();
            $isMedicationFormValid = true;
			$errorMessages = array();
            
            $medicationDataUpdate['med_name'] = $this->filterFormElements($postedData['med_name']);
            if ($medicationDataUpdate['med_name'] == '') {
                $isMedicationFormValid = False;
                $errorMessages['messages']['med_name']['required'] = 'Please enter medication name.';
            }
            
            $medicationDataUpdate['generic_name'] = $this->filterFormElements($postedData['generic_name']);
            $medicationDataUpdate['dosage'] = $this->filterFormElements($postedData['dosage']);
            $medicationDataUpdate['dosage_frequency'] = $this->filterFormElements($postedData['dosage_frequency']);
            $medicationDataUpdate['pharmacy'] = $this->filterFormElements($postedData['pharmacy']);
            $medicationDataUpdate['med_notes'] = $this->filterFormElements($postedData['med_notes']);
            $medicationDataUpdate['physician'] = $this->filterFormElements($postedData['physician']);
            $medicationDataUpdate['date_prescribed'] = $this->filterFormElements($postedData['date_prescribed']);
            
            if ($medicationDataUpdate['date_prescribed'] != '') {
                
                // check if the entered date is present in the valid format.
                $validator = new Date(array('format' => 'm/d/Y'));
                $isValid = $validator->isValid($medicationDataUpdate['date_prescribed']);
                
                if (!$isValid) {
                    $isMedicationFormValid = False;
                    unset($medicationDataUpdate['date_prescribed']);
                    $errorMessages['messages']['date_prescribed']['invalid'] = 'Please provide Date Prescribed in mm/dd/yyyy format or use the popup calendar to select a date.';
                }
                else {
                    $medicationDataUpdate['date_prescribed'] = date('Y-m-d', strtotime($medicationDataUpdate['date_prescribed']));
                }
            }
            
            $medicationDataUpdate['symptoms_address']   = $this->filterFormElements($postedData['symptoms_address']);
            $medicationDataUpdate['side_effects']       = $this->filterFormElements($postedData['side_effects']);
            $medicationDataUpdate['drug_interactions']  = $this->filterFormElements($postedData['drug_interactions']);
            $medicationDataUpdate['physical_desc']      = $this->filterFormElements($postedData['physical_desc']);
            $medicationDataUpdate['medicationid']       = $this->filterFormElements($resourceRecord['fk_type_recid']);
            
            if ($isMedicationFormValid) {
                
                // then update medication data update in to the table
                $updateStatus = $this
                                    ->getMedicationTable()
                                    ->updateMedication($medicationDataUpdate, $userId);
                
                if ($updateStatus) {
                        
                    /* prepare the array as required so that it navigates to the
                        tasks tab */
                    $encodeArray = array();
                    
                    // resourceid of the parent
                    $encodeArray['resourceid'] = $resourceRecord['fk_resourceid_parent'];
                    
                    // mention the tab type.
                    $encodeArray['tab_type']    = self :: MEDICATION_RESOURCE;
                    
                    // mentiont the list action
                    $encodeArray['action']      = self :: LIST_ACTION;
                    
                    $encodedString = $this->createEncodedString($encodeArray);
                    return $this->redirect()->toRoute('secure', 
                                                            array(
                                                                'action'=> 'index', 
                                                                'resourceid' => $encodedString
                                                            )
                    );
                }
            }
            
            return array(
                    'medication_record' =>  $medicationDataUpdate,
                    'messages'          =>  $errorMessages,
                    'resourceid'        =>  $resourceRecord['resourceid'],
                    'resourceArray'     =>  $resourceArray['resourceArray'],
                    'count'             =>  $resourceArray['count'],
                    'breadCrumbString'  =>  $breadCrumbString
            );
        }
        
        // populate the form with the retrieved data
		return array(
                'medication_record'     =>  $medicationRecord,
                'resourceid'            =>  $resourceRecord['resourceid'],
                'resourceArray'         =>  $resourceArray['resourceArray'],
                'count'                 =>  $resourceArray['count'],
                'breadCrumbString'      =>  $breadCrumbString,
        );
    }
    
    /**
     * Function to edit medications
     *
	 * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function viewMedication($userId, $resourceRecord)
    {
        $medicationRecord = $this
                                ->getMedicationTable()
                                ->getMedicationRecordById($resourceRecord['fk_type_recid']);
        
        $resourceArray = array();
        // first collect the parent resource id from the given resource id
        $parentResourceRecordOne = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($resourceRecord['resourceid']);
        
        $breadCrumbString = $this
							   ->getResourceTable()
							   ->createBreadcrumb($parentResourceRecordOne['resourceid'], $userId);
        
        if ($parentResourceRecordOne) {
            $parentResourceRecordTwo = $this
                                    ->getResourceTable()
                                    ->getParentResourceRecord($parentResourceRecordOne['resourceid']);
            $resourceArray = $this->getResourceArray($userId, $parentResourceRecordTwo['resourceid']);
        }
        
        // populate the form with the retrieved data
		return array(
            'medicationRecord'      => $medicationRecord,
            'resourceArray'         =>  $resourceArray['resourceArray'],
            'count'                 => $resourceArray['count'],
            'breadCrumbString'      =>  $breadCrumbString
        );
    }
    
    /**
     * Function to give Betafeedback for the beta release of Help to care site.
     */
    public function betaFeedbackAction()
    {
        $isLoggedIn = 'yes';
		
		// check if the user is already logged in.
		if (!$this->getAuthService()->hasIdentity()) {
			$isLoggedIn = 'no';
			return $this->redirect()->toRoute('login');
		}
		
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
        
        $session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        $emailLogin = $session->offsetGet('email_login');
        
		$rowSet = $this->getPersonTable()->getUserName($userId);
        
        // If we get the user information then send it to the layouts file.
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] 
                    . ' ' . $rowSet['last_name'];
		}
        
        $request = $this->getRequest();
        $postedData = $request->getPost();
        $betaFeedbackData = array();
        
        if ($request->isPost()) {
            $betaFeedbackData['referer_url'] = $this->filterFormElements($postedData['referer_url']);
        }
        else {
            $betaFeedbackData['referer_url'] = $_SERVER['HTTP_REFERER'];
        }
        
        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'noBetaFeedbackLink'   => TRUE
                )
        );
		
		// check if the request method is a post.
        if ($request->isPost()) {
            
            if ($postedData['cancel'] == 'Cancel') {
                return $this->redirect()->toRoute('secure');
			}
            
            $isBetaFeedbackFormValid = true;
			$errorMessages = array();
            
            
            $betaFeedbackData['subject'] = html_entity_decode(htmlentities(trim($postedData['subject'])));
            
            if ($betaFeedbackData['subject'] == '') {
                $isBetaFeedbackFormValid = False;
                $errorMessages['messages']['subject']['required'] = 'Please enter subject for the feedback.';
            }
            
            $betaFeedbackData['body'] = html_entity_decode(htmlentities(trim($postedData['body'])));
            
            if ($betaFeedbackData['body'] == '') {
                $isBetaFeedbackFormValid = False;
                $errorMessages['messages']['body']['required'] = 'Please enter body for the feedback.';
            }
           
            $betaFeedbackData['timestamp'] = time();
            $betaFeedbackData['userid_created'] = $userId;
            $betaFeedbackData['ip_address'] = $_SERVER['REMOTE_ADDR'];
            
            // check if the Beta Feedback form valid.
            if ($isBetaFeedbackFormValid) {
                
                // insert in to the required table
                $insertStatus = $this
                                    ->getBetaFeedbackInfoTable()
                                    ->saveBetaFeedbackRecord($betaFeedbackData);
                
                // if successfully inserted data in to the BetaFeedbackInfoTable.
                if ($insertStatus) {
                    
                    // send an email to info@helptocare.com
                    $convert = $this->getServiceLocator()->get('SendEmail');
                    $config = $this->getServiceLocator()->get('Config');

                    // create the link for activation code.
                    $smtpOptions = $config['mailer']['smtp_options'];
                    $htmlBody = $betaFeedbackData['body'];
                    $subject = $betaFeedbackData['subject'];
                    $fromName = $rowSet['first_name'] . ' ' . $rowSet['last_name'];
                    $fromEmail = $emailLogin;
                    $to = $config['mailer']['beta_feedback_mail_constants']['feedback_to_email'];

                    // $htmlBody, $subject, $fromName, $fromEmail, $to
                    $convert->sendEmail($smtpOptions, $htmlBody, $subject, $fromName, $fromEmail, $to);
                    $this->redirect()->toRoute('secure');
                }
                
                return new viewModel(
                        array(
                            'messages'          => $errorMessages,
                            'betaFeedbackData'  => $_SERVER['HTTP_REFERER']
                        )
                );
                
            }
            
            return new viewModel(
                array(
                    'messages'          => $errorMessages,
                    'betaFeedbackData'  => $betaFeedbackData,
                    'refererUrl'        => $betaFeedbackData['referer_url']
                )
            );
        }   
        
        return new viewModel(
            array(
                'betaFeedbackData'  => $betaFeedbackData,
                'refererUrl'        => $betaFeedbackData['referer_url']
            )
        );
    }
	
	/**
     * Process the ajax post request and inserts event data into the date in DB
	 *
     * @param  (int)$containerId
     * @return instance of view model(send data to view).
     */
	public function addEventAction($containerId = 0)
    {
        $session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
		$request = $this->getRequest();
        if ($request->isPost()) {
			$event = new Events();
            $postData = $request->getPost();
			$event->container_id = (int)$postData->container_id;
			$event->event_name = $postData->event_name;
			if($postData->all_day == 0){
				$event->start_date = date('Y-m-d H:i:s',strtotime($postData->start_date.' '.$postData->start_time));
				$event->end_date =  date('Y-m-d H:i:s',strtotime($postData->end_date.' '.$postData->end_time));
			} else {
				$event->start_date = date('Y-m-d H:i:s',strtotime($postData->start_date));
				$event->end_date =  date('Y-m-d H:i:s',strtotime($postData->end_date));
			}
			$event->id = (int)$postData->id;
			$event->where = $postData->where;
			$event->description = $postData->description;
			$event->is_all_day = $postData->all_day;
			$event->color = $postData->color;
			echo $this->getEventsTable()->saveEvent($event, $userId); 
		}
		exit;
	}
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getEventsTable()
	{
        if (!$this->eventsTable) {
            $sm = $this->getServiceLocator();
            $this->eventsTable = $sm->get('Secure\Model\eventsTable');
        }
        return $this->eventsTable;
    }
    
    /**
     * Get all the event present under a container or foler as string.
     *
     * @return instance of view model(send data to view).
     */
	public function getAllEventUnderContainerAction()
    {
        $request = $this->getRequest();
		
		// check if the request method is a post.
        if ($request->isPost()) {
			$postData = $request->getPost();
			$eventRecords = $this->getEventsTable()->fetchAll($postData->container_id);
			
            
			// count whether any record exists or not.
			if (count($eventRecords) > 0) {
                
                $eventTableString = '<tr>
                    <th>Start Date</th>
                    <th>End Date</th>
                    <th>Event Name</th>
                    <th>Desc</th>
                </tr>';
                
				foreach($eventRecords as $result) {
                    
                    $eventTableString .= '<tr>';
                    
                    if ($result->is_all_day) {
                        $eventTableString .= '<td colspan="2">' . 'All day' . '</td>';
                    }
                    else {
                        $eventTableString .= '<td>' . date("m/d/Y h:i A", strtotime($result->start_date)) . '</td>';
                        $eventTableString .= '<td>' . date("m/d/Y h:i A", strtotime($result->end_date)) . '</td>';
                    }
                    
                    $eventTableString .= '<td>' . $result->event_name . '</td>';
                    
                    if ($result->where && $result->description) {
                        $eventTableString .= '<td>' . $result->where . ' - ' . $result->description . '</td>';
                    }
                    else {
                        $eventTableString .= '<td>' . $result->where . ' ' . $result->description . '</td>';
                    }
                    
                    $eventTableString .= '</tr>';
				}
			}
            else {
                $eventTableString = '<tr>';
                $eventTableString .= '<th>No events has been entered. Please navigate to Calendar view to enter events.</th>';
                $eventTableString .= '</tr>';
            }
		}
        echo $eventTableString;
        exit;
    }
	
	/**
     * Process the ajax post request and shows all existing events in the database.
     *
     * @return instance of view model(send data to view).
     */
	public function getAllEventAction()
    {
		$request = $this->getRequest();
		$eventArray = array();
		
		// check if the request method is a post.
        if ($request->isPost()) {
			$postData = $request->getPost();
			$results = $this->getEventsTable()->fetchAll($postData->container_id);
			
			// count whether any record exists or not.
			if (count($results) > 0) {
				$count = 0;
				foreach($results as $result) {
					$eventArray[$count]['id'] = (int)$result->eventid;
					$eventArray[$count]['title'] = $result->event_name;
					$eventArray[$count]['start'] = $result->start_date;
					$eventArray[$count]['end'] = $result->end_date;
					$eventArray[$count]['where'] = $result->where;
					$eventArray[$count]['description'] = $result->description;
					$eventArray[$count]['is_all_day'] = $result->is_all_day;
					if($result->is_all_day == 0)
						$eventArray[$count]['allDay'] = false;
					else
						$eventArray[$count]['allDay'] = true;
					if($result->color != '' ) {                      
						$eventArray[$count]['backgroundColor'] = $result->color;
						$eventArray[$count]['borderColor'] = $result->color;
					} else {
						$eventArray[$count]['backgroundColor'] = '#668CD9';
						$eventArray[$count]['borderColor'] = '#668CD9';
					}
					$count++;
				}
			}
			echo json_encode($eventArray);
		}
		exit;
	}
    
    /**
     * Ajax action to delete tasks.
     * 
     */
    function deleteTaskAction()
    {
        // get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            $postedData = $this->getRequest()->getPost();
            $taskId = $postedData['task_id'];
            
            if ($taskId) {
                $deleteStatus = $this->getTaskTable()->deleteTask($taskId);
                
                if ($deleteStatus) {
                    echo 'success';
                    exit;
                }
            }
        }
        echo 'failure';
        exit;
    }
	
	/**
     * Ajax action to delete documents.
     * 
     */
    function deleteDocumentAction()
    {
        // get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            $postedData = $this->getRequest()->getPost();
            $documentId = $postedData['document_id'];
            
            if ($documentId) {
                $deleteStatus = $this->getDocumentTable()->deleteDocument($documentId);
                
                if ($deleteStatus) {
                    echo 'success';
                    exit;
                }
            }
        }
        echo 'failure';
        exit;
    }
    
    /**
     * Delete this contact from the contact table.
     */
    function deleteContactAction()
    {
        // get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            $postedData = $this->getRequest()->getPost();
            $contactId = $postedData['contact_id'];
            
            if ($contactId) {
                $deleteStatus = $this->getContactTable()->deleteContact($contactId);
                
                if ($deleteStatus) {
                    echo 'success';
                    exit;
                }
            }
        }
        echo 'failure';
        exit; 
    }
    
    /**
     * Delete this medication from the contact table.
     */
    function deleteMedicationAction()
    {
       // get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            $postedData = $this->getRequest()->getPost();
            $medicationId = $postedData['medication_id'];
            
            if ($medicationId) {
                $deleteStatus = $this->getMedicationTable()->deleteMedication($medicationId);
                
                if ($deleteStatus) {
                    echo 'success';
                    exit;
                }
            }
        }
        echo 'failure';
        exit; 
    }
	
	/**
	 * Adding a function to save the posts added
	 * It will be redirected to the secure controller once the posts are added
	 */
	public function savePostAction()
    {
		//Checking whether user logged in or not
		$isLoggedIn = 'yes';
        
		// check if the user is already logged in.
        if (! $this->getServiceLocator()->get('AuthService')->hasIdentity()) {
			$isLoggedIn = 'no';
            return $this->redirect()->toRoute('login');
        }
		$this->getEvent()->getViewModel()->is_logged_in = $isLoggedIn;
        //Getting user id
		$session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
        
        $convert = $this->getServiceLocator()->get('ConvertParams');
		
		//Functionality to redirect it to the same profile page
		$resourceidString = $this->params()->fromRoute('resourceid', 0);
        
        $decodedPersonId = $convert->paramDecrypt($resourceidString);
        $inputArray = $convert->stringToArray($decodedPersonId);
		
		//get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            //collect the posted data
			$postedData = $this->getRequest()->getPost();
			
			//post data is the data of post form
			$postData = array();
			
			//Functionality for Validating the comment
			$postData['comment'] = $this->filterFormElements($postedData['post_new_comment']);
			
			if (trim($postData['comment']) != '') {
		    
                $this->getPostTable()->savePostData($postData, $userId, $inputArray['resourceid']);
            }
		}
		
		//Returning to the secure controller
		return $this->redirect()->toRoute('secure', array('resourceid' => $resourceidString));
	}
	
	/**
     * Ajax action to delete posts.
     * @param int postid
     */
    function deletePostAction()
    {
        // get an instance of register form
		$request = $this->getRequest();
		
		// check if the request is a post request
        if ($request->isPost()) {
            
            $postedData = $this->getRequest()->getPost();
            $postId = $postedData['post_id'];
            
            if ($postId) {
                $deleteStatus = $this->getPostTable()->deletePost($postId);
                
                if ($deleteStatus) {
                    echo 'success';
                    exit;
                }
            }
        }
        echo 'failure';
        exit;
    }
	
	/**
     * Apply strip tags to the provided input
     *
     * @return filtered value else return false.
     */
	public function filterFormElements($valueToFilter = '')
	{
		// instantiate the strip tag filter
		$stripTagFilter   = new StripTags();
		$StringTrimFilter = new StringTrim();
		if ($valueToFilter) {
			return $stripTagFilter->filter($StringTrimFilter->filter($valueToFilter));
		}
		return false;
	}
	
	/**
     * Process the ajax post request and delete event data 
	 *
     * @param  (int)$eventId
     * @return .
     */
	public function deleteEventAction()
    {
		$request = $this->getRequest();
        if ($request->isPost()) {
            $postData = $request->getPost();
			
			$id = (int)$postData->id;
			echo $this->getEventsTable()->deleteEvent($id); 
		}
		exit;
	}
    
	/**
     * Process the ajax post request and creates the required resource
	 *
     * @param  (int)$eventId
     * @return .
     */
	public function addResourceAction()
    {
        $session = $this->intializeSessionInstance();
		$userId = $session->offsetGet('user_id');
		$request = $this->getRequest();
        if ($request->isPost()) {
            $postData = $request->getPost();
			
            $insertStatus = $this
                                ->getResourceTable()
                                ->createResource($postData->resource_id, $postData->resource_type, $postData->resource_name, $userId);
            if ($insertStatus) {
                echo 'success';
            }
            else {
                echo 'failure';
            }
		}
		exit;
	}
    
    /**
     * Get the return encoded string for redirection.
     * 
     * @param int $typeRedId
     * @param string $resourceType
     * @param string $parentResourceType
     * @return type string on success and false on failure
     * 
     */
    public function getReturnEncodedString($typeRedId, $resourceType, $parentResourceType)
    {
        
        $convert = $this->getServiceLocator()->get('ConvertParams');
        $resourceRecord = $this
                            ->getResourceTable()
                            ->getParentResourceIdByType($typeRedId, $resourceType);
                
                
        $tempArray = array(
            'type'          =>  $parentResourceType,
            'resourceid'    =>  $resourceRecord['fk_resourceid_parent']
        );

        $tempString = $convert->arrayToString($tempArray);
        return $convert->paramEncrypt($tempString);
    }
    
    /**
     * Get the return encoded string for containers.
     * 
     * @param int $typeRedId
     * @param string $resourceType
     * @param string $parentResourceType
     * @return type string on success and false on failure
     * 
     */
    public function getReturnEncodedStringForContainer($resourceId)
    {
        if ($resourceId) {
            $parentResourceRecord = $this
                                        ->getResourceTable()
                                        ->getParentResourceRecord($resourceId);
            
            $convert = $this->getServiceLocator()->get('ConvertParams');

            $tempArray = array(
                'type'          =>  $parentResourceRecord['type'],
                'resourceid'    =>  $parentResourceRecord['resourceid']
            );
            

            $tempString = $convert->arrayToString($tempArray);
            return $convert->paramEncrypt($tempString);
        }
        return false;
    }
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getBetaFeedbackInfoTable()
	{
		// check if usersTable previously defined.
		if (!$this->betaFeedbackInfoTable) {
			$sm = $this->getServiceLocator();
			$this->betaFeedbackInfoTable = $sm->get('Secure\Model\BetaFeedbackInfoTable');
		}
		return $this->betaFeedbackInfoTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getPersonTable()
	{
		// check if usersTable previously defined.
		if (!$this->personTable) {
			$sm = $this->getServiceLocator();
			$this->personTable = $sm->get('Secure\Model\PersonTable');
		}
		return $this->personTable;
	}
	
	/**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getProviderTable()
	{
		// check if usersTable previously defined.
		if (!$this->providerTable) {
			$sm = $this->getServiceLocator();
			$this->providerTable = $sm->get('Secure\Model\ProviderTable');
		}
		return $this->providerTable;
	}
	
	/**
     * Get an instance of resourceTable by using service manager.
     *
     * @return service manager instance for resource table.
     */
	public function getResourceTable()
	{
		// check if usersTable previously defined.
		if (!$this->resourceTable) {
			$sm = $this->getServiceLocator();
			$this->resourceTable = $sm->get('Secure\Model\ResourceTable');
		}
		return $this->resourceTable;
	}
    
    /**
     * Get an instance of calendarTable by using service manager.
     *
     * @return service manager instance for calendar table.
     */
	public function getCalendarTable()
	{
		// check if usersTable previously defined.
		if (!$this->calendarTable) {
			$sm = $this->getServiceLocator();
			$this->calendarTable = $sm->get('Secure\Model\CalendarTable');
		}
		return $this->calendarTable;
	}
    
    /**
     * Get an instance of medicationsTable by using service manager.
     *
     * @return service manager instance for medications table.
     */
	public function getMedicationsTable()
	{
		// check if usersTable previously defined.
		if (!$this->medicationsTable) {
			$sm = $this->getServiceLocator();
			$this->medicationsTable = $sm->get('Secure\Model\MedicationsTable');
		}
		return $this->medicationsTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for medication table.
     */
	public function getMedicationTable()
	{
		// check if usersTable previously defined.
		if (!$this->medicationTable) {
			$sm = $this->getServiceLocator();
			$this->medicationTable = $sm->get('Secure\Model\MedicationTable');
		}
		return $this->medicationTable;
	}
    
    /**
     * Get service locater instance of getTaskTable
     * 
     * @return service manager instance for getTasksTable
     */
	public function getTasksTable()
	{
		// check if usersTable previously defined.
		if (!$this->tasksTable) {
			$sm = $this->getServiceLocator();
			$this->tasksTable = $sm->get('Secure\Model\TasksTable');
		}
		return $this->tasksTable;
	}
    
    /**
     * Get service locater instance of getTaskTable
     * 
     * @return service manager instance for getTaskTable
     */
	public function getTaskTable()
	{
		// check if usersTable previously defined.
		if (!$this->taskTable) {
			$sm = $this->getServiceLocator();
			$this->taskTable = $sm->get('Secure\Model\TaskTable');
		}
		return $this->taskTable;
	}
    
    /**
     * Get service locater instance of getContactsTable
     * 
     * @return service manager instance for getContactsTable
     */
	public function getContactsTable()
	{
		// check if usersTable previously defined.
		if (!$this->contactsTable) {
			$sm = $this->getServiceLocator();
			$this->contactsTable = $sm->get('Secure\Model\ContactsTable');
		}
		return $this->contactsTable;
	}
    
    /**
     * Get service locater instance of getContactTable
     * 
     * @return service manager instance for getContactTable
     */
	public function getContactTable()
	{
		// check if usersTable previously defined.
		if (!$this->contactTable) {
			$sm = $this->getServiceLocator();
			$this->contactTable = $sm->get('Secure\Model\ContactTable');
		}
		return $this->contactTable;
	}
    
    /**
     * Get service locater instance of getUserResourceTable
     * 
     * @return service manager instance for getUserResourceTable
     */
	public function getUserResourceTable()
	{
		// check if usersTable previously defined.
		if (!$this->userResourceTable) {
			$sm = $this->getServiceLocator();
			$this->userResourceTable = $sm->get('Secure\Model\UserResourceTable');
		}
		return $this->userResourceTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersPermTempTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersPermTempTable) {
			$sm = $this->getServiceLocator();
			$this->usersPermTempTable = $sm->get('Secure\Model\UsersPermTempTable');
		}
		return $this->usersPermTempTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (!$this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
	
	/**
     * Get an instance of documentsTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getDocumentsTable()
	{
		// check if usersTable previously defined.
		if (!$this->documentsTable) {
			$sm = $this->getServiceLocator();
			$this->documentsTable = $sm->get('Secure\Model\DocumentsTable');
		}
		return $this->documentsTable;
	}
	
	/**
     * Get an instance of documentTable by using service manager.
     *
     * @return service manager instance for document table.
     */
	public function getDocumentTable()
	{
		// check if usersTable previously defined.
		if (!$this->documentTable) {
			$sm = $this->getServiceLocator();
			$this->documentTable = $sm->get('Secure\Model\DocumentTable');
		}
		return $this->documentTable;
	}
    
	/**
     * Get an instance of resource_type table by using service manager.
     *
     * @return service manager instance for resource_type table.
     */
	public function getResourceTypeTable()
	{
		// check if usersTable previously defined.
		if (!$this->resourceTypeTable) {
			$sm = $this->getServiceLocator();
			$this->resourceTypeTable = $sm->get('Secure\Model\ResourceTypeTable');
		}
		return $this->resourceTypeTable;
	}
	
	/**
     * Get an instance of postTable by using service manager.
     *
     * @return service manager instance for post table.
     */
	public function getPostTable()
	{
		// check if usersTable previously defined.
		if (!$this->postTable) {
			$sm = $this->getServiceLocator();
			$this->postTable = $sm->get('Secure\Model\PostTable');
		}
		return $this->postTable;
	}
}