<?php

//module/Secure/src/Secure/Model/ContactTable.php

/**************
	Purpose: Model functions for accessing the Contact table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert;
use Zend\Db\Adapter\Adapter;

class ContactTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	private $sessionInstance;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
	
	/**
     * @var form
     */
	protected $eventsTable;
	protected $dbSql;
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance() {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Model function to get all contacts present under the resource id of 
     * given contact group
     * 
     * @param int $resourceid
     * @return boolean required record on success and false on failure.
     * 
     */
    public function getAllContacts($resourceid)
    {
        if ($resourceid) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT *
                    FROM resource AS t1
                    LEFT JOIN contact AS t2
                    ON ( t1.fk_type_recid = t2.contactid)
                    WHERE t1.fk_resourceid_parent = $resourceid  
                    AND t1.type='contact' AND t1.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
    /**
     * Model function to insert a contact record in to the contacts table.
     * 
     * @param array $insertMedicationData
     * @param int $user_id
     * @return boolean true if contact record successfully save false on failure.
     * 
     */
    public function saveContact($insertMedicationData, $user_id)
    {
        $medicationData = array(
            'contact_name'          => $insertMedicationData['contact_name'],
            'business_name'         => $insertMedicationData['business_name'],
            'contact_category'      => $insertMedicationData['contact_category'],
            'email_address1'        => $insertMedicationData['email_address1'],
            'mobile_number'         => $insertMedicationData['mobile_number'],
            'home_number'           => $insertMedicationData['home_number'],
            'work_number'           => $insertMedicationData['work_number'],
            'work_address'          => $insertMedicationData['work_address'],
            'work_city'             => $insertMedicationData['work_city'],
            'work_state'            => $insertMedicationData['work_state'],
            'work_zip'              => $insertMedicationData['work_zip'],
            'home_address'          => $insertMedicationData['home_address'],
            'home_city'             => $insertMedicationData['home_city'],
            'home_state'            => $insertMedicationData['home_state'],
            'home_zip'              => $insertMedicationData['home_zip'],
            'home_page'             => $insertMedicationData['home_page'],
            'contact_notes'         => $insertMedicationData['contact_notes'],
            'created'               => date('Y-m-d H:i:s'),
            'userid_created'        => $user_id
        );
		try {
            
			$this->tableGateway->insert($medicationData);
			$lastInsertIdPerson = $this->tableGateway->lastInsertValue;
			$resourceData = array(
				'type'					=>	'contact',
				'fk_type_recid'			=> 	$lastInsertIdPerson,
                'description'           =>  $insertMedicationData['contact_name'],
				'fk_userid_owner'  		=> 	$user_id,
				'fk_resourceid_parent' 	=>  $insertMedicationData['parent_resource_id'],
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $user_id,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->insert($resourceData);
			return true;
		}
		catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * Updates a contact record for the given
     * 
     * @param array $updateContactData
     * @param int $userid
     * @return boolean true on success and false on failure.
     * 
     */
    public function updateContact($updateContactData, $userid)
    {
        $contactData = array(
            'contact_name'          => $updateContactData['contact_name'],
            'business_name'         => $updateContactData['business_name'],
            'contact_category'      => $updateContactData['contact_category'],
            'email_address1'        => $updateContactData['email_address1'],
            'mobile_number'         => $updateContactData['mobile_number'],
            'home_number'           => $updateContactData['home_number'],
            'work_number'           => $updateContactData['work_number'],
            'work_city'             => $updateContactData['work_city'],
            'work_address'          => $updateContactData['work_address'],
            'work_state'            => $updateContactData['work_state'],
            'work_zip'              => $updateContactData['work_zip'],
            'home_city'             => $updateContactData['home_city'],
            'home_address'          => $updateContactData['home_address'],
            'home_state'            => $updateContactData['home_state'],
            'home_zip'              => $updateContactData['home_zip'],
            'home_page'             => $updateContactData['home_page'],
            'contact_notes'         => $updateContactData['contact_notes'],
            'userid_updated'        => $userid
        );
        
        $updateResource = array(
			'description'           => $updateContactData['contact_name'],
            'updated'  				=>  date('Y-m-d H:i:s')
		);
        
		try {
			$this->tableGateway->update($contactData, array('contactid' => $updateContactData['contactid']));
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $updateContactData['contactid'],
                            'type'          => 'contact'
                        )
            );
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * Get contact record by id.
     * 
     * @param int $contactId
     * @return boolean contact record on success and false on failure.
     * 
     */
    function getContactRecordById($contactId) {
        $rowset = $this->tableGateway->select(array('contactid' => $contactId));
        $row = $rowset->current();
        if ($row) {
           return $row;
        }
        return false;
    }
    
     /**
     * Delete user permission.
     * 
     * @param type $userResourceId
     * @return boolean tru on success and false on failure
     */
    public function deleteContact($contactId) {
        
        if ($contactId) {
             if ($this->tableGateway->delete(array('contactid' => $contactId))) {
                 $resourceInstance = new TableGateway('resource', $this->dbSql);
                 if ($resourceInstance->delete(array('fk_type_recid' => $contactId, 'type'=>'contact'))) {
                     return true;
                 }
             }
        }
        return false;
    }
}