<?php

//module/Secure/src/Secure/Model/DocumentTable.php

/**************
	Purpose: Model functions for accessing the Document table
	Author: Kevin Gyberg
	Date: 10/06/2013
*************/

namespace Secure\Model;

use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert;
use Zend\Db\Adapter\Adapter;
//Functionality to access the service locator in side the model
use Zend\ServiceManager\ServiceLocatorAwareInterface;
use Zend\ServiceManager\ServiceLocatorInterface;

class DocumentTable implements ServiceLocatorAwareInterface {
	/**
	 * To access service locator
	 */
	protected $serviceLocator;

	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	private $sessionInstance;
	
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
    
	/**
     * @var form
     */
	protected $eventsTable;
	protected $dbSql;
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
	 * Function used to get the service locator
	 */
	public function setServiceLocator(ServiceLocatorInterface $serviceLocator) {
		$this->serviceLocator = $serviceLocator;
	}
	/**
	 * Function used to get the service locator
	 */
	public function getServiceLocator() {
	  return $this->serviceLocator;
	}
  
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
	
	/**
     * Model function to get all documents for the given document group.
     * @param  int $resourceid
     */
    public function getAllDocuments($resourceid)
    {
        if ($resourceid) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT *
                    FROM resource AS res
                    LEFT JOIN document AS doc
                    ON ( res.fk_type_recid = doc.documentid)
                    WHERE res.fk_resourceid_parent = $resourceid  
                    AND res.type='document' AND res.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
	
	/**
     * Model function to insert document information into the database.
     * @param  array $documentData
     * @param  int $userId
     */
    public function saveDocument($documentData, $userId)
    {
        $insertDocumentData = array(
            'document_description'  => $documentData['document_description'],
            'actual_file_name'      => $documentData['actual_file_name'],
			'file_name'             => $documentData['file_name'],
            'created'               => date('Y-m-d H:i:s'),
            'userid_created'        => $userId
        );
        
		try {
            
			$this->tableGateway->insert($insertDocumentData);
        
            $lastInsertIdDocument = $this->tableGateway->lastInsertValue;
            $resourceData = array(
                'type'					=>	'document',
                'description'           =>  $documentData['document_description'],
                'fk_type_recid'			=> 	$lastInsertIdDocument,
                'fk_userid_owner'  		=> 	$userId,
                'fk_resourceid_parent' 	=>  $documentData['parent_resource_id'],
                'created'  				=>  date('Y-m-d H:i:s'),
                'userid_created' 		=>  $userId,
                'updated'  				=>  date('Y-m-d H:i:s')
            );

            $resourceInstance = new TableGateway('resource', $this->dbSql);
            $resourceInstance->insert($resourceData);
            
            $lastInsertIdDocument = $resourceInstance->lastInsertValue;
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
    }
	
	/**
     * Delete a document.
     * 
     * @param int $documentId
     * @return boolean true on success and false on failure
     */
    public function deleteDocument($documentId) {
        
        if ($documentId) {
			$sql = $this->dbSql;
			$resultSet = $sql->query("SELECT file_name
                    FROM document WHERE documentid = $documentId",
                    Adapter::QUERY_MODE_EXECUTE);
			
			//Deleting file from server when deleting the document
			if($resultSet) {
				
				foreach($resultSet as $result) {
					$file_name = $result['file_name'];
				}
				
				$config = $this->getServiceLocator()->get('Config');
				$file_location = $config['document_upload_location'].$file_name;
				
				//delete the file if file exists
				if(file_exists($file_location)) {
					unlink($file_location);
				}
			}
			
            if ($this->tableGateway->delete(array('documentid' => $documentId))) {
				$resourceInstance = new TableGateway('resource', $this->dbSql);
				
				if ($resourceInstance->delete(array('fk_type_recid' => $documentId, 'type'=>'document'))) {
					return true;
				}
            }
        }
        return false;
    }
	
	/**
	 * Model function to edit the document information
	 * @param array $documentUpdateData document info
	 * @param int $userId 
	 * @return boolean True/False
	 */
	function updateDocument($documentUpdateData, $userId, $isEmpty) {
        
        $updateDocumentData = array();
		
        if ($isEmpty == 'yes') {
            $updateDocumentData = array(
                'document_description'  => $documentUpdateData['document_description'],
                'updated'               => date('Y-m-d H:i:s'),
                'userid_updated'        => $userId
            );
        }
        else if ($isEmpty == 'no') {
            $updateDocumentData = array(
                'document_description'  => $documentUpdateData['document_description'],
                'actual_file_name'      => $documentUpdateData['actual_file_name'],
                'updated'               => date('Y-m-d H:i:s'),
                'userid_updated'        => $userId
            );
        }
		
		//If new file entered then removing the previous file
		if(is_numeric($documentUpdateData['documentid']) && ($documentUpdateData['file_error'] === 0)) {
			$documentId = $documentUpdateData['documentid'];
			$sql = $this->dbSql;
			$resultSet = $sql->query("SELECT file_name
                    FROM document WHERE documentid = $documentId",
                    Adapter::QUERY_MODE_EXECUTE);
			
			//Deleting file from server when deleting the document
			if($resultSet) {
				
				foreach($resultSet as $result) {
					$file_name = $result['file_name'];
				}
				
				$config = $this->getServiceLocator()->get('Config');
				$file_location = $config['document_upload_location'].$file_name;
				
				//delete the file if file exists
				if(file_exists($file_location)) {
					unlink($file_location);
				}
			}
			//Putting new file name in the update array
			$updateDocumentData['file_name'] = $documentUpdateData['file_name'];
		}
		
		try {
            
			$this->tableGateway->update($updateDocumentData, array('documentid' => $documentUpdateData['documentid']));
            
			$updateResource = array(
				'description'   => $documentUpdateData['document_description'],
				'updated'  		=>  date('Y-m-d H:i:s')
			);
			
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $documentUpdateData['documentid'],
                            'type'          => 'document'
                        )
            );
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
	}
	
	/**
     * Model function to get the document record by ID.
     * @param int $documentId
     */
    function getDocumentRecordById($documentId) {
        
		if($documentId) {
			$rowset = $this->tableGateway->select(array('documentid' => $documentId));
			$row = $rowset->current();
			
			if ($row) {
			   return $row;
			}
		}
        
        return false;
    }
}