<?php

//module/Secure/src/Secure/Model/DocumentsTable.php

/**************
	Purpose: Model functions for accessing the document_group table
	Author: Kevin Gyberg
	Date: 10/06/2013
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Adapter\Adapter;

class DocumentsTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
	/**
     * @var dbSql
     */
    protected $dbSql;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
        $this->dbSql = $this->tableGateway->getAdapter();
    }
	
	/**
     * Model function to get the document name from id.
     * 
     * @param type $documentid
     * @return boolean (object) on success and false on failure.
     * 
     */
	public function getDocumentsNameFromId($documentid)
    {
		$rowset = $this->tableGateway->select(array('document_groupid' => $documentid));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
	
	/**
     * Update a documents record by id.
     * 
     * @param int $documentGroupId
     * @param string $documentGroupName
     * @return boolean true on success and false on failure.
     *
     */
	public function updateDocumentsRecordById($documentGroupId, $documentGroupName)
    {
        $updataUsers = array(
            'document_group_name' =>  $documentGroupName
        );
        
        $updateResource = array(
			'description'   => $documentGroupName,
            'updated'       => date('Y-m-d H:i:s')
		);      
        
        try {
            $this->tableGateway->update($updataUsers, array('document_groupid' => $documentGroupId));
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $documentGroupId,
                            'type'          => 'document_group'
                        )
            );
            return true;
        } catch (Exception $exc) {
            return false;
        }
        // system has undergone an unknown exception.
    }
}