<?php

//module/Secure/src/Secure/Model/MedicationsTable.php

/**************
	Purpose: Model functions for accessing the medications table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Adapter\Adapter;

class MedicationsTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
    
	/**
     * @var dbSql
     */
    protected $dbSql;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
        $this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Model function to get the medication name from id.
     * 
     * @param type $medicationid
     * @return boolean (object) on success and false on failure.
     * 
     */
	public function getMedicationsNameFromId($medicationid)
    {
		$rowset = $this->tableGateway->select(array('medication_groupid' => $medicationid));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Update a medication record by id.
     * 
     * @param int $contactid
     * @param string $task_name
     * @return boolean true on success and false on failure.
     * 
     */
	public function updateMedicationsRecordById($medicationGroupId, $medicationGroupName)
    {
        $updataUsers = array(
            'medication_group_name' =>  $medicationGroupName
        );
        
        $updateResource = array(
			'description'   => $medicationGroupName,
            'updated'       =>  date('Y-m-d H:i:s')
		);      
        
        try {
            $this->tableGateway->update($updataUsers, array('medication_groupid' => $medicationGroupId));
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $medicationGroupId,
                            'type'          => 'medication_group'
                        )
            );
            return true;
        } catch (Exception $exc) {
            return false;
        }
        // system has undergone an unknown exception.
    }
}