<?php

//module/Secure/src/Secure/Model/PostTable.php

/**************
	Purpose: Model functions for accessing the Post table
	Author: Kevin Gyberg
	Date: 06/14/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert;
use Zend\Db\Adapter\Adapter;

class PostTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	private $sessionInstance;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
	
	/**
     * @var form
     */
	protected $eventsTable;
	protected $dbSql;
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
	
	/**
     * Model function to insert user posted comment information into the database.
     * @param  array $postdata
     * @param  int $userId
     * @return boolean 
     */
    public function savePost($postData, $userId)
    {
        $postDataArray = array(
            'post'              => $postData['post_new_comment'],
            'created'           => date('Y-m-d H:i:s'),
            'userid_created'    => $userId
        );
        
		try {
			$this->tableGateway->insert($postDataArray);
            
            // insert data into the resource table as post resource
            $lastInsertIdPost = $this->tableGateway->lastInsertValue;
			$resourceData = array(
				'type'					=>	'post',
                'description'           =>  $postData['post_new_comment'],
				'fk_type_recid'			=> 	$lastInsertIdPost,
				'fk_userid_owner'  		=> 	$userId,
				'fk_resourceid_parent' 	=>  $postData['parent_resource_id'],
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $userId,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->insert($resourceData);
            
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
    }
	
	/**
	 * Functionality to retrive all posts
	 * It will return the post along with the user name
	 * @return array
	 */
	public function getAllPost($parentResourceId)
    {
		$sql = $this->dbSql;
		
		$resultSet = $sql->query("SELECT * FROM resource AS t1
                LEFT JOIN post AS t2 on t1.fk_type_recid = t2.postid
                WHERE t1.type = 'post' AND t1.fk_resourceid_parent = $parentResourceId 
                    AND t2.rec_status != -1 ORDER BY t2.created DESC",
				Adapter::QUERY_MODE_EXECUTE);
		
		if ($resultSet) {
			return $resultSet;
		}
        
        return false;
	}
	
	/**
     * Delete post
     * 
     * @param int $postId
     * @return boolean true on success and false on failure
     */
    public function deletePost($postId) {
        
        if ($postId) {
             if ($this->tableGateway->delete(array('postid' => $postId))) {
                 $resourceInstance = new TableGateway('resource', $this->dbSql);
                 if ($resourceInstance->delete(array('fk_type_recid' => $postId, 'type'=>'post'))) {
                     return true;
                 }
             }
        }
        return false;
    }
}