<?php
//module/Secure/src/Secure/Model/Users.php

/**************
	Purpose: Set input filters and validations for users login form.
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;

use Zend\InputFilter\Factory as InputFactory;     // <-- Add this import
use Zend\InputFilter\InputFilter;                 // <-- Add this import
use Zend\InputFilter\InputFilterAwareInterface;   // <-- Add this import
use Zend\InputFilter\InputFilterInterface;        // <-- Add this import
use Zend\Validator\EmailAddress;

class Users implements InputFilterAwareInterface
{
	/**
     * @var id
     */
    public $userid;
	
	/**
     * @var user_name
     */
    public $first_name;
	
	/**
     * @var user_name
     */
    public $last_name;
	
	/**
     * @var pass_word
     */
    public $password;
	
	/**
     * @var email_address
     */
    public $email_login;
	
	/**
     * @var terms_conditions_agree
     */
    public $terms_conditions_agree;
	
	/**
     * @var set_up_cared_for_profile
     */
    public $set_up_cared_for_profile;
	
	/**
     * @var set_up_provider_profile
     */
    public $set_up_provider_profile;
	
	/**
     * @var no_profile_setup
     */
    public $no_profile_setup;

	/**
     * Preparing the posted data to store in database.
     *
     * @param  $data(array) with arbitrary key values.
     */
    public function exchangeArray($data)
    {
        $this->first_name = (isset($data['first_name'])) ? $data['first_name'] : null;
        $this->last_name = (isset($data['last_name'])) ? $data['last_name'] : null;
        $this->password = (isset($data['password'])) ? $data['password'] : null;
        $this->email_login  = (isset($data['email_login'])) ? $data['email_login'] : null;
        $this->terms_conditions_agree  = (isset($data['terms_conditions_agree'])) ? 1 : 0;
    }
	
	/**
     * Sets input filter.
     *
     * @param  InputFilterInterface $inputFilter
     * @throws \Exception
     */
    public function setInputFilter(InputFilterInterface $inputFilter)
    {
        throw new \Exception("Not used");
    }

	/**
     * Retrieve input filter. Associate validation to the input fields in register form.
	 * @return \Zend\InputFilter\InputFilterInterface
     */
    public function getInputFilter()
    {
		// check if inputFilter field is initialized
        if (!$this->inputFilter) {
            $inputFilter = new InputFilter();
            $factory     = new InputFactory();
			
			// add filter and validators to username field
            $inputFilter->add($factory->createInput(array(
                'name'     => 'first_name',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY => 'Please enter User name.' 
                            ),
                        ),
                    ),
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 0,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' => 'The entered user name must be less than 100 characters!' 
                            ),
                        ),
                    ),
                ),
            )));
			
			// add filter and validators to username field
            $inputFilter->add($factory->createInput(array(
                'name'     => 'last_name',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY => 'Please enter User name.' 
                            ),
                        ),
                    ),
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 0,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' => 'The entered user name must be less than 100 characters!' 
                            ),
                        ),
                    ),
                ),
            )));

			// add filter and validators to password field
            $inputFilter->add($factory->createInput(array(
                'name'     => 'password',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY => 'Please enter Password.' 
                            ),
                        ),
                    ),
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 0,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' => 'The entered Password must be less than 100 characters!'
                            ),
                        ),
                    ),
                ),
            )));
			
			// add filter and validators to confirm password field
			$inputFilter->add($factory->createInput(array(
                'name'     => 'conf_password',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY => 'Please enter Confirm password.' 
                            ),
                        ),
                    ),
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 0,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' => 'The entered Confirm password must be less than 100 characters!' 
                            ),
                        ),
                    ),
					array(
                        'name' => 'Identical',                        
                        'options' => array(
                            'token' => 'password',
                            'messages' => array(
                                \Zend\Validator\Identical::NOT_SAME => 'Re-entered password does not match!'                             ),
                        ),
                    )
                ),
            )));
			
			// add filter and validators to email address field.
			$inputFilter->add($factory->createInput(array(
                'name'     => 'email_login',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
					array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY => 'Please enter E-mail address.' 
                            ),
                        ),
                    ),
					array(
                      'name' =>'EmailAddress',
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\EmailAddress::INVALID => 'Please enter E-mail address on format abc@abc.com.' 
                            ),
                        ),
                    ),
                ),
            )));
			
			// add filter and validators to email address field.
			$inputFilter->add($factory->createInput(array(
                'name'     => 'terms_conditions_agree',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
					array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY => 'Please agree to the terms and conditions.' 
                            ),
                        ),
                    )
                ),
            )));

            $this->inputFilter = $inputFilter;
        }

        return $this->inputFilter;
    }
}