<?php

//module/Secure/src/Secure/Model/CalendarTable.php

/**************
	Purpose: Model functions for accessing the calendar table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where, Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Adapter\Adapter;

class UsersPermTempTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
    /**
     * @var dbSql
     */
	protected $dbSql;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
	
    /**
     * Function to get the calendarid from the input resource id.
     * 
     * @param int $resourceid
     * @return boolean on success return the required record and false on failure.
     * 
     */
	public function createTempUser($userData)
    {
		if ($this->tableGateway->insert($userData)) {
            return true;
        }
        return false;
    }
    
    /**
     * Function to update the userPermTemp table.
     * 
     * @param array $userData
     * @return boolean TRUE on success and false on failure.
     */
    public function updateTempUserPerm($userData) {
        
        if ($userData) {
            
            $permissionData = array(
                'perm_type'         => $userData['perm_type'],
                'userid_updated'    => $userData['userid_updated']
            );
            
            if ($this->tableGateway->update(
                                        $permissionData,
                                        array(
                                            'fk_email_login'    => $userData['fk_email_login'],
                                            'fk_resourceid'     => $userData['fk_resourceid']
                                        )
                    )) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Function to delete temporary permission in the user_perm_temp table.
     * 
     * @param array $userPermissionInsertData
     * @return boolean
     */
    public function deleteTempPerm($userPermissionInsertData) {
        
        if ($userPermissionInsertData) {
            
            if ($this->tableGateway->delete(
                                        array(
                                            'fk_email_login'    => $userPermissionInsertData['fk_email_login'],
                                            'fk_resourceid'     => $userPermissionInsertData['fk_resourceid']
                                        )
                    )) {
                return true;
            }
        }
        return false;
    }

    /**
     * Check if the permission exist in the user_permission_temp table.
     * 
     * @param type $emailAddress
     * @return boolean
     */
    public function checkIfPermissionExist($emailAddress, $resourceId) {
        $rowset = $this->tableGateway->select(
                                            array(
                                                'fk_email_login'     => $emailAddress,
                                                'fk_resourceid'      => $resourceId,
                                                'rec_status'         => 1
                                            )
        );
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Function to copypermissions from the user_perm_temp and insert them into 
     * user_resource table as needed. $userId( of the last users record created).
     * 
     * @param   string $emailId
     * @param   int $userId
     * @return  true on success and false an failure.
     */
    public function copyPermissions($emailId, $userId) {
        
        $records = $this->tableGateway->select(
                                array(
                                    'fk_email_login'   => $emailId,
                                    'rec_status'       => 1
                                )
        );
        
        if (count($records) > 0) {
            
            $userResourceInstance = new TableGateway('user_resource', $this->dbSql);
            foreach ($records as $record) {

                // insert all permission into the user_resource table.
                $userResourceInsert = array(
                    'fk_userid'     => $userId,
                    'fk_resourceid' => $record['fk_resourceid'],
                    'perm_type'     => $record['perm_type'],
                    'created'  		=> date('Y-m-d H:i:s'),
                    'userid_created'=> $record['userid_created']
                );
                
                // copy and insert records into user_resource table.
                if (!$userResourceInstance->insert($userResourceInsert)) {
                    break;
                    return false;
                }
            }
        }
        
        // update all pending record 
        $set = array(
            'rec_status' => -1
        );
        
        $where = array(
            'fk_email_login'   => $emailId
        );
        
        try {
            $this->tableGateway->update($set, $where);
            return true;
        }
        catch (Exception $e) {
            return false;
        }
        
        // system has undergone an unknown exception.
    }
    
    /**
     * Get all pending shares form User permission temp table.
     * 
     * @param type $resourceId
     * @return boolean
     */
    public function getAllPendingShares($resourceId)
    {
        if ($resourceId) {
            $pendingShares = $this->tableGateway->select(
                                    array(
                                        'fk_resourceid' => $resourceId,
                                        'rec_status'    => 1
                                    )
            );
            
            if ($pendingShares) {
                return $pendingShares;
            }
        }
        
        return false;
    }
    
    /**
     * Function to delete a user permission temp record.
     * 
     * @param int $userResourceId
     * @return boolean false
     * 
     */
    public function deleteUserPermission($userResourceId) {
        
        if ($userResourceId) {
             if ($this->tableGateway->delete(array('users_permisson_tempid' => $userResourceId))) {
                 return true;
             }
        }
        return false;
    }
}